function [index_new, not_ok] = permute_var_bel(index_old,order,stored_as) 
% permuting the order of the variables in a belief object
% execute for all bp of this belief object
% 
% index_old = index of the belief which variables must be reordered
% index_new = index of new belief object 	if stored_as = 'new'
% 			= index_old 					if stored_as = 'old'
% order = the variable numbers in new order
% stored_as = 'new' as a new belief object
% 			= 'old' on input belief object itself

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global belief pin_belief

not_ok = 0;

var = belief(index_old).domain_variable;
cardi = belief(index_old).domain_cardinality;
card_total = prod(cardi);

% testing if sharing same domain variables
vardif = setxor(order,var);
if ~isempty(vardif)
	hdw = warndlg(['You ask to reorder the variables of a belief object,', 
	'and they don''t share the same sets. I cancel.']);
	uiwait(hdw);
	not_ok = 1;
else
	for i = 1:length(order) 
		pos(i) = find(ismember(var,order(i))); 
		% + 1 as nbr focal sets is first index
	end
	% if order = [3 1 5 2] and var = [5 1 2 3], then pos = [4 2 1 3]
	% for all k : order(k) = var(pos(k))
	
	% what to do with results	
	switch stored_as
	case 'new'
		pin_new = get_next_pin('belief');
		index_new = pin_new(2);
		belief(index_new) = belief(index_old);
		belief(index_new).number = pin_new(3);	
		useless = pin_manipulation('belief','all',[]);
	case 'old'
		index_new = index_old;		
	end			

	belief(index_new).domain_variable = order;
	belief(index_new).domain_cardinality = belief(index_old).domain_cardinality(pos);
	
	for ibp = 1: length(belief(index_old).bp)
		for ifr = 1:length(belief(index_old).bp(ibp).form)
			
			switch belief(index_old).bp(ibp).form(ifr).representation
			case 'selected'
				vect_bit = long_num2bit(belief(index_old).bp(ibp).form(ifr).focal_sets,...
				card_total);	
				nbr_foc = size(vect_bit,1);
				
				% the real permutation
				new_foc = long_bit2num(reshape(...
				permute(  reshape(vect_bit,[nbr_foc,cardi])  ,[1,pos+1]), ...
				[nbr_foc,card_total]),card_total);
				
				belief(index_new).bp(ibp).form(ifr).focal_sets = new_foc;	
			case 'ordered'
				vect_bit = long_num2bit([0:card_total-1]',card_total);
				nbr_foc = card_total;
				
				% forbidden if more than 50 elements 
				% it would be crazy to use oredred with more than 15-20 elements.
				% the real permutation
				new_foc = long_bit2num(reshape(...
				permute(  reshape(vect_bit,[nbr_foc,cardi])  ,[1,pos+1]), ...
				[nbr_foc,card_total]),card_total);
				
				belief(index_new).bp(ibp).form(ifr).values = ...
				belief(index_old).bp(ibp).form(ifr).values(new_foc+1); 
			end
		end
	end
end


