function [m_final,foc_final,list_number_var,list_fr_card ] = compute_distinct_combi(...
expr,list_bbf_unique,list_bbf_labels)
% conjunctive and disjunctive combination of bp's and their negation 
% where bp's are produced by distinct pieces of evidence

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input :
% expr = an expression that states which computation is requested. 

% expr is made of logical operators (and, or, not, () ) and of bp refrences
% bp are denoted as <n.x.y.z>
% where n = bel_number, x = bel_index, y = bp_index, z = form_index
% comming from 	- belief(bel_index).number = bel_number
% 				- belief(bel_index).bp(bp_index).form(form_index)

% expr = <12.7.1.1> and (<15.3.2.3> or <17.9.1.2>)  
% is equal to 
% expr = (<12.7.1.1> and <15.3.2.3>) or (<12.7.1.1> and <17.9.1.2>)

% allowed: diff bp in same bel number. ( <12.4.1.3> or <12.4.2.1> or <12.4.3.2> ) and <24.5.2.3>
% but not >1 or 0 bba in a bp

% list_bbf_unique    are the bp's really involved without repetitions
% listed as [n1 x1 y1 z1; n2 x2 y2 z2; ...]
% where 	belief(x1).number = n1
%			belief(x1).bp(y1).form(z1) is used

% list_bbf_labels are the labels B[   ] (see below)
% We create B(   ) variables for the eval programs.

% both list_bbf_unique and list_bbf_labels produced by translating_request

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% output :
% m_final 		- the bbm 
% foc_final 	- the corresponding focal sets as long numbers
% var_num_final	- the variable numbers involved in the final bp

% every bp is vacuously extedned on the common join space, 
% the cartesian product of the variables in var_num_final

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DANGER. The product pr of the number of masses in each bp must be reasonable.
% We have to store as many masses as pr. Memory overflow is possible.  
%
% DANGER. No more than 99 bp's 
% as the index for variable name in B() in the logical expr allows maximum 2 digits 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global nbr_bit_per_word
global length_one_focal % space needed for one B(:,:,jj)

% finding the variables of the bp's without repetition
list_number_var 	= unique([belief(list_bbf_unique(:,2)).domain_variable]);
% BEWARE: var are ordered by their numbers, smallest to largest, this is done by unique

list_index_var 		= pin_manipulation('variable','num2ind',list_number_var);
list_number_frame 	= [variable(list_index_var).frame];
list_index_frame 	= pin_manipulation('frame','num2ind',list_number_frame);
list_fr_card 		= [frame(list_index_frame).cardinality];

nbr_worlds = prod(list_fr_card);
% nbr_worlds = nbr of worlds in the common frame

[nbr_full_words, remain] =  words_for_set(nbr_worlds);
% bit representation parameters,nbr_full_words of nbr_bit_per_word bits + remain bits in next one

nbr_words = nbr_full_words + 1;
% nbr of words needed to store a long_number (a set)

universe = get_universe(nbr_worlds);
% the representation of the overal frame, used to put 0 on the unused bits

[nbr_m useless] = size(list_bbf_unique)	;
B = [];
lengthB = 1;
m = 1;
for iv = 1:nbr_m  % for every involved bp
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% computing the masses by products
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	bel_ind = list_bbf_unique(iv,2);
	bp_ind = list_bbf_unique(iv,3);
	form_ind = list_bbf_unique(iv,4);
	m1 = belief(bel_ind).bp(bp_ind).form(form_ind).values;
	m = kron(m1,m);
	% The equation is m123(X123) = sum {(i,j,k): expr(X1i, X2j, X3k) = X123} m1(X1i).m2(X2j).m3(X3k)
	
	% Note: kron([1 2]',[5 6 7]') = [5 6 7 10 12 14]'
	
	% order m =	m31	m21 m11
	%			m31 m21 m12
	%			m31 m21 m13
	%			m31 m22 m11
	%			...........
	%			m32 m21 m11
	% m1 goes faster than m2 that goes faster than m3...
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% K tells which are the index of the focal sets involved for the mass product
	% we don't use it, but for pedagogical purpose we leave it here in the comments
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% K = [kron(ones(length(m1),1),K) , kron([1:length(m1)]',ones(length(K),1)) ] ;
	
	% column j concerns Xj.
	%
	% K =		1 1 1   for	X11	X21	X31 	and m(1) allaocated to f(X11, X21, X31)
	%			2 1 1		X12	X21	X31		and m(2) allaocated to f(X12, X21, X31)
	% 			3 1 1		X13	X21	X31		and m(3) allaocated to f(X13, X21, X31)
	%			1 2 1		X11	X21	X31		and m(?) allaocated to f(X11, X22, X31)
	%			1 1 2		X11	X21	X32		and m(!) allaocated to f(X11, X21, X32)
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% B tells which are the index of the focal sets involved for the mass product
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% The equation is m123(X122) = sum {(i,j,k): expr(X1i, X2j, X3k) = X123} m1(X1i).m2(X2j).m3(X3k)
	
	% In previous example, I want 
	% K =	1 1 1   for	X11	X21	X31 	and B(1,1,:) = X11,	B(1,2,:) = X21, 	B(1,3,:) = X31
	%		2 1 1		X12	X21	X31		and B(2,1,:) = X12,	B(2,2,:) = X21, 	B(2,3,:) = X31
	% 		3 1 1		X13	X21	X31		and B(3,1,:) = X13,	B(3,2,:) = X21, 	B(3,3,:) = X31
	%		1 2 1		X11	X21	X31		and B(?,1,:) = X11,	B(?,2,:) = X22, 	B(?,3,:) = X31
	%		1 1 2		X11	X21	X32		and B(!,1,:) = X11,	B(!,2,:) = X21, 	B(!,3,:) = X32
	
	% B(i,j,:) = X j K(i,j)
	% I want f({B(:,j,:) : j=1:nbr_m } ) = f({ B(:,:,:) )  is OK, 
	
	% algorithm:
	% we build a vector m with all the products m1(X1i).m2(X2j).m3(X3k), with length(m) = n.
	% a matrix K with the corresponding (i,j,k). K is n x 3 (if 3 bp's involved).
	% a matrix B with X1i,X2j,X3k, thus B is n x 3 x nbr_words (for storing bits for a set)
	
	% B(n1,2,1:nbr_words) = X2j that entered in m(n1) for n1 = 1:n and j = K(n1,2)
	
	% Explaining the B matrix with a full example.	
	% The example: 
	%		m1 based on p1 dubious,   
	%		m2 based on maybe p2 or maybe p2,     
	%		m3 based on p1->p2 dubious
	
	% expr =  bp1 and bp2 and bp3    
	% remember logical are F T, thus {} = 0, F = 1, T = 2, F or T = 3
	
	% bp1 = [.2 .8]'		foc1 = [2 3]'	var p1
	% bp2 = [.3 .2 .5] 		foc2 = [2 1 3]'	var p2
	% bp3 = [.6 .4]			foc3 = [13 15]'	var = p1 x p2
	%
	% p2	0   0   1   1   col 1
	% p1	0   1   0   1   col 2
	% w		1   2   3   4   p1->p2 is w = [1 3 4] w = 13
	% this is what display order is, the first is at right, and turns the fastest
	% beware in bit stored in long number, TBMLAB put bit 1 at right, so decimal for w1 = 1, for w2 = 2, for w1+w2 = 3...
	
	% m'= 	.6   .6   .6   .6   .6   .6   .4   .4   .4   .4   .4   .4
	%		.3   .3   .2   .2   .5   .5   .3   .3   .2   .2   .5   .5     
	%		.2   .8   .2   .8   .2   .8   .2   .8   .2   .8   .2   .8 
	% m'= .036 .144 .024 .096 .060 .240 .024 .096 .016 .064 .040 .160	col of length 12
	
	% K :    1    1    1    1    1    1    2    2    2    2    2    2    col 3
	%		 1    1    2    2    3    3    1    1    2    2    3    3    col 2
	%		 1    2    1    2    1    2    1    2    1    2    1    2    col 1
	
	% the vacuous extension on p1xp2 of foc1, foc2 and foc3 are:
	% foc1 = [10 15]'
	% foc2 = [12 3 15]'
	% foc3 = [13 15]'  
	
	% B : B(r,s,t) with s always 1 here as one word enough to store bits for one set
	% r is 1 to 12, corresponds to the m
	% t is 1 to 3, corresponds to the 3 bp's onvolved
	
	% then evall(...     
	% bitor(  bitand(B(:,:,1),B(:,:,2))   ,  ...
	%         bitand(B(:,:,1),:),B(:,:,3)   )     ) 
	
	% for m(6) = .240, I need   	bitor(bitand(15,15), bitand(15,13))  
	% which is 		bitor(bitand(B(1,2,1),B(2,3,1), bitand(1B(1,2,1),B(3,1,1))) )
	% thus 			bitor(bitand(B(1,K(6,1),:),B(2,K(6,2),:), bitand(1B(1,K(6,1),:),B(3,K(6,3),:))) ) 
	
	% B(1:n, 1, 1:3 )'  = 
	%  13  13  13  13  13  13  15  15  15  15  15  15  last index = 3
	%  12  12   3   3  15  15  12  12   3   3  15  15  last index = 2  
	%  10  15  10  15  10  15  10  15  10  15  10  15  last index = 1
	% last index is the bp
	
	% end of example.
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% vacuous extension of every belief function on common frame
	% building the new focal sets on the common frame
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% handling the selected/ordered cases for the focal sets
	if strcmp(belief(list_bbf_unique(iv,2)).bp(list_bbf_unique(iv,3)).form(list_bbf_unique(iv,4)).representation,...
		'selected')
		foc_in = belief(list_bbf_unique(iv,2)).bp(list_bbf_unique(iv,3)).form(list_bbf_unique(iv,4)).focal_sets;
				
	else % bbm given on ordered sets. I build the focal sets
		cardinal = prod([belief(list_bbf_unique(iv,2)).domain_cardinality]);
		foc_in = [0:pow2(cardinal)-1]';
		% BEWARE: when ordered, cardinal always lower than nbr_bit_per_word, so ONE word. 
	end

	% used_var 	- the variable index of unextended bp, same order as belief().domain_variable
	% new_var	- the variable index of the new variables, whatever order
	% result on variable with order as in list_index_var 
	% {list_index_var} = union(used_var, new_var) same sets
	
	list_new = setdiff(list_index_var,belief(list_bbf_unique(iv,2)).domain_variable);
	
	% the focal sets extended
	focal_prod_sp(iv).foc_out =  vacuous_ext_long_focal  (foc_in, ...
	belief(list_bbf_unique(iv,2)).domain_variable, list_new, list_index_var, list_fr_card);
		
	% focal_prod_sp(iv).foc_out(ii,:) = list of the ii focal sets for bp(iv)
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% building the matrix B with all the focal sets
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% example   n_words = 2, length(m) = 2 3 4
	% foc1 = [a b ; c d]   foc2 = [ e f ; g h ; i j]   foc3 = [k l ; m n ; o p ; q r]
	% iv = 1: B = foc1
	% iv = 2: B = [ [foc1 ; foc1 ; foc1] [ e f ; e f ; g h ; g h ; i j ; i j ] ]
	% iv = 3: B = [ [B ; B ; B ; B]  [ k l ; k l ; k l ; k l ; k l ; k l ; m n ... 6 times; o p ... 6 times, q r ... 6 times]]
	
	D =  reshape(repmat(focal_prod_sp(iv).foc_out',lengthB,1),nbr_words,lengthB*length(m1))';
	B = [ repmat(B,length(m1),1)  D] ;
	[lengthB useless] = size(B);
	% B = previous matrix of focal elements	
	
end

B = reshape(B, lengthB, nbr_words, nbr_m); % B is a   nbr_foc_total x nbr_words x nbr_m   matrix

% B : B(r,s,t) 
% r is 1 to length(m), 	corresponds to the m(r) as obtained after combination (thus length = prod(length(m(iv))
% s is 1 to nbr_words , corresponds to one of the numbers that store one focal set as a long number (bits representation)
% t is 1 to nbr_m, 		corresponds to the nbr_m of bp's involved

foc_new = eval(expr); % THE REAL COMPUTATION FOR THE FOCAL ELEMENTS

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% adding masses given to same focal sets
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

% mass m(i) given to foc_new(i,:)
% there are duplicate in foc_new(i,:)

sorted_data = sortrows([foc_new m]);
[nr nc] = size(foc_new);

mm = sorted_data(:,nc+1);

indx = [0 find(sum(abs(diff(sorted_data(:,1:nc))),2))' nr];
% these are the position where focal sets change
% with sorted_data = 
%      1     3
%      1     3
%      1     3
%      2     1
%      2     4
%      2     4
% we get indx = [0 3 4 6], so we must add mm(1:3) mm(4:4) mm(5:6)

for i = 1:length(indx)-1
	m_final(i) = sum(mm(indx(i)+1:indx(i+1)));
	foc_final(i,1:nc) = sorted_data(indx(i)+1,1:nc);
end
m_final = m_final';

% % debug
% m_final(1)
% display_set(foc_final(1,1:nc) ,   prod(list_fr_card) )



