% Fast Moebius Transform.  Version FMT 2.0
%
% Package of FMT : 
% performs the transformation from any of   m b q pl w   to any of   m b q pl w betp
%

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

% Beware: functions are unnormalized (as in the TBM): so positive m({})is possible.
%
% symbols are m b q pl w with : (see papers on TBM for details)
% m 	the bba, 
% b 	the implicability fct, 
% q 	the communality fct, 
% pl 	the plausibility fct, 
% w 	the weight of evidence, canonical conjnuctive decomposition
%
% For the transformation from any of  m b q pl w  to any of   m b q pl w betp
% the name of each function is fromtowhat : e.g., mtob, mtow qtopl ...
% their input parameter is the first function (from), the output is the second function (what)
%
% vectors with the functions are line vectors, ordered 'numerically'
% i.e. {} a b a+b c a+c b+c a+b+c d a+d .....(+ means union)
%
% All these transform can be reached by a single call
% out = fmt(in, in_type, out_type)
% where 	in is the row vector containing the input function
% and 		out is the output row vector containing the resulting function
% input is in_type, 
% output is out_type
% where ..._type can be any of m,b,q,pl,w,bel,betp written as 'm', 'b', .... 
% intype = 'betp' is not allowed (it is useless)
%
% You get also 
%
%	normalize(x, type, nbr_atoms) that returns x after normalization
%
% 	createbbm(n)	that creates an output line vector containing a bbm 
%     	built randomly on a space with n elements  (length 2^n)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Computational notes:
%
% some of the transforms are basic:
% 		mtob btom mtoq qtom 
% These are the four computationaly 'heavy' tranfsformations.
% The other transforms are obtained from these 4 transformetions with a little adaptation.
%
% this table tells which algortihm is used to tansform xxx (line) to yyy (column)
% xy means algorithm to transform x to y
%
% 		m	b	q	pl	wc
%	m	-	mb	mq	mb
%	b	bm	-	bm	x
%	q	qm		-	bm	qm
%	pl	bm	x	bm	-	
%	wc  			mq		-

% the tricks are:
% b to q = b to pl to q
% q to b = q to pl to b
% q to pl = abs(b to m) with q({}) set = 0 by function
% pl to q = abs(b to m) with pl({}) set = 1 by function
% m to pl = m to b to pl
% pl to m = pl to b to m
% q to w = q to m applied on log q with exp at end and a trick
% w to q = m to q applied on log w with exp at end and a trick
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% SPEED:
% It works for a frame with up to 16 elements, above beware of CPU time and memory size
% as it stores a vector of 2^nbr_atoms length.
%

