function [foc_out] = vacuous_ext_long_focal(foc_in, list_in, list_new, list_out, list_fr_card_out)
% We vacuously extend a bp build on variables in list_in 
% on a larger space with variables in list_out

% foc_in = set of vectors with focal sets in long_number 
% foc_in(nbr_foc, nbr_words) where there need nbr_words to represent one set
% foc_out same representation
% but focal sets obtained by cylindrical extesion on a marger frame

% list_in  = list of variable index at input
% list_new = list of variable index for new var
% list_out = list of variable index at output (for the order)
% list_fr_card_out = cardinalities of the variables after extension (order as list_out)

% we ccan use numbers or indexes, it does not matter, but all ref must be of same type

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global nbr_bit_per_word

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% A comment on the bit representation of subsets

% when I say that the frame of discernemment for a belief potential is [2 5 8]
% I mean it is based on variables.number = 2 5 and 8.
% If their cardinalities are [2 3 4]
% in world 	1  2  3  4  5  6  7  8  9 10 ...24
% var 8	=	1  1  1  1  1  1  2  2  2  2 ... 4   displayed in col 1
% var 5	=	1  1  2  2  3  3  1  1  2  2 ... 3   displayed in col 2
% var 2	=	1  2  1  2  1  2  1  2  1  2 ... 2   displayed in col 3

% this is what   showing_bp_selected   produces, 
% the first variable in the list is at right (thus col 3), 
% and turns the fastest

% what we do is a refinement
% so for every world in in domain, I build a list of world in out domain
% example in =  4 7   and out = 1 4 6 7 9 each with its own cardinality
% Card in     = 2 3  Card out = 2 2 3 3 2 = 72 worlds
%
% So in_worlds are: 
% 7		1  1  2  2  3  3 
% 4		1  2  1  2  1  2 
% w     1  2  3  4  5  6
% pow2  1  2  4  8 16 32

% I compute a new space with N = 2 x 3 x 2 states  = 12 (labels = 1 2 .. 9 A B C)
% (the product of the cardinalities of the new variables)

% So out_worlds are:

% 7		1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 2 ... 3  
% 4		1 1 1 1 1 1 1 1 1 1 1 1	2 2 2 2 2 2 2 2 2 2 2 2 1 ... 2
% N		1 2 3 4 5 6 7 8 9 A B C 1 2 3 4 5 6 7 8 9 A B C 1 ... C
% w     1 2 3 4 5 6 7 8 910111213141516171819202122232425 ...72

% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% testing procedure
% nbr_bit_per_word = 50;

% permutation simple

% list_in = [7 4];
% list_new = [];
% list_out = [ 4 7];
% list_fr_card_out = [2 3];
% foc_in = 7;
% foc_in = [  18 ];
% % old space
% % 4		1  1  1  2  2  2
% % 7		1  2  3  1  2  3
% % w     1  2  3  4  5  6
% % new space
% % 7		1  1  2  2  3  3 
% % 4		1  2  1  2  1  2 
% % w     1  2  3  4  5  6
% % old 7  is old w 1 2 3 new 1 3 5 = 21  OK
% % old 18 is old w 2 5   new 3 4 = 12  OK


% list_in = [4 9 7];
% list_new = [];
% list_out = [9 7 4];
% list_fr_card_out = [2 3 2];
% foc_in = 1026;
% % old space (2 2 3)
% % 7		1  1  1  1  2  2  2  2  3  3  3  3
% % 9		1  1  2  2  1  1  2  2  1  1  2  2  
% % 4		1  2  1  2  1  2  1  2  1  2  1  2
% % w     1  2  3  4  5  6  7  8  9 10 11 12
% % new space
% % 4		1  1  1  1  1  1  2  2  2  2  2  2
% % 7		1  1  2  2  3  3  1  1  2  2  3  3
% % 9		1  2  1  2  1  2  1  2  1  2  1  2  
% % w     1  2  3  4  5  6  7  8  9 10 11 12
% % old =  1026  old  2 11       new 96 =   7 6  OK


% list_in = [4];
% list_new = [7 9];
% list_out = [9 7 4];
% list_fr_card_out = [2 3 2];
% foc_in = 2;
% % old space (2 2 3)
% % 4		1  2  
% % w     1  2  
% % new space
% % 4		1  1  1  1  1  1  2  2  2  2  2  2
% % 7		1  1  2  2  3  3  1  1  2  2  3  3
% % 9		1  2  1  2  1  2  1  2  1  2  1  2  
% % w     1  2  3  4  5  6  7  8  9 10 11 12
% % % old =  2  w2        new 4032 =   7:12   0000001111 11  OK


% list_in = [1 6 7 ];
% list_new = [4 9];
% list_out = [6 9 1 7 4];
% list_fr_card_out = [3 2 2 3 2];
% foc_in = long_onbit2num([2 5 6 9 15 17], 18)
% % old space (2 3 3)
% % 7		1  1  1  1  1  1  2  2  2  2  2  2  3  3  3  3  3  3
% % 6		1  1  2  2  3  3  1  1  2  2  3  3  1  1  2  2  3  3  
% % 1		1  2  1  2	1  2  1  2  1  2  1  2  1  2  1  2	1  2  
% % w     1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 
% % new space  card = 72
% % 4		1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1 
% % 7		1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  3  3  3  3  3  3  3  3  3  3  3  3       
% % 1		1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2     
% % 9		1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2 
% % 6		1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3
% % w     1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 
% 
% % 4		2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2
% % 7		1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  3  3  3  3  3  3  3  3  3  3  3  3       
% % 1		1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2     
% % 9		1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2  1  1  1  2  2  2 
% % 6		1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3
% % w    37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72
% 
% % only usefull lines for finding old ones
% % 7		1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  3  3  3  3  3  3  3  3  3  3  3  3       
% % 6		1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3
% % 1		1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2     
% % w     1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 
% 
% % 7		1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  3  3  3  3  3  3  3  3  3  3  3  3       
% % 6		1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3  1  2  3
% % 1		1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2  1  1  1  1  1  1  2  2  2  2  2  2     
% % w    37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72
% 
% % old = 2 5 6 9 15 17 = bit on
% % new =      3     6     7     9    10    12    14    17    26    27    29    30
% % biton		39    42    43    45    46    48    50    53    62    63    65    66
% % conversion 
% % 2 = 112  7 43 10 46
% % 5 = 131  3 39  6 42
% % 6 = 132  9 45 12 48
% % 9 = 221 14 50 17 53
% %15 = 321 26 62 29 65
% %17 = 331 27 63 30 66    OK

% 1 4 6 7 9
% 2 2 3 3 2

%    IT WORKS  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if length(list_new) == 0
	% the degenerate case where there is no new var, thus no real extension	
	% but maybe a reorganisation if order diff between in and out

	% cardinalities of input variables ordered as list_in
	for i = 1:length(list_in)
		fr_card_in(i)  = list_fr_card_out(find(list_out == list_in(i)));
	end

	card_in  = prod(fr_card_in);
	card_out = card_in;
	
	order = []; 
	pos =  [list_in]; 
	% with list_in = 4 7 
	% pos = [4 7]  The first var in a list is the one that changes the fasted
	% [4 7 ] is the order after performing the reshape below.
	% i want results to be on list_out = [7 4]	
	% so requested order = [2 1] 
	for i = 1:length(list_out)
		order = [order find(list_out(i) == pos)]; 
	end	

	nfoc = size(foc_in,1);
	
	a = long_num2bit(foc_in,card_in);
	d = reshape(a, [nfoc fr_card_in]);
	e = permute(d,[ 1 order+1 ] );
	f = reshape(e,nfoc,card_out);
	foc_out = long_bit2num(f,card_out);
		
else
	% cardinalities of input variables ordered as list_in
	for i = 1:length(list_in)
		fr_card_in(i)  = list_fr_card_out(find(list_out == list_in(i)));
	end

	% cardinalities of new variables ordered as list_new
	for i = 1:length(list_new)
		fr_card_new(i) = list_fr_card_out(find(list_out == list_new(i)));
	end

	card_in  = prod(fr_card_in);
	card_new = prod(fr_card_new);
	card_out = card_in * card_new;
	
	order = []; 
	pos =  [list_in list_new]; 
	% with list_in = 4 7 and list_new = 1 6 9
	% pos = [4 7   1 6 9]  The first var in a list is the one that changes the fasted
	% [4 7 1 6 9] is the order after performing the reshape below.
	% i want results to be on list_out = [1 4 6 7 9]	
	% so requested order = [3 1 4 2 5] 
	for i = 1:length(list_out)
		order = [order find(list_out(i) == pos)]; 
	end	

	nfoc = size(foc_in,1);
	
	a = long_num2bit(foc_in,card_in);
	c = repmat(a,1,card_new);
	d = reshape(c, [nfoc fr_card_in fr_card_new]);
	e = permute(d,[ 1 order+1 ] );
	f = reshape(e,nfoc,card_out);
	foc_out = long_bit2num(f,card_out);
	
end

% control area for ONE focal set only
% normaly not used but helps for debugging
% list_in
% list_new
% list_out
% list_fr_card_out
% 
% % % bitget(foc_in(1),1:card_in)
% % % for i = 1:length(foc_out)
% % % 	bitget(foc_out(i),1:50)
% % % end



