function [ind_new_obj] = make_empty(object,ind_obj,ind_bp,ind_form)
% build a new empty object or make an old object empty

% object can be any of frame, strucuture, mapping, attribute, variable, belief, bp, form

% if nargin = 1, we use next index
% if nargin = 2, ind_obj = index for frame, strucuture, mapping, attribute, variable, belief
% if nargin = 3 or 4, we provide the index for bp and form

% ind_bp   = index for belpot	case object = bp        can be a list
% the 2 others indexes are e number
% ind_new_obj = index of the created object, 
% = the ind_obj for frame, ... belief, the list for bp, and the ind_form

% to get form, you MUST enter at least ind_obj,ind_bp
% to get  bp,  you MUST enter at least ind_obj
% for bp, if ind_bp is absent, one bp for each conditioning element

% output = ind_obj so I knwo which object was make_empty
% pin_table not updated as I might cancel the whole operation

%
%   Version : Aug, 10, 03
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief

if  ismember(object,{'frame','structure','mapping','attribute','variable','belief'}) 
	new_pin = get_next_pin(object);
	nbr = new_pin(3);
	if nargin == 1
		ind_obj = new_pin(2);
	end
end

ind_new_obj = 0;

switch object
case 'frame'
	frame(ind_obj).number = nbr;
	frame(ind_obj).short_name = ['F',num2str(nbr)];
	frame(ind_obj).full_name = ['F',num2str(nbr)];
	frame(ind_obj).type = 'Binary';
	frame(ind_obj).cardinality = [2];
	frame(ind_obj).elements = strvcat('0','1');	
	frame(ind_obj).min = 0;	
	frame(ind_obj).max = 0;	
	ind_new_obj = ind_obj;
	
case 'structure'
	structure(ind_obj).number = nbr;
	structure(ind_obj).short_name = ['S',num2str(nbr)];
	structure(ind_obj).full_name = ['S',num2str(nbr)];
	structure(ind_obj).most_refined = [];
	structure(ind_obj).sub_frames = [];
	structure(ind_obj).mapping = [];
	ind_new_obj = ind_obj;
	
case 'mapping'
	mapping(ind_obj).number = nbr;
	ind_new_obj = ind_obj;
	
case 'attribute'
	attribute(ind_obj).number = nbr;
	attribute(ind_obj).short_name = ['A',num2str(nbr)];
	attribute(ind_obj).full_name = ['A',num2str(nbr)];
	attribute(ind_obj).structure = [];
	attribute(ind_obj).index = [];	
	ind_new_obj = ind_obj;
	
case 'variable'
	variable(ind_obj).number = nbr;
	variable(ind_obj).short_name = ['V',num2str(nbr)];
	variable(ind_obj).full_name = ['V',num2str(nbr)];
	variable(ind_obj).attribute = [];
	variable(ind_obj).frame = [];
	variable(ind_obj).index_value = [];
	ind_new_obj = ind_obj;
	
case 'belief'
	belief(ind_obj).number = new_pin(3);
	belief(ind_obj).short_name = ['B',num2str(new_pin(3))];
	belief(ind_obj).full_name = ['B',num2str(new_pin(3))];
	belief(ind_obj).domain_variable = [];
	belief(ind_obj).domain_cardinality = [];
	belief(ind_obj).conditioning_variable = [];
	belief(ind_obj).conditioning_cardinality = [];
	belief(ind_obj).agent = [];
	belief(ind_obj).time = [];
	belief(ind_obj).ec = [];
	belief(ind_obj).origin = [] ;
	belief(ind_obj).computed = [];
	belief(ind_obj).disposable = 0;
	belief(ind_obj).dci_var.pin_d = [];  % for discrete
	belief(ind_obj).dci_var.pin_c = [];  % for continuous
	belief(ind_obj).dci_var.pin_i = [];  % for integres
	ind_new_obj = ind_obj;
	
case {'bp','potential'}
	if nargin >= 3 & ~isempty(ind_bp)
		list_ind_bp = ind_bp;
	elseif (nargin >= 3 & isempty(ind_bp)) |  nargin == 2
		pr = prod([belief(ind_obj).conditioning_cardinality]);
		if pr == 0
			pr = 1;
		end	
		list_ind_bp = [1:pr];
	else
		boum_no_bel_ind	% accident absurd	
	end

	for ind_bp = list_ind_bp
		belief(ind_obj).bp(ind_bp).number = ind_bp;	
		belief(ind_obj).bp(ind_bp).full_name = ['bp',num2str(ind_bp)];
		if length(list_ind_bp) > 1 % this is a case with conditioning variables
			[var_name_list, element_list, element_numbers,label_details] = ...
			creating_elements(belief(ind_obj).conditioning_variable);	
			belief(ind_obj).bp(ind_bp).cd_focal_set = ind_bp;
			ccd = [];	
			nvar = length(belief(ind_obj).conditioning_variable);
			if nvar 
				for k = 1:nvar 
					ind_var = pin_manipulation('variable','num2ind',belief(ind_obj).conditioning_variable(k));
					ccd = [ccd ,variable(ind_var).short_name,'=',ddblank(label_details.text{k,element_numbers(ind_bp,nvar+1-k)}),', '];
				end
				ccd(end) = '';
				ccd(end) = '';
			end
			belief(ind_obj).bp(ind_bp).condition = ccd;
			
		else % the marginal case
			belief(ind_obj).bp(ind_bp).cd_focal_set = [];
			belief(ind_obj).bp(ind_bp).condition = [];			
		end
		belief(ind_obj).bp(ind_bp).form = [];	
		
		index_object = make_empty('form',ind_obj,ind_bp,1);
		% for each bp, I create a vacuous bba
	end
	ind_new_obj = list_ind_bp;
	
case 'form'	
	if nargin < 3
		boum_no_bel_bp_ind
	elseif nargin == 3
		if ~isfield(belief(ind_obj).bp(ind_bp ),'form')
			ind_form = 1;
		else
			ind_form = length(belief(ind_obj).bp(ind_bp).form) + 1;
		end
	elseif nargin == 4 & isempty(ind_form) % not | with previous as if nargin = 3, isempty(ind_form) crashes
		if ~isfield(belief(ind_obj).bp(ind_bp ),'form')
			ind_form = 1;
		else
			ind_form = length(belief(ind_obj).bp(ind_bp).form) + 1;
		end	
	end

	[univd,univci] = get_universe_dci(ind_obj,'domain');
	
	belief(ind_obj).bp(ind_bp ).form(ind_form).type = 'm'; 
	belief(ind_obj).bp(ind_bp ).form(ind_form).focal_sets(1,:) = univd;
	belief(ind_obj).bp(ind_bp ).form(ind_form).values = 1;
	belief(ind_obj).bp(ind_bp ).form(ind_form).representation = 'selected';
	belief(ind_obj).bp(ind_bp ).form(ind_form).log_expr = []; % to clear it if necessary
	belief(ind_obj).bp(ind_bp ).form(ind_form).log_expr{1} = [];	
	belief(ind_obj).bp(ind_bp ).form(ind_form).contint = []; % for ci means vacuous (by convention)
	ind_new_obj = ind_form;
end

