function [ok_objects] = verifying_object_data(option_in)
% verifying that every object is correct and printing a report on inadequate data
% option_in = 0, all details, 1 = skip display of warndlg on window (for secret verification)

global frame structure mapping attribute variable belief
global infinity

ok_objects = 1;

if nargin == 1
	option = option_in;
else
	option = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% I fill all fields
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fill_objects

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% frame
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ind_obj = 1:length(frame)
	nbr_err = 0;
	[nbr_err, error_mess] = chk_begin(frame(ind_obj),nbr_err);
	
	if ~ismember(frame(ind_obj).type,{'Logical','Binary','Classes_Un_ordered','Classes_Ordered','Integers','Continuous'})
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['frame(',num2str(ind_obj),').type wrong.'];
	end

	if ismember(frame(ind_obj).type,{'Logical','Binary','Classes_Un_ordered','Classes_Ordered'})
		if length(frame(ind_obj).cardinality)~= 1 | ~isnumeric(frame(ind_obj).cardinality)
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['frame(',num2str(ind_obj),').cardinality not one numerical.'];
		end
	
		if 	size(frame(ind_obj).elements,1) ~= frame(ind_obj).cardinality
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['number of frame(',num2str(ind_obj),').elements not correct.'];
		end
	end

	if ismember(frame(ind_obj).type,{'Integers','Continuous'})		
		if length(frame(ind_obj).min)~=1 | ~isnumeric(frame(ind_obj).min)
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['frame(',num2str(ind_obj),').min not one numerical.'];
			minX = infinity;
		else
			minX = frame(ind_obj).min;
		end
		if length(frame(ind_obj).max)~=1 | ~isnumeric(frame(ind_obj).max)
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['frame(',num2str(ind_obj),').max not one numerical.'];
			maxX = -infinity;
		else
			maxX = frame(ind_obj).max;
		end
		switch frame(ind_obj).type
		case 'Integers'
			if length(frame(ind_obj).cardinality)~=1 | ~isnumeric(frame(ind_obj).cardinality)
				nbr_err = nbr_err + 1;
				error_mess{nbr_err} = ['frame(',num2str(ind_obj),').cardinality not one numerical.'];
			end
			% element is empty for integers
			% 			if size(frame(ind_obj).elements,1) ~= frame(ind_obj).cardinality
			% 				nbr_err = nbr_err + 1;
			% 				error_mess{nbr_err} = ['frame(',num2str(ind_obj),').elements is not correct.'];
			% 			end	
			if frame(ind_obj).min == infinity | frame(ind_obj).max == infinity
				if frame(ind_obj).cardinality ~= infinity
					nbr_err = nbr_err + 1;
					error_mess{nbr_err} = ['frame(',num2str(ind_obj),').cardinality should be infinity.'];
				end
				if ~isempty(frame(ind_obj).elements)
					nbr_err = nbr_err + 1;
					error_mess{nbr_err} = ['frame(',num2str(ind_obj),').elements is not empty for an integer frame.'];
				end							
			else
				if frame(ind_obj).cardinality ~= maxX - minX + 1
					nbr_err = nbr_err + 1;
					error_mess{nbr_err} = ['frame(',num2str(ind_obj),').cardinality not equal max - min + 1.'];
				end				
			end
		case 'Continuous'
			if length(frame(ind_obj).cardinality)~=1 | ~isnumeric(frame(ind_obj).cardinality)
				nbr_err = nbr_err + 1;
				error_mess{nbr_err} = ['frame(',num2str(ind_obj),').cardinality not one numerical.'];
			end
			if ~isempty(frame(ind_obj).elements)
				nbr_err = nbr_err + 1;
				error_mess{nbr_err} = ['frame(',num2str(ind_obj),').elements is not empty.'];
			end				
		end	
	end

	if nbr_err 
		disp(' ')
		disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
		disp('Error in next frame object')
		frame(ind_obj)
		error_mess'
		ok_objects = 0;
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% structure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ind_obj = 1:length(structure)
	nbr_err = 0;
	[nbr_err, error_mess] = chk_begin(structure(ind_obj),nbr_err);
	
	if length(structure(ind_obj).most_refined)~=1 | ~isnumeric(structure(ind_obj).most_refined)
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['structure(',num2str(ind_obj),').most_refined not one numerical.'];
	end

	if ~isnumeric(structure(ind_obj).sub_frames) 
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['structure(',num2str(ind_obj),').sub_frames not numerical.'];		
	end

	if min(ismember([structure(ind_obj).most_refined   structure(ind_obj).sub_frames], [frame(:).number])) == 0		
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['structure(',num2str(ind_obj),') refers to non exisiting frame objects.'];			
	end

	if ~isnumeric(structure(ind_obj).mapping)
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['structure(',num2str(ind_obj),').mapping not numerical.'];
	else
		% 		if min(ismember([structure(ind_obj).mapping], [mapping(:).number])) == 0		
		% 			nbr_err = nbr_err + 1;
		% 			error_mess{nbr_err} = ['structure(',num2str(ind_obj),') refers to non exisiting mapping objects.'];				
		% 		end
	end

	if nbr_err 
		disp(' ')
		disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
		disp('Error in next structure object')
		structure(ind_obj)
		error_mess'
		ok_objects = 0;
	end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% mapping
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% attribute
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ind_obj = 1:length(attribute)
	nbr_err = 0;
	[nbr_err, error_mess] = chk_begin(attribute(ind_obj),nbr_err);
	
	if ~isnumeric(attribute(ind_obj).structure) | length(attribute(ind_obj).structure)~=1
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['attribute(',num2str(ind_obj),').structure not one numerical.'];	
	else
		if ismember([attribute(ind_obj).structure], [structure(:).number]) == 0		
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['attribute(',num2str(ind_obj),') refers to non exisiting structure objects.'];				
		end
	end

	for i = 1:length(attribute(ind_obj).index) 
		mess = issymbolic(attribute(ind_obj).index(i).name);
		for ii = 1:length(mess)
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = mess{ii};
		end		
		if length(attribute(ind_obj).index(i).frame)~= 1 |  ismember(attribute(ind_obj).index(i).frame, [frame(:).number]) == 0
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['attribute(',num2str(ind_obj),').index(',num2str(i),') refers to non exisiting structure objects.'];
		end
	end

	if nbr_err 
		disp(' ')
		disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
		disp('Error in next attribute object')
		attribute(ind_obj)
		error_mess'
		ok_objects = 0;
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% variable
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ind_obj = 1:length(variable)
	nbr_err = 0;
	[nbr_err, error_mess] = chk_begin(variable(ind_obj),nbr_err);
	
	if ~isnumeric(variable(ind_obj).attribute) | length(variable(ind_obj).attribute)~=1
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['variable(',num2str(ind_obj),').attribute not one numerical.'];	
	else
		if ismember([variable(ind_obj).attribute], [attribute(:).number]) == 0		
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['variable(',num2str(ind_obj),') refers to non exisiting attribute objects.'];				
		end
	end

	if ~isnumeric(variable(ind_obj).frame) | length(variable(ind_obj).frame)~=1
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['variable(',num2str(ind_obj),').frame not one numerical.'];	
	else
		if ismember([variable(ind_obj).frame], [frame(:).number]) == 0		
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['variable(',num2str(ind_obj),') refers to non exisiting frame objects.'];				
		end
	end

	ind_att = pin_manipulation('attribute','num2ind',variable(ind_obj).attribute);
	if length(attribute(ind_att).index)
		if ~isnumeric(variable(ind_obj).index_value) | length(variable(ind_obj).index_value)~=length(attribute(ind_att).index)
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['variable(',num2str(ind_obj),').index_value not numerical or their number is not correct.'];	
		else
			for k = 1:length(variable(ind_obj).index_value)				
				ind_fra = pin_manipulation('frame','num2ind',  attribute(ind_att).index(k).frame);
				if 	ismember(frame(ind_fra).type,{'Logical','Binary','Classes_Un_ordered','Classes_Ordered'})
					if variable(ind_obj).index_value(k) > frame(ind_fra).cardinality | variable(ind_obj).index_value(k) < 1
						nbr_err = nbr_err + 1;
						error_mess{nbr_err} = ['variable(',num2str(ind_obj),').index_value(',num2str(k),') not an element of the frame used by the index.'];						
					end
				else			
					if variable(ind_obj).index_value(k) > frame(ind_fra).max | variable(ind_obj).index_value(k) < frame(ind_fra).min
						nbr_err = nbr_err + 1;
						error_mess{nbr_err} = ['variable(',num2str(ind_obj),').index_value(',num2str(k),') not an element of the frame used by the index.'];						
					end
				end
			end
		end
	end

	if nbr_err 
		disp(' ')
		disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
		disp('Error in next variable object')
		variable(ind_obj)
		error_mess'
		ok_objects = 0;
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% belief
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ind_bel = 1:length(belief)
	nbr_err = 0;
	[nbr_err, error_mess] = chk_begin(belief(ind_bel),nbr_err);
	
	if ~isnumeric(belief(ind_bel).domain_variable) | length(belief(ind_bel).domain_variable)==0
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['belief(',num2str(ind_bel),').domain_variable not numerical.'];	
	else
		if ismember([belief(ind_bel).domain_variable], [variable(:).number]) == 0		
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['belief(',num2str(ind_bel),').domain_variable refers to non exisiting variable objects.'];				
		end
	end

	for i = 1:length(belief(ind_bel).domain_variable)
		ind_var = pin_manipulation('variable','num2ind',belief(ind_bel).domain_variable(i));
		ind_fra = pin_manipulation('frame','num2ind',variable(ind_var).frame);
		if belief(ind_bel).domain_cardinality(i) ~= frame(ind_fra).cardinality
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['belief(',num2str(ind_bel),').domain_cardinality(',num2str(i),') not equal to frame cardinality.'];				
		end		
	end

	if length(belief(ind_bel).conditioning_variable) ~= 0
		if ~isnumeric(belief(ind_bel).conditioning_variable)
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['belief(',num2str(ind_bel),').conditioning_variable exists but not one numerical.'];				
		else
			if min(ismember([belief(ind_bel).conditioning_variable], [variable(:).number])) == 0		
				nbr_err = nbr_err + 1;
				error_mess{nbr_err} = ['belief(',num2str(ind_bel),').conditioning_variable refers to non exisiting variable objects.'];				
			end
		end
		for i = 1:length(belief(ind_bel).conditioning_variable)
			ind_var = pin_manipulation('variable','num2ind',belief(ind_bel).conditioning_variable(i));
			ind_fra = pin_manipulation('frame','num2ind',variable(ind_var).frame);
			if belief(ind_bel).conditioning_cardinality(i) ~= frame(ind_fra).cardinality
				nbr_err = nbr_err + 1;
				error_mess{nbr_err} = ['belief(',num2str(ind_bel),').conditioning_cardinality(',num2str(i),') not equal to frame cardinality.'];				
			end		
		end	
	else
		belief(ind_bel).conditioning_cardinality = []; % to make sure it is not a 0 which brakes next rule (prod([]) = 1)
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% potential   
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	if length(belief(ind_bel).bp) ~= prod(belief(ind_bel).conditioning_cardinality)
		nbr_err = nbr_err + 1;
		error_mess{nbr_err} = ['belief(',num2str(ind_bel),') has not the correct number of potentials.'];				
	else
		for ind_pot = 1:length(belief(ind_bel).bp)
			if ~isnumeric(belief(ind_bel).bp(ind_pot).number)	
				nbr_err = nbr_err + 1;
				error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').number is not a numerical.'];				
			end				
		end	
		% maybe to be done %%%%%%%%%   for CI   belief(ind_bel).bp(ind_pot).cd_focal_set = 1;	not user accessible, so normaly OK	
		% belief(ind_bel).bp(ind_pot).condition = [];	blabla so OK
		if length(belief(ind_bel).bp(ind_pot).form) == 0
			nbr_err = nbr_err + 1;
			error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),') has no form.'];				
		else
	
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%
			% form
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			list_type = lower({'m','b','bel','pl','q','wc','wd','betp','BetP','m_pdf','BetP_pdf','m_from_BetP_pdf'});
			for ind_for = 1:length(belief(ind_bel).bp(ind_pot).form)
				if ~ismember(lower(belief(ind_bel).bp(ind_pot).form(ind_for).type),list_type);
					nbr_err = nbr_err + 1;
					error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').form(',num2str(ind_for),').type is unknown.'];				
				end
			
				if ~ismember(lower(belief(ind_bel).bp(ind_pot).form(ind_for).representation),...
					{'selected','ordered','binomial','beta','gamma','gaussian','laplace',...
					'binomial_based','beta_based','gamma_based','gaussian_based','laplac_based', })
					nbr_err = nbr_err + 1;
					error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').form(',num2str(ind_for),').representation is unknown.'];				
				end
			
			
				% no check for CI
				if length(belief(ind_bel).dci_var.pin_c) == 0 & length(belief(ind_bel).dci_var.pin_i) == 0					
					switch belief(ind_bel).bp(ind_pot).form(ind_for).representation
					case 'selected'
						if size(belief(ind_bel).bp(ind_pot).form(ind_for).values,1) ~= size(belief(ind_bel).bp(ind_pot).form(ind_for).focal_sets,1)
							nbr_err = nbr_err + 1;
							error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').form(',num2str(ind_for),'): number of values different from number of focal sets.'];				
						end
						[n, remain] =  words_for_set(prod(belief(ind_bel).domain_cardinality));
						if size(belief(ind_bel).bp(ind_pot).form(ind_for).focal_sets,2) ~= n+1
							nbr_err = nbr_err + 1;
							error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').form(',num2str(ind_for),'): focal sets badly represented.'];				
						end							
					case 'ordered'
						% always ok
					otherwise % all the pdf cases
						if length(belief(ind_bel).bp(ind_pot).form(ind_for).values) ~= 4
							nbr_err = nbr_err + 1;
							error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').form(',num2str(ind_for),'): a pdf based potential and nbr parameters is not 4.'];				
						end						
					end
				end
				if ~isnumeric(belief(ind_bel).bp(ind_pot).form(ind_for).values)
					nbr_err = nbr_err + 1;
					error_mess{nbr_err} = ['belief(',num2str(ind_bel),').bp(',num2str(ind_pot),').form(',num2str(ind_for),'): some values are not numerical.'];				
				end
			
			end
		end % for this form
	end % for every bp

	if nbr_err 
		disp(' ')
		disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
		disp('Error in next belief object')
		detail_on_belief(belief(ind_bel))
		disp(' ')
		disp('Error message')
		error_mess'
		ok_objects = 0;
	end
end % for every belief

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% verifying unicity of .number and .short_name
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isempty(frame) & length(unique([frame(:).number])) ~= length([frame(:).number])
	disp(' ')
	disp('Several frame objects share the same number. List of numbers.')
	for i = 1:length(frame)
		fprintf(1,'number %5i\n', [frame(i).number]);
	end
	ok_objects = 0;
end

if ~isempty(frame) & length(unique({frame(:).short_name})) ~= length({frame(:).short_name})
	disp(' ')
	disp('Several frame objects share the same short name. List of numbers, short names and long names.')
	for i = 1:length(frame)
		fprintf(1,'%s\n', [num2str(frame(i).number),'   ',frame(i).short_name,'   ',frame(:).full_name]);
	end
	ok_objects = 0;
end

if ~isempty(structure) & length(unique([structure(:).number])) ~= length([structure(:).number])
	disp(' ')
	disp('Several structure objects share the same number')
	for i = 1:length(structure)
		fprintf(1,'number %5i\n', [structure(i).number]);
	end
	ok_objects = 0;
end

if ~isempty(structure) & length(unique({structure(:).short_name})) ~= length({structure(:).short_name})
	disp(' ')
	disp('Several structure objects share the same short name. List of numbers, short names and long names.')
	for i = 1:length(structure)
		fprintf(1,'%s\n', [num2str(structure(i).number),'   ',structure(i).short_name,'   ',structure(:).full_name]);
	end
	ok_objects = 0;
end

if ~isempty(attribute) & length(unique([attribute(:).number])) ~= length([attribute(:).number])
	disp(' ')
	disp('Several attribute objects share the same number')
	for i = 1:length(attribute)
		fprintf(1,'number %5i\n', [attribute(i).number]);
	end
	ok_objects = 0;
end

if ~isempty(attribute) & length(unique({attribute(:).short_name})) ~= length({attribute(:).short_name})
	disp(' ')
	disp('Several attribute objects share the same short name. List of numbers, short names and long names.')
	for i = 1:length(attribute)
		fprintf(1,'%s\n', [num2str(attribute(i).number),'   ',attribute(i).short_name,'   ',attribute(:).full_name]);
	end
	ok_objects = 0;
end

if ~isempty(variable) & length(unique([variable(:).number])) ~= length([variable(:).number])
	disp(' ')
	disp('Several variable objects share the same number')
	for i = 1:length(variable)
		fprintf(1,'number %5i\n', [variable(i).number]);
	end
	ok_objects = 0;
end

if ~isempty(variable) & length(unique({variable(:).short_name})) ~= length({variable(:).short_name})
	disp(' ')
	disp('Several variable objects share the same short name. List of numbers, short names and long names.')
	for i = 1:length(variable)
		fprintf(1,'%s\n', [num2str(variable(i).number),'   ',variable(i).short_name,'   ',variable(:).full_name]);
	end
	ok_objects = 0;
end

if ~isempty(belief) & length(unique([belief(:).number])) ~= length([belief(:).number])
	disp(' ')
	disp('Several belief objects share the same number')
	for i = 1:length(belief)
		fprintf(1,'number %5i\n', [belief(i).number]);
	end
	ok_objects = 0;
end

if ~isempty(belief) & length(unique({belief(:).short_name})) ~= length({belief(:).short_name})
	disp(' ')
	disp('Several belief objects share the same short name. List of numbers, short names and long names.')
	for i = 1:length(belief)
		fprintf(1,'%s\n', [num2str(belief(i).number),'   ',belief(i).short_name,'   ',belief(:).full_name]);
	end
	ok_objects = 0;
end
disp(' ')
disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
if ok_objects
	disp('XXXXXXXXX  Data Verification Satisfied   XXXXXXXXXXX')
	if option == 0
		hdw = warndlg('Data Verification Satisfied');
		uiwait(hdw)
	end	
else
	disp('XXXXXXXXX    Data Verification ERRORS    XXXXXXXXXXX')
	hdw = warndlg('Data Verification ERRORS: see details in MATLAB Command Window.');
	uiwait(hdw)		
end
disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')
disp('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [nbr_err, error_mess] = chk_begin(object,nbr_err)
% verifying number, short name and full name
error_mess = '';
if ~isnumeric(object.number) | isempty(object.number)
	nbr_err = nbr_err + 1;
	error_mess{1} = 'number incorrect';
end
mess = issymbolic(object.short_name);
for i = 1:length(mess)
	nbr_err = nbr_err + 1;
	error_mess{nbr_err} = mess{i};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [mess] = issymbolic(t)
% is t a string with a letter a... z, A...Z in front, and then only letters, digits, , . _ - ( ) < > = 
% ascii codes for letters digits . and _, for digits - and . < = >
% 46 ., digit = 48:57,  A...Z = 65:90,  a...z = 97:122  - = 45, _ = 95    60:62 = <=>  40:41 () 44 , & 38
% ()- 40:41  , = 44

allowed_char = [48:57 60:62 65:90 97:122 40 41 44 45 46 95 38];
allowed_initial_symb = [65:90 97:122];

n_mess = 0;
mess = '';
if ~ischar(t)
	n_mess = n_mess + 1;
	mess{n_mess} = 'Name is a number. ';
else
	if ~ismember(t(1),allowed_initial_symb)
		n_mess = n_mess + 1;
		mess{n_mess} = 'Name does not start with a letter. '; 
	else
		for j = 2:length(t)
			if  ~ismember(t(j),allowed_char)
				n_mess = n_mess + 1;
				mess{n_mess} = 'Name has forbidden charaters.';
			end
		end
	end
end
