function out = get_bbd_uv(x1,x2,v_pdf,u_pdf,v_p,u_p,potential_list)
% generating bel([x1,x2]), pl([x1,x2]), etc from an u-v representation, 
% based on U-non-Interactive bbd
% g^U(u,v) = h(u).f(v)    where u  0, v from -infnity to infinity (or less)

% potential_list = list of those potential asked for.
% one or several among { m, b, bel, pl, q, M, Bel, Pl, Q, BetP}

% (u_pdf v_pdf) = (beta,beta), (laplace,gamma), (gauss,gamma)
% u_p v_p are the u and v pdf parameters

%
%   Version : Dec 15,03.
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% input data verification
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

out = [];
ok = 1;
if min(ismember(potential_list,{'m', 'b', 'bel', 'pl', 'q', 'M', 'Bel', 'Pl', 'Q', 'BetP'})) == 0
	hdw = warndlg('In get_bbd_uv, you ask for an element in potential_list that does not exists. I skip.')
	uiwait(hdw)
	ok = 0;
end	

if x1 > x2
	hdw = warndlg('In get_bbd_uv, the limits form x1 to x2 are not valid. I skip.')
	uiwait(hdw)
	ok = 0;
end

if ok 
	switch lower(v_pdf)
	case 'beta'
		switch lower(u_pdf)
		case 'beta'
			hdw = warndlg('You ask a beta-beta U-non_interactive that is not yet available. I skip.')
			uiwait(hdw)
		otherwise
			hdw = warndlg('You ask a U-non_interactive that I cannot handle. I skip.')
			uiwait(hdw)
		end
	case 'laplace'
		switch lower(u_pdf)
		case 'gamma'
			if u_p(1) <= 0 | v_p(2) <= 0
				hdw = warndlg('In get_bbd_uv, some parameters not aceptable. I skip.')
				uiwait(hdw)
				ok = 0;
			else
				out = bbd_laplace_gamma(x1,x2,v_p(1),v_p(2),u_p(1),potential_list);
			end
		otherwise
			hdw = warndlg('You ask a U-non_interactive that I cannot handle. I skip.')
			uiwait(hdw)
		end
	case {'gauss','gaussian'}
		switch lower(u_pdf)
		case 'gamma'	
			if u_p(1) <= 0 | v_p(2) <= 0
				hdw = warndlg('In get_bbd_uv, some parameters not aceptable. I skip.')
				uiwait(hdw)
				ok = 0;
			else
				out = bbd_gauss_gamma(x1,x2,v_p(1),v_p(2),u_p(1),potential_list);
			end
		otherwise
			hdw = warndlg('You ask a U-non_interactive that I cannot handle. I skip.')
			uiwait(hdw)
		end
	end
end

