function structure_objects_Aug_09_02
% structure of the belief functions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the obejcts are = frame, strycture, mapping, attribute, variable, belief
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% THE GENERAL STRUCTURE
%
% belief potentials are defined on frame of discernement = a set of variables
%
% variable 	= (attrinute, frame) 
%			= age of John in years
%
% frame 	= [0 1 2 ... 120]
%
% attribute = age : refers to a structure, can be indexed

% structure = several frames that are domains for same attribute. 
%     		  integers from 0 to 120 for age in year, 
%             integers from 0 to 120 for young people age in year
%             bianry as in Young Not_Young
%     	It contains the interrelations: mapping
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% FRAME
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% A frame is a set of values for a variable. It can be of several types:
% 		Logical      
% 		Binary
% 		Classes_Un_ordered
% 		Classes_Ordered
% 		Integers
% 		Continuous (it works only for one dimensional closed intervals, olr based on pdf (= bba or BetP))
%
% The definition of a frame depends on its type
%
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The default frame
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

frame(ind_obj).number = nbr;
frame(ind_obj).short_name = ['F',num2str(nbr)];
frame(ind_obj).full_name = ['F',num2str(nbr)];
frame(ind_obj).type = 'Binary';
frame(ind_obj).cardinality = [2];
frame(ind_obj).elements = strvcat('N','Y');	
frame(ind_obj).min = 0;	
frame(ind_obj).max = 0;	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A frame of type 'Classes_Ordered'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
frame(1).number = 5;
frame(1).full_name='Theta';
frame(1).short_name='Th';	
frame(1).type = 'Classes_Ordered';
frame(1).cardinality = 3;
frame(1).elements = strvcat('young','middle-aged','old'); 
% call frame(1).element(1,:) and you get young, 
% elements must be only letters, digits, space, _
% make then short, it helps user.
% cardinality = 0 when continuous.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Another frame of type 'Classes_Ordered'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
frame(2).number = 10;
frame(2).full_name='Theta_1';
frame(2).type = 'Classes_Ordered';
frame(2).cardinality = 10;
frame(2).elements = strvcat('00-10';'11-20';'21-30';'31-40';'41-50';'51-60';'61-70';'71-80';'81-90';'91-100');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A frame of type 'integer'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
frame(3).number = 6;
frame(3).full_name='Omega';
frame(3).type = 'Integers';
frame(3).min = 0;
frame(3).max = 100;   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Another frame of type 'integer', can be seen as a subset of the above
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
frame(4).number = 7;
frame(4).full_name='Omega_1';
frame(4).type = 'Integers';
frame(4).min = 0;
frame(4).max = 20;   


frame(4).type = 'Continuous';
frame(4).min = 0;
frame(4).max = 20;   

% continuous is [0,1], [a,b], [0, inf),  (-inf,inf)
% infinity is  by definition 1E40
% so (min,max) = (0,1), (a,b), (0, 1E40),  (-1E40,1E40)
% and infinity is a global with 1E40 (so ca	n be changed if necessary)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% STRUCTURE 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Note:
% A structure is a collection of frames which can be used to express information
% regarding the value of an uncertain quantity, or "matter of interest".

% An attribute is defined on a structure (a collection of frames).
% If you pick a frame in the structure, you define a variable. 
% So variable =(attribute,frame,index_value). 
% Defining a structure is a way to designate a collection of frames 
% and express the fact that they can be used to sepecify the value of an attribute.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the default strucutre
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
structure(ind_obj).number = nbr;
structure(ind_obj).short_name = ['S',num2str(nbr)];
structure(ind_obj).full_name = ['S',num2str(nbr)];
structure(ind_obj).most_refined = [];
structure(ind_obj).sub_frames = [];
structure(ind_obj).mapping = [];

structure(1).number = 5;
structure(1).full_name = 'age';
structure(1).short_name = 'age';
structure(1).most_refined = 5;      % it is the most refined frame
structure(1).sub_frames = [10 6 7]; % other frames being either coarsening of subsets of it 
structure(1).mapping = [...
0 3 2 0;
0 0 1 0;
0 0 0 0;
0 0 0 0];
% ???? the above matrix R summarizes the coarsening relations. 
% If frame indicated in row i is a coarsening of frame indicetaed in column j,
% then R(i,j) is the number
% of the corresponding coarsening object in the data base. Otherwise R(i,j)=0
% i and j are in the order 5 10 6 7

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% MAPPING Not yet available
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ATTRIBUTE
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the default attribute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
attribute(ind_obj).number = nbr;
attribute(ind_obj).short_name = ['A',num2str(nbr)];
attribute(ind_obj).full_name = ['A',num2str(nbr)];
attribute(ind_obj).structure = [];
attribute(ind_obj).index = [];	

attribute(1).number = 17;   					% the number of the attribute
attribute(1).full_name = 'position';   			% full name of the attribute
attribute(1).short_name = 'pos';				% short name of the attribute
attribute(1).structure = 5;                     % an attribute is defined on a structure (a collection of frames)
attribute(1).index(1).name = 'time';
attribute(1).index(1).frame = 17;
attribute(1).index(2).name = 'target';
attribute(1).index(2).frame = 124;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% VARIABLE
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% A variable : an attribute + a frame
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the default variable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
variable(ind_obj).number = nbr;
variable(ind_obj).short_name = ['V',num2str(nbr)];
variable(ind_obj).full_name = ['V',num2str(nbr)];
variable(ind_obj).attribute = [];
variable(ind_obj).frame = [];
variable(ind_obj).index_value = [];

variable(189).number = 93;   							
variable(189).full_name = 'position(time = 10, target = Fregate) in km';	
variable(189).short_name = 'pos(t=10,Fregate)_km';				
variable(189).attribute = 53;
variable(189).index_value = [10 3];	 %  frame(124).element(3) = 'Fregate'
variable(189).frame = 10;	% points to a frame in km
% short names must be only letters,digits, space, _
% make it really short as using logical input will be simpler

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% a variable on an attribute without index

variable(19).number = 43;   							
variable(19).full_name = 'income';	
variable(19).short_name = 'inc';				
variable(19).attribute = 23;
variable(19).index_value = [];	 
variable(19).frame = 10;	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% BELIEF POTENTIAL
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% A belief is a set of belief potentials (one if marginal, K if conditional)
% with one or several forms for each potential
%
% a belief is an object which contains a belief potential under some form (m,b,...)
% it is linked to a variable
% it is hold by an agent, at a given time, 
% based on some evidential corpus
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the default belief
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
belief(ind_obj).number = new_pin(3);
belief(ind_obj).short_name = ['B',num2str(new_pin(3))];
belief(ind_obj).full_name = ['B',num2str(new_pin(3))];
belief(ind_obj).domain_variable = [];
belief(ind_obj).domain_cardinality = [];
belief(ind_obj).conditioning_variable = [];
belief(ind_obj).conditioning_cardinality = [];
belief(ind_obj).agent = [];
belief(ind_obj).time = [];
belief(ind_obj).ec = [];
belief(ind_obj).origin = [] ; % set of pair of numbers
belief(ind_obj).computed = []; % text
belief(ind_obj).disposable = 0;
belief(ind_obj).dci_var.pin_d = [];
belief(ind_obj).dci_var.pin_c = [];
belief(ind_obj).dci_var.pin_i = [];

belief(1).number = 24; 			% each belief state has a number
belief(1).full_name = 'Sensor 1 bel on aircraft type';
belief(1).short_name = 'B4';
belief(1).disposable = [];		% 1 = may be erased (clear) , 0 = [] = must be kept (default)

belief(1).agent = 'You';		% who is the belief holder
belief(1).time = 4;				% at what epoch
belief(1).ec = {};				% ec = evidential corpus, on what bel is based, blabla	

belief(1).domain_variable = [93 25] ;		% the domain can be a variable or a product of variables
belief(1).domain_cardinality = [12 5]; % list of cardinalities of the variables, same order

belief(1).conditioning_variable = 45; 	% the domain of the conditioning variable 
% if not empty, belief potential given for each singleton of the conditioning_variable
% if 0 or empty, belief is a marginal belief potential
belief(1).conditioning_cardinality = 0; % as for domain

% list of discrete, continuous, integer  variable position, index,numbers
% built in   type_variable  for the domain_variable

belief(belief_index).dci_var.pin_d = [ position ; index ; number ]  % for discrete
belief(belief_index).dci_var.pin_c = [ position ; index ; number ]  % for continuous
belief(belief_index).dci_var.pin_i = [ position ; index ; number ]  % for integres

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% potential
% normaly we have as many belief potentials as there are singletons in the conditioning_variable 
% or 1 if marginal.
% the k index cannot really be used as we can delete a bp.
% the delete is so that bp_index is still 1 to .... one less than before
% With  belief(33).bp(2) = [] what was bp(4) is now bp(3)
% hence relation potential index with elements conditioning is not OK.
% we use a foset in bp(i).foset so I know which element bp(i) is attached to.
% all missing bp are vacuous

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the defaut potential
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
belief(ind_obj).bp(ind_bp ).number = ind_bp;	
belief(ind_obj).bp(ind_bp ).full_name = ['bp',num2str(ind_bp)];
belief(ind_obj).bp(ind_bp ).cd_focal_set = 1;  % will be 1,2,3... cardinal cond space
belief(ind_obj).bp(ind_bp ).condition = [];	 % the value of the first , second, .... variable
% as conditioning on elements, condition = var1 = 3, var2 = 4, ....
% writtten as {'Male','E','45'}
belief(ind_obj).bp(ind_bp ).form = [];	% see below
belief(ind_obj).bp(ind_bp ).form(ind_form).contint; 
% the focal set to which it refers, table with limits for the integer and one continuous variables

belief(1).bp(i).number = i;	
belief(1).bp(43).full_name = 'bp43 m b BetP';
belief(1).bp(1).cd_focal_set = 26  		% position of element in product space
% data in cd_focal_set is one long number, hence not continuous/integers variables
belief(1).bp(1).condition = 'Sex=Male, Diag=Hep, Age=45';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% form
% .type state what exists 
% bf(k).form(1).type = 'm'; 
% capital = normalized,  not all exist
% m   / M   = bba
% b         = implicability fct
% bel / Bel = belief function
% pl  / Pl	= plausibility function
% q   / Q   = commonality function
% wc        = conjuncitve decomposition
% wd        = disjunctive decomposition
% lower_appox
% upper_approx
% BetP

% wa can have several forms, as we might have stored m and q for the same belief potential

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the default form
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
belief(ind_obj).bp(ind_bp ).form(ind_form).type = 'm'; 
belief(ind_obj).bp(ind_bp ).form(ind_form).focal_sets(1,:) = get_universe(card);
belief(ind_obj).bp(ind_bp ).form(ind_form).values = 1;
belief(ind_obj).bp(ind_bp ).form(ind_form).representation = 'selected';
belief(ind_obj).bp(ind_bp ).form(ind_form).log_expr{1} = [];
belief(ind_obj).bp(ind_bp ).form(ind_form).contint = [];

belief(1).bp(1).form(1).type = 'm'; 
belief(1).bp(1).form(2).type = 'b'; 

belief(1).bp(2).form(1).type = 'pl'; 
belief(1).bp(2).form(2).type = 'BetP';  

belief(1).bp(3).form(1).type = 'BetP'; 
% bp(1) because with conditional bp there are several, beware BedtP needs also a betting frame
% by default the same as the one on which belief is defined
% otherwise, build first the coarsening and then OK

belief(1).bp(5).cd_focal_set = i; 
% indicating the focal set of the conditioning domain ; i is element number in the conditioning domain
% to know which element bp(5) refers to.

belief(1).bp(1).form(1).representation = 'ordered'; 	 
% states how the focal elements are listed. It can be
% ordered = the whole power set ordered according to binary representation
% selected = some of the focal elements only, listed in .focal_set
% ordered requires cardinality <= nbr_bit_per_word, so ONE world is enough
% In fact ordered should not be used when carindality is larger that  15, 
% as we have to store a vector of length pow2(cardinality)

belief(1).bp(1).form(1).focal_sets = [];		
% list of focal sets when not 'ordered', vertical
% if more than 50 elements, focal_set is a line vectore with enough component 
% to represent the elements
belief(1).bp(1).form(1).values = [.15; .05; .03; .02; .20; .12; .08; .37];
% list of the values, vertical vector

% another example
belief(35).bp(1).form(1).type = 'm'; 
belief(35).bp(1).form(1).representation = 'selected'; 
belief(35).bp(1).form(1).focal_sets = [	12 234512 234; 
34 44 543;
2341 34 32;
4654 454243 55454];
% 4 focal elements, frame with 140 elements (so we need 3 memories)
belief(35).bp(1).form(1).values = [.3 .2 .1 .4]';
% values is vertical vector, same lines for focal set and value

belief(1).bp(1).form(2).representation = 'selected';  	% the b 	for cond bp given granule 1
belief(1).bp(2).form(1).representation = 'ordered'; 	% the pl 	for cond bp given granule 2
belief(1).bp(2).form(2).representation = 'ordered'; 	% the BetP 	for cond bp given granule 2
belief(1).bp(3).form(2).representation = 'selected'; 	% the BetP	for cond bp given granule 3

% case Integers and bbf 
belief(4).bp(3).form(1).type = 'm_pdf';
belief(4).bp(3).form(1).representation = 'binomial';
belief(4).bp(3).form(1).values = [p n];

% case Integers and BetP
belief(4).bp(3).form(1).type = 'BetP_pdf';
belief(4).bp(3).form(1).representation = 'binomial';
belief(4).bp(3).form(1).values = [p n];

belief(4).bp(3).form(2).type = 'm_from_BetP_pdf';
belief(4).bp(3).form(2).representation = 'binomial_based';
belief(4).bp(3).form(2).values = [p n];

% case Continuous and bbf
belief(4).bp(3).form(1).type = 'm_pdf';
belief(4).bp(3).form(1).representation = 'beta' or 'exponential' or 'gaussian';
belief(4).bp(3).form(1).values = [a b] or [t 1] or [mu sigma];

% case Continuous and BetP
belief(4).bp(3).form(1).type = 'BetP_pdf';
belief(4).bp(3).form(1).representation = 'beta' or 'gamma' or 'gaussian' or 'laplace';
belief(4).bp(3).form(1).values = [a b] or [t 1] or [mu sigma];

belief(4).bp(3).form(2).type = 'm_from_BetP_pdf';
belief(4).bp(3).form(2).representation = 'beta_based' or 'gamma_based' or 'gaussian_based' or 'laplace based';
belief(4).bp(3).form(2).values = [a b] or [t 1] or [mu sigma];


belief(1).bp(3).form(2).log_expr{i} = 'here is a logical formula' 
% when focal set obtained by logical formula, here is the formula, it concerns the i-th focal set of selected

belief(ind_bel).bp(ind_bp).form(ind_form).contint(i).limits = matrix;  % i comes from value(i)
% when continuous or integer var, for value(i) given to focal_sets(i,1:N) with bit(1:N) = list of bits of foset i
% limits = matrix(j,k) is the limits for bits j, and k means var1_inf var1_sup var2_inf var2_sup ...
% to give a limits for bits 0 is useless, and bit on without limits means limits are universe 
% ...form().contint = [] 					means for every bits, 	cont/int = universe
% ...form().contint(i).limits = [] 			means for bit i, 		cont/int = universe
% ...form().contint(i).limits = matrix  	means for bit i, 		cont/int = in matrix

% in fact if for bit i cont/int = universe, we just don't generate ...form().contint(i).limits

% example
% index values  	focal_set		
%   1	 .2			01011 = 11		matrix 3x4 		[2 v1l v1u v2l v2u ; 1 v1l v1u v2l v2u ; 4 v1l v1u v2l v2u ] 
%   2	 .5			00101 =  6		matrix 2x4		[1 v1l v1u v2l v2u ; 3 v1l v1u v2l v2u ]
%   3	 .3			10000 = 16		matrix 1x4		[5 v1l v1u v2l v2u ]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% storing how the belief potentilal were computeds
% NOT YET USED
%
% belief(1).origin = [7 11], the varibales involved.   = 0 if from external devise (keyboard, file, program,
% belief(12).computed = '7 and 11'  a string that say which computation is done, so I can read it if I want
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

belief(1).origin = [3 12; 5 13; 4 21; 1 34]   numerical
% the bp I am using (num_bel,ind_pot)
% belief(x).number = 3, belief(x).bp(12), belief(x).number = 5, belief(y).bp(13), ...


belief(1).computed = 'batch_mode' ; % entered by batch mode instr
belief(1).computed = 'keyboard' ; % entered by keyboard instr

belief(1).computed = '3.12 and 5.13 and 4.21 and 1.24' ; 
% conjunctive combination of the 4 bp  as above, num_bel,ind_pot

% the un-informative modifications  (pairs as above:  num_bel,ind_pot)
belief(12).computed = 'concat [3.12 5.13 4.21 1.34] '; % concatenation for set of conditional bp
% set of simple operation that can be perfomred on a bp
belief(12).computed = 'downarrow = 7.45';  % = marginalize on the variable 45 domain
belief(12).computed = 'uparrow = 6.32';	% = vacuous extension on the variable 32 domain
belief(12).computed = 'balloon_ext 2.21';	% = ballooning extension on product space variable 21 (to be created)
% the monitoring control unit will read belief(1).input.to_be_computed
% and ask the adequate function ot evaluate (with eval)

% the informative revisions
% we tell which computation was perfomred on the input bp in order to get the bp #35
belief(12).computed =  '(1.15 or 2.13) and 4.21 or not 6.34' % by default and or are for distinct pev
% telling how to combine them when distinct (ready for cautious combi)
% '12 and 17' means conj combi of bel 12 and bel 17 (handle domains if not =)
% bel[bel[E1],bel[E2],E1 and E2] = = bel[E1] conjunction bel[E2]
% '(15 or 13) and 21 or not 34' = disj(conj(disj(15,13), 21),neg 34)
belief(12).computed =  '(1.15 minus_or 2.13) minus_and 5.21 or not 4.34'		
% we also need  minus_and, minus_or, the inverse of conj, disj when distinct
belief(12).computed =  '(3.15 cautious_or 6.13) cautious_and 4.21 or not 2.34'	
% the case of cautious and and or
belief(12).computed =  'DRC 2'	% disjunctgive rule of combination
% the input is made of two components: the set of bel on X given T (the 12) and a bp on T (the 34) (a priori).
belief(12).computed =  'GBT 2'	% general Bayesian Theorem
% the input is made of two components: the set of bel on X given T (the 12) and a bp on X (the 34) (observation).


% etc....... open list		

belief(12).computed = 'discounting .7'; 
% the result of the computation or the only one received to discount
% [discouning .7] = the resulting bba has been discounted by disc.factor = .7
% bel_Y[bel_Z,disc^Y_Z = .7] == disc bel_Z by .7  (disc^Y_Z = Y disc Z by ...)
belief(12).computed = 'conditioning  9';
% conditioning on set with binary represented by 9 = 1001

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% A comment on the bit representation of subsets

% when I say that the frame of discernemment for a belief potential is [2 5 8]
% I mean it is based on variables.number = 2 5 and 8.
% If their cardinalities are [2 3 4]

% in wolrd 	1  2  3  4  5  6  7  8  9 10 ...24
% var 8	=	1  1  1  1  1  1  2  2  2  2 ... 4
% var 5	=	1  1  2  2  3  3  1  1  2  2 ... 3
% var 2	=	1  2  1  2  1  2  1  2  1  2 ... 2

% The function  [vect] = decompose(x,base)  produces then
% [1 2 3]   = decompose(23,[2 3 4]) where 23 is the long_number for world 24
% and 1 2 3 are the upper limits for each variable (they can be 0, 1, 2...
% whereas in the table I had put 1, 2, 3,...


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%              EVN Objects              %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

node(kn).number = nbr;				% its number
node(kn).label = expr; 				% variable().short_name or belief().short_name
node(kn).children = my_children;	
% if node(kn).nature = variable : number of (belief) nodes where condit refers to me 
% if node(kn).nature = belief   : the number of the nodes representing a var used in my domain
node(kn).parents = my_parents;		
% if node(kn).nature = variable : number of (belief) nodes where domain refers to me 
% if node(kn).nature = belief   : the number of the nodes representing a var used in my condit (end with arrow)
node(kn).center = pos;				% position of its center (x,y) in pixel
node(kn).center_initial = pos;		% position in initial plot, used for going back to initial plot
node(kn).nature = 'variable';		% either variable or belief
node(kn).shape = 'oval';			% shape of left riught end, oval (variable) long_hexagon (belief)
node(kn).obj_num = num_var; 		% the number of the variable / belief I represent
node(kn).obj_num_for_JVBS = num_bel;% the number of the belief used for JVBS that obj_num represents
% for cond belief, 
% obj_num is the number of the cond_bel (for plot), 
% obj_num_for_JVBS is the number of the joint_bel (for VBS computation) (built in run_VBS
node(kn).link = [];					% the links numbers that leaves from this node
node(kn).betp_localization = [bel_ind,ind_bp,ind_form_betp,bel_num ];
node(kn).hd_menu = 					% the node handle
node(kn).hd_picto
node(kn).localization
node(kn).localization_init_sc 

% arrows go from var_num to bel_num :     var ---> bel    when var_num is among conditioning_variable of bel_num
% let node(k1) represent var_num and node(k2) represent bel_num, 
% node(k1).nature = 'variable', node(k2).nature = 'belief'
% so arrow goes from node(k1) to node(k2) :  node(k1) ---> node(k2)
% node(k1).children  contains k2 (thus bel_num)
% node(k2).parents   contains k1 (thus var_num)
% these are the only arrows I use. 
% example   
% var1  ---->         ----- var3
%               bel
% var2  ---->         ----- var4
% means that we have bel domain_varaibale = (var3 X var4), conditioning_variable = (var1 X var2)
% bel ^var3 X var4  [ var1 X var2 ]

% related to pin tables
pin_node pin_link pin_histo

link(lk).number = nlk;		% link number
link(ik).nodes = list;		% list of node numbers to which this link attached
link(ik).tip = 1;			% arrow : 0 if none, 1 if link ends with arrow

link(lk).nodes = [nbr_bel_node  nbr_var_node];
child = 1; % this variable is a domain var of bel
child = 0; % this variable is a condit var of bel

histo(ind_histo).number = 
histo(ind_histo).node_ind = i_node;
histo(ind_histo).center = 
histo(ind_histo).title_text = variable(ind_var).short_name;
histo(ind_histo).cardinal = cardi;
histo(ind_histo).height_fr_histo = h_element*(cardi+1)+skipv;
histo(ind_histo).width_element = min(max(15,nbr_char*w_char),w_char*7);
histo(ind_histo).betp = belief(ix(1)).bp(ix(2)).form(ix(3)).values;		
histo(ind_histo).elements = frame(ind_fra).elements;	
histo(ind_histo).handle = hd_histo;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%            Menu  Objects              %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pull_down_menu(ind).label = 'Propagate';
pull_down_menu(ind).status = 1;
pull_down_menu(ind).hd = hd_menu(ik);
pull_down_menu(ind).fig_parent = 'EVN figure';
pull_down_menu(ind).father = 'root';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%             figure_data               %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Structure of figure_data

for a window
	ind_fig = length(figure_data) + 1;
	figure_data(ind_fig).name = 'Belief Editor';
	figure_data(ind_fig).handle = hd_fig;
	figure_data(ind_fig).position = gui_position(:,5);
	
	for each menu, in make_fig_menu
		figure_data(ind_fig).menu(ind_menu).label = label;
		figure_data(ind_fig).menu(ind_menu).father = father;
		figure_data(ind_fig).menu(ind_menu).clb = clb;
		figure_data(ind_fig).menu(ind_menu).enable = enable;
		figure_data(ind_fig).menu(ind_menu).separator = 'on';
		
		in menu_manager at creation
		figure_data(ind_fig).menu(i).depth = now_menu(i).depth;
		figure_data(ind_fig).menu(i).handle = now_menu(i).handle;
		figure_data(ind_fig).menu(i).path = now_menu(i).path;
		figure_data(ind_fig).menu(i).seq_nbr = now_menu(i).seq_nbr;
		
		
		when closing
		figure_data(ind_fig) = [];
		
				
