function [lkh] = lkh_from_pdf(pdf,param,from,to)
% integrating pdf with param, from to
% if to == from, point value, you get pdf itself, no integral
% for the case where pdf is bba
% the other case (pdf is BetP) see lciso_pl_from_pdf

if nargin == 3
	to = from;
end

switch pdf
case 'beta' 
	a = param(1); b = param(2);
	if to == from
		lkh = (from^(a-1)*(1-from)^(b-1))/beta(a,b);
	else
		lkh = betainc(from,a,b) -  betainc(to,a,b);  % betainc = int 0 to x 
	end
case 'gamma'  % param(1) = theta, param(2) = p 
	% t^n.x^(n-1).exp(-tx)/G(n).	
	t = param(1); n = param(2);
	if to == from
		lkh =  (1/ gamma(n))*t*(from/t)^(n-1)*exp(-from/t); 
	else
		lkh = gammainc(from/t,n) - gammainc(to/t,n);	
	end
case 'laplace' % param(1) = mode alpha, param(2) = scale 
	% (1/2*b)  exp ( -|x-a| / b )     
	% mean = a,    stdev = sqrt(2)*b
	a = param(1); b = param(2);	
	if to == from
		lkh =  (.5/ b) *exp(-abs(from-a)/b); 
	else
		if from > a
			lkh = (exp(-abs(from - a)/b) - exp(-abs(to - a)/b))/2;
		elseif to < a
			lkh = (exp(-abs(to - a)/b) - exp(-abs(from - a)/b))/2;
		else
			lkh = (2 - exp(-abs(from - a)/b) - exp(-abs(to - a)/b))/2;
		end
	end	
case 'gaussian'  % param(1) = mean, param(2) = stdev 
	m = param(1); s = param(2);	
	if to == from
		lkh = (1/(sqrt(2*pi)*s))*exp(-(from-m)^2/(2*s^2));
	else
		int1 = erf(abs((m-from))/(sqrt(2)*s))/2;
		int2 = erf(abs((m-to))/(sqrt(2)*s))/2;
		if to < param(1)
			lkh = int1 - int2;
		elseif from >= param(1)
			lkh = int2 - int1;
		else 
			lkh = int1 + int2;
		end
	end
case 'binomial'  % param(1) = p, param(2) = n 
	p = param(1); n = param(2);	
	lkh = 0;
	for i = from:to
		lkh = lkh + nchoosek(param(2),i)*p^i * (1-p)^(n-i);
	end
end	
