function showbel(toshowlist, showform, numberformat)
%SHOWBEL shows belief potentials
% it prints a belief after another belief
% for each belief information includes
%   a belief's id_number
%   the list of variables in the domain
%   a 2-D matrix contains focal sets in the coordinate forms
%       each elem is represented by a coordinate vector
%       each foset ends by a row of -1s
%       the last column contains masses
% VARIABLE variable cell array
% BELIEF  belief cell array
% FRAME   frame cell array
% TOSHOWLIST: list of belief ids to be shown
% showform -- a variable that can take either 
% showform = 'label' show element's label (this option is default)
% showform = 'code' show element's code or index

if nargin==1
    showform = 'label';         % show labels of elems; otherwise it shows code
    numberformat = 'fixed';     % default is fixed format; otherwise use scientific
elseif nargin==2
    numberformat = 'fixed';
end

global BELIEF VARIABLE FRAME;                     % MOFI ATTRIBUTE STRUCTURE;

allbelnumbers=[BELIEF(:).number];
allvarnumbers=[VARIABLE(:).number];
indx_bel=extfind(toshowlist, allbelnumbers);
indx_bel=indx_bel(find(indx_bel));          % remove 0 from indx_bel
varref=varreference(allvarnumbers);

for counter=1:length(indx_bel)
    currbel_indx=extfind(toshowlist(counter), allbelnumbers);
    indxvec=extfind(BELIEF(currbel_indx).domain_variable, varref.nums);
    cardvec=varref.card(indxvec);
    framesize=prod(cardvec);
    bitvmat=int2bitv(BELIEF(currbel_indx).bp(1).form(1).focal_sets, framesize);
    masses=BELIEF(currbel_indx).bp(1).form(1).values;
    var1.nums=BELIEF(currbel_indx).domain_variable; 
    var1.card=cardvec;
    crdnfoset = bvec2set(bitvmat, var1);
    [nrows, ncols]=size(crdnfoset);
    lastcol=zeros(nrows, 1);
    separation_indx=find(crdnfoset(:,1)==-1);


    lensep=length(separation_indx);
    if lensep==1
        acopy_sepa_indx=1;
    else
        acopy_sepa_indx=[1 separation_indx(1:lensep-1)']';
    end
    masspositions=floor((separation_indx+acopy_sepa_indx)./2);
    lastcol(masspositions)=masses;
    
    if strncmp(showform, 'code', 4)
        charmatfoset=num2str(crdnfoset);
    else
        celllabels=code2label(crdnfoset, BELIEF(currbel_indx).domain_variable);
        charmatfoset=cat(2,celllabels{:}); 
    end
    
    tnrows = length(lastcol);
    if strcmp(numberformat, 'scientific')
        lennumstr = length(sprintf('%e', lastcol(1)));
    else
        lennumstr = length(sprintf('%.5f', lastcol(1)));
    end
    charmatmasse = repmat(' ',[tnrows lennumstr]);              % '%e' format requires 13 places; former length=7
    if strcmp(numberformat, 'scientific')
        for i=1:tnrows
            if ~(lastcol(i)==0)
                charmatmasse(i,:)=sprintf('%e', lastcol(i));     % scientific format
            end
        end
    else
        for i=1:tnrows
            if ~(lastcol(i)==0)
                charmatmasse(i,:)=sprintf('%.5f', lastcol(i));   % fixed format with 5 places after dec
            end
        end
    end
    sep_col=repmat('| ', size(lastcol));
    charmat=cat(2, charmatfoset, sep_col, charmatmasse);
    
    [nrows, ncols]=size(charmat);
    sep_line=repmat('-',[1, ncols]);        % separation line consists of dashes
    for i=1:lensep
        charmat(separation_indx(i),:)=sep_line;
    end

    fosetsizes = (separation_indx - acopy_sepa_indx)-1; % interval between two sep lines
    tautoset_indx = find(fosetsizes==framesize);
    if ~isempty(tautoset_indx)
        lowerrow = separation_indx(tautoset_indx(1)-1)+1;
        upperrow = separation_indx(tautoset_indx(1))-1;
        rowtokeep = masspositions(tautoset_indx(1));
        rowstoremove = [lowerrow:rowtokeep-1 rowtokeep+1:upperrow];
        tautorow = charmat(rowtokeep,:);
        lefttomaspos = find(tautorow=='|');
        if ~isempty(lefttomaspos)
            tautostr = repmat('*', [1 lefttomaspos-1]);
            tautorow(1:lefttomaspos-1) = tautostr(:);
            charmat(rowtokeep,:) = tautorow(:);
        end
        charmat(rowstoremove,:)=[];
    end

    if strcmp(numberformat, 'fixed')
        tmp_col=charmat(:,ncols);               % last column 
        zero_indx=find(tmp_col=='0');
        tmp_col(zero_indx)=' ';
        charmat(:,ncols)=tmp_col;
    end
    
    dom_var_indx=extfind(BELIEF(currbel_indx).domain_variable, allvarnumbers);
    dom_var_indx=dom_var_indx(find(dom_var_indx));

    lendomvar=length(dom_var_indx);
    header=cell(1, lendomvar);
    allframeids = [FRAME(:).number];
    
    for i=1:lendomvar
        frameid = VARIABLE(dom_var_indx(i)).frame;
        frameindx = extfind(frameid, allframeids);
        if (frameindx==0)
            error('A undefined frame is encountered')
        end
        [nrofelems, lenofelem]=size(FRAME(frameindx).elements);
		if strncmp(showform, 'code', 4)
            lenofelem =2;
        end
        tmpheader = repmat(' ', [1, lenofelem+1]);
        shortname = VARIABLE(dom_var_indx(i)).short_name;
        lenshortname = length(shortname);
        minlen = min(lenshortname, lenofelem);
        tmpheader(1:minlen)=shortname(1:minlen);
        header{i}=tmpheader;
    end
    
    charheader=cat(2, header{:}, '| mass');

    fprintf('Belief # %d\n', currbel_indx)
    strvcat(charheader, sep_line, charmat)
end