function nrofbelief=uil2bm(uilfilename, bmfilename)
% UIL2BM converts belief data from the Unified Interface Language to the format that BM accepts
% uilfilename: is the name of the file (text) that contains the data in UIL format
% bmfilename: is the file name for the file that contains converted data
% nrofbelief: number of beliefs in the converted data

global VARIABLE FRAME BELIEF;

stmt_cells = local_readstmt(uilfilename);
stmt_counter = length(stmt_cells);

uilrelcount = 0;
uilfsmcount = 0;

bmvarcount = 0;
bmfracount = 0;
bmbelcount = 0;

frametype = cell(1,7);
frametype{1}='logical';
frametype{2}='binary';
frametype{3}='class unordered';
frametype{4}='class ordered';
frametype{5}='integer';
frametype{6}='interval';
frametype{7}='continuous';

for i=1:stmt_counter
	thestatement=stmt_cells{i};
	if strncmp(thestatement, 'DEFINE VARIABLE', 15)
        thestatement(1:16)=[];
        readcells = readwords(thestatement);
        nofcells = length(readcells);
        if (nofcells == 1) & isempty(readcells{1})
            error('No variable is defined.');
        elseif (nofcells > 1) & isempty(readcells{nofcells})
            readcells(nofcells)=[];
        end
            
        nrofword = length(readcells);
        if nrofword <= 1
            error('Illegal variable definition.')
        end
        
        bmfracount=bmfracount+1;
        bmframe(bmfracount).number = bmfracount;
        bmframe(bmfracount).full_name = sprintf('Frame%u', bmfracount);
        bmframe(bmfracount).short_name = bmframe(bmfracount).full_name(1);
        bmframe(bmfracount).type = frametype{4};
        bmframe(bmfracount).cardinality = nrofword -1;
        bmframe(bmfracount).elements = strvcat(readcells{2:nrofword});
        
        bmvarcount = bmvarcount+1;
        bmvariable(bmvarcount).number = bmvarcount;
        bmvariable(bmvarcount).full_name = readcells{1};
        bmvariable(bmvarcount).short_name = readcells{1}(1);
        bmvariable(bmvarcount).attribute = [];
        bmvariable(bmvarcount).index_value = [];
        bmvariable(bmvarcount).frame = bmfracount;
        
    elseif strncmp(thestatement, 'DEFINE RELATION', 15)
        thestatement(1:16)=[];
        readcells = readwords(thestatement);
        nofcells = length(readcells);
        if (nofcells == 1) & isempty(readcells{1})
            error('No relation is defined.');
        elseif (nofcells > 1) & isempty(readcells{nofcells})
            readcells(nofcells)=[];
        end
        
        nrofword = length(readcells);
        if nrofword <= 1
            error('Illegal variable definition.')
        end

        uilrelcount=uilrelcount+1;
        uilrelations(uilrelcount).name = readcells{1};
        uilrelations(uilrelcount).domain = readcells(2:nrofword);

        listofindx = extstrmatch(uilrelations(uilrelcount).domain, {bmvariable(:).full_name});
        if ~isempty(find(listofindx==0))
            uilrelations(uilrelcount).domain
            {bmvariable(:).full_name}
            listofindx
            error('A variable in the domain is not defined.')
        end
        listofvarid = [bmvariable(listofindx).number];
        uilrelations(uilrelcount).domvarid = listofvarid;

    elseif strncmp(thestatement, 'SET VALUATION', 13) | strncmp(thestatement, 'COMBINE VALUATION', 17)
        if strncmp(thestatement, 'SET VALUATION', 13)
            thestatement(1:14)=[];
        else
            thestatement(1:18)=[];
        end
        strtfspos = find(thestatement == '{');
        finifspos = find(thestatement == '}');
        if ~(length(strtfspos)==length(finifspos))
            error('Mismatch numbers of openning and closing brackets.')
        elseif ~isempty(find(strtfspos > finifspos))
            error('Mismatch positions of brackets.')
        end
        
        wordpos = read1word(thestatement,1);  % read the first word
        relname = wordpos.word;                 % take the name of relation
        rel_id = strmatch(relname, {uilrelations(:).name}, 'exact');     % determine relation_id from relation_name
        if ~isempty(rel_id)
            nvarindom = length(uilrelations(rel_id).domain);        % nr of vars in domain
            thedomain = uilrelations(rel_id).domvarid;              % the list of var_ids in the domain
            theframes = cell(length(thedomain));
            for i=1:length(thedomain)           % extract frames
                theframes(i) = {bmframe(bmvariable(thedomain(i)).frame).elements};
            end
        else                                    % belief on one variable
            varname = wordpos.word;
            var_indx = strmatch(varname, {bmvariable(:).full_name}, 'exact');    % look for variable id
            if ~isempty(var_indx)
                nvarindom = 1;          % consists of 1 var 
                thedomain = bmvariable(var_indx).number;
                theframes = {bmframe(bmvariable(var_indx).frame).elements};
            else
                fprintf('Relation name: %s\n', relname);
                error('Valuation name is not defined.')
            end
        end
        
        noffosets = length(strtfspos);          % each foset is begin with '{'
        massvalues = zeros(1, noffosets);       % to store the masses
        for i=1:noffosets
            wordpos = read1word(thestatement, finifspos(i)+1);
            if ~isempty(wordpos)
                massvalues(i) = str2num(wordpos.word);
            else
                warning('Can find mass after a focal set.')
            end
        end
        
        bmcodefos = cell(noffosets,1);          % to store bm coding of focal sets
        for i=1:noffosets
            tmp_strfoset = thestatement(strtfspos(i):finifspos(i));
            strelems = readwords(tmp_strfoset);
            nofcells = length(strelems);
            if (nofcells == 1) & isempty(strelems{1})      % empty focal set
                tmpcodefos = zeros(2*nvarindom,1);
                tmpcodefos(nvarindom+1:2*nvarindom) = repmat(-1, [nvarindom, 1]);
                nrows = 2;
                ncols = nvarindom;
            else                                            % normal focal set
                if (nofcells > 1) & isempty(strelems{nofcells})    % the last word is empty
                    strelems(nofcells)=[];
                end
                nrofword = length(strelems);
                tmpcodefos = zeros(nrofword+nvarindom, 1);
                for j=1:nrofword            % convert elem given as strings to their code
                    theword = strelems{j};

                    indxfrm = mod(j, nvarindom);
                    if indxfrm==0
                        indxfrm=nvarindom;
                    end
                    thecode = strmatch(theword, theframes(indxfrm), 'exact');
                    if ~isempty(thecode)
                        tmpcodefos(j) = thecode(1);
                    else
                        fprintf('thecode is empty\n');
                        fprintf('theword = %s\n', theword);
                        fprintf('The frame is \n');
                        
                        tmplengcell = length(theframes(indxfrm));

                        for k=1:tmplengcell
                            theframes{k}
                        end
                        
                        error('A value of variable is not defined. There is potentially a conflict in UIL declaration.')
                    end
                end
                tmpcodefos(nrofword+1:nrofword+nvarindom)=repmat(-1, [nvarindom, 1]); % separation line
                nrows = nrofword/nvarindom + 1;
                ncols = nvarindom;
            end
            
            if ~(floor(nrows)==nrows)
                fprintf('Foset number = %d\n', i);
                fprintf('Number of words = %d\n', nrofword);
                fprintf('Number of variables = %d\n', nvarindom);
                error('Number of rows is not an integer.');
            end
            bmcodefos{i}=reshape(tmpcodefos, [ncols, nrows])';
        end
        
        fosets = cat(1, bmcodefos{:});
        
        % prepare data for set2intmat.m
        varsum.nums = [bmvariable(:).number];
        frmnums = [bmvariable(:).frame];
        allfrmnumbers = [bmframe(:).number];
        frmindx = extfind(frmnums, allfrmnumbers);
        varsum.card = [bmframe(frmindx).cardinality];
        
        if sum(massvalues) < 1  % check if there is a mass to tautology
            remmass = 1 - sum(massvalues);
            thisfrmcard = varsum.card(extfind(thedomain, varsum.nums));
            thistautofset = thisfrmcard+1;
            thisseprow = repmat(-1, 1, length(thedomain));
            thisaddfoset = cat(1, thistautofset, thisseprow);
            fosets = cat(1, fosets, thisaddfoset);
            massvalues = cat(2, massvalues, remmass);
        end

        bmfs.domain = thedomain;
        bmfs.indices = fosets;
        bmfs.values = massvalues';

        dmindx = extfind(thedomain, varsum.nums);
        if ~isempty(find(dmindx==0))
            thedomain
            varsum.nums
            dmindx
            error('A variable in domain not defined');
        end
        dmcard = varsum.card(dmindx);

        bmbelcount = bmbelcount+1;
        bmbelief(bmbelcount).number = bmbelcount;
        bmbelief(bmbelcount).full_name = [];
        bmbelief(bmbelcount).short_name = [];
		bmbelief(bmbelcount).disposable = 0;		
		bmbelief(bmbelcount).agent = 'You';		  
		bmbelief(bmbelcount).time = 1;			
		bmbelief(bmbelcount).ec = [];
		bmbelief(bmbelcount).domain_variable = thedomain;
		bmbelief(bmbelcount).domain_cardinality = dmcard;
		bmbelief(bmbelcount).conditioning_variable = [];
        bmbelief(bmbelcount).bp(1).form(1).type = 'm'; 
        bmbelief(bmbelcount).bp(1).form(1).ordered = 1;                             % 1 = ordered by masses
		bmbelief(bmbelcount).bp(1).form(1).focal_sets = set2intmat(varsum, bmfs);   % for 4 focal sets address in bin form
		bmbelief(bmbelcount).bp(1).form(1).values = bmfs.values;                    % masses given
    end
end

VARIABLE = bmvariable;
FRAME = bmframe;
BELIEF = bmbelief;

[commandsave errmessage] = sprintf('save %s VARIABLE FRAME BELIEF', bmfilename);
eval(commandsave);

nrofbelief = length(BELIEF);
%%%%%%%%%%%%%%%% end of UIL2BM

function stmtcellarray = local_readstmt(readfname)
% LOCAL_READSTMT reads lines from READFNAME and put statements in an cell array STMTCELLARRAY
% input: readfname is a text file name (default extension .txt)
% output: stmtcellarray a cell array in each cell is a statement

isfullname = ~isempty(find(readfname=='.'));
if isfullname
	fid = fopen(readfname, 'rt');
else 
    readfname = [readfname '.txt'];
    fid = fopen(readfname, 'rt');
end
if isempty(fid)
    fprintf('Could not find file %s', readfname);
    error('Error in open file.'); 
end

comment_sym='#';
endstmt_sym=';';

stmt_len  = 10000;                       % the max length of a statement
maxofstmt = 1000;

tmp_stmt = repmat(' ', stmt_len);       % to store the current statement

stmtcellarray = cell(maxofstmt,1);         % cell array containing statements
stmt_counter = 0;
tmpstmt_pos = 1;                       % the next position

% read uil file line by line
while ~feof(fid)
    tline = fgetl(fid);
    if isempty(tline) | strncmp(tline, comment_sym, 1)      % the line is a comment line or empty line
        continue
    end
    
    tline = local_cleanup(tline);       % remove unnecessary (successive) blanks, comments
    smcolon_pos = find(endstmt_sym == tline);   % find positions of ';'
    lentline=length(tline);
    seensmcolon = 0;

    if isempty(smcolon_pos)     % no ';' in the line, copy into tmp_stmt
        seensmcolon = 0;        % the statement is not finished 
        tmp_stmt(tmpstmt_pos: (tmpstmt_pos + lentline - 1))=tline;
        tmpstmt_pos = tmpstmt_pos + lentline;
    else
        nrofcmcolon = length(smcolon_pos);
        startpos = [1, smcolon_pos+1];              % starting position of a statement
        endinpos = [smcolon_pos-1, length(tline)];  % ending position 
        
        for i = 1:nrofcmcolon
            if seensmcolon == 0         % previous line does not complete the statement
                if smcolon_pos(i) > 1   % symbol ';' does not occurs at the beginning of line
                    tmp_stmt(tmpstmt_pos:(tmpstmt_pos + smcolon_pos(i) - 2))=tline(1:smcolon_pos(i)-1);
                    tmpstmt_pos = tmpstmt_pos + smcolon_pos(i)-1;
                    seensmcolon=1;
                end
                stmt_counter=stmt_counter+1;        % add the statement into the cell array
                stmtcellarray(stmt_counter) = {tmp_stmt(1:tmpstmt_pos-1)};
                tmpstmt_pos = 1;
            else    % seensmcolon == 1 when previous statement has been complete
                if startpos(i) < endinpos(i) 
                    stmt_counter=stmt_counter+1;        % add the statement into the cell array
                    stmtcellarray(stmt_counter) = {tline(startpos(i):endinpos(i))};
                end
            end
        end
        
        if startpos(nrofcmcolon+1) < endinpos(nrofcmcolon+1)  % last ';' does not occur at line's end
            len = endinpos(nrofcmcolon+1) - startpos(nrofcmcolon+1)+1;
            tmp_stmt(1:len)=tline(startpos(nrofcmcolon+1): endinpos(nrofcmcolon+1));
            seensmcolon=0;
            tmpstmt_pos=len+1;
        end
    end
end

status = fclose(fid);       % close the file
stmtcellarray(stmt_counter+1:maxofstmt)=[];        % remove unused cells

%%%%%%%%%%%%%%% end of LOCAL_READSTMT

function outline = local_cleanup(inline)
% LOCAL_CLEANUP cleans the inline
% Input: inline -- a line of text
% Output: outline -- cleaned line

comment_sym = '#';

len = length(inline);
markline = zeros(1, len);

comment_pos = find(inline == comment_sym);
if ~isempty(comment_pos)
    inline(comment_pos(1):end)=[];
end

len = length(inline);

spacepos = isspace(inline);         % replace tab by space
inline(spacepos)=' ';

blankbefore = 1;
for i = 1:len
    if inline(i)==' '
        if blankbefore == 1
            markline(i)=1;
        else
            blankbefore = 1;
        end
    elseif (inline(i) == ';' | inline(i) == ',')
        blankbefore = 1;
    else
        blankbefore = 0;
    end
end
pos_vec=find(markline);
if ~isempty(pos_vec)
    inline(pos_vec)=[];
end
outline = inline;
%%%%%%%%%% end of LOCAL_CLEANUP