function run_VBS(modality,parameter) 
% interface between the GUI and the VBS 

% modality how it is used
% 	= 'from_EVN' if called in EVN menu
% 	= 'from_Tools_Menu' if called in Calculator Menu

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global nbr_bit_per_word
global node pin_node link pin_link  need_uiresume  flagstr scaling_coef
global gui_position  figure_data  flags  help_level

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% VBS author uses capital convention for his global, and gui author uses lower char.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global frame structure mapping attribute variable belief
global FRAME STRUCTURE MAPPING ATTRIBUTE VARIABLE BELIEF QUERY

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% I rebuld all the index for objects in lower cases 
% by eliminating those with an empty number
% hence there is no gap in the object and none is empty (avoid crash)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pack_index 

switch modality

case 'from_Tools_Menu'
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% You ask ONE propagation for a set of bba on a set of variable, nothing memorized
	% for future: ask another projection with same bba. XXXX
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% get the figure
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	hd_bel_fig = figure(...
	'name','Run VBS Window');
	set(gcf,'position',gui_position(:,5))
	ind_fig = length(figure_data) + 1;
	
	figure_data(ind_fig).name = 'Run VBS Window';
	figure_data(ind_fig).handle = hd_bel_fig;
	figure_data(ind_fig).position = gui_position(:,5);
	
	hd_help = forced_display_helps(0,36);
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% asking which beliefs are to be used as input
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	hd_bel_list = listing_existing('belief', gui_position(:,6), hd_bel_fig,[]);
	set(hd_bel_list(2), 'Enable','on', 'Max',1000,'Value',[]); % make it multi selection
	
	text = 'Select the beliefs that are to be combined by the VBS';
	[choice] = text_with_n_buttons(hd_bel_fig, text, {'OK','Cancel'}, [1]);
	pos_sel = get(hd_bel_list(2),'Value');
	
	if choice == 2 | length(pos_sel) == 0
		delete(hd_bel_fig)
		delete(hd_help)
		figure_data(ind_fig) = [];
		return
	end

	list_bels_number = pin_manipulation('belief','pos2num',pos_sel);
	list_bels_index  = pin_manipulation('belief','pos2ind',pos_sel);
		
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% check belief domain NCI 
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	ok = check_belief_nci(list_bels_index);
	if ok == 0
		% there is some continuous integers frame, I refuse
		hdw = warndlg('No continuous no integers variables allowed. I cancel.')
		uiwait(hdw)
		delete(hd_bel_fig)
		delete(hd_help)
		figure_data(ind_fig) = [];
		return
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% create m if absent + transform conditonal_beliefs into joint_beliefs 
	% if there are conditional. done as in GBT/DRC algo. NCI
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	[list_bels_number, list_bels_index] = building_joint_from_condit(list_bels_number)
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% asking on which variables beliefs are to be marginalized
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	list_poss_var_number = unique([belief(list_bels_index).domain_variable]);
	% no need for conditonal as only for undirected network
	list_poss_var_index = pin_manipulation('variable','num2ind',list_poss_var_number);
	for i = 1:length(list_poss_var_index )
		list{i} = variable(list_poss_var_index(i)).full_name;
	end

	handle_list = listing_existing('general',gui_position(:,7),hd_bel_fig,list);
	set(handle_list(1),'string','Variables');
	select_list(handle_list,'on','many');
	
	text = 'Select the variables on which marginal must be computed.';
	sel_var_pos = [];
	while isempty(sel_var_pos)
		
		[choice] = text_with_n_buttons(hd_bel_fig, text, {'OK','Cancel'},[1]);		
		if choice == 2 
			delete(hd_bel_fig)
			delete(hd_help)
			figure_data(ind_fig) = [];
			return
		end
	
		sel_var_pos = get(handle_list(2),'value');
		list_var_num = list_poss_var_number(sel_var_pos);
		list_var_ind = list_poss_var_index(sel_var_pos);
	end

	delete(hd_bel_fig)
	delete(hd_help)	
	figure_data(ind_fig) = [];
	
	all_marginals = 0; % I only ask for the requested marginal
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% preparing data for VBS, = clean all OBJECTS, build new one based on list_bels_number 
	bel_ind_for_BEL = prepare_initial_VBS_request(list_bels_number);
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
case 'from_EVN'

	list_bels_number = [node(find(  strcmp({node(:).nature},'belief')   ) ).obj_num];
	list_bels_number_mem = list_bels_number;
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% create m if absent + transform conditonal_beliefs into joint_beliefs 
	% if there are conditional. done as in GBT/DRC algo. NCI
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	[list_bels_number list_bels_index] = building_joint_from_condit(list_bels_number);
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% storing node(:).obj_num_for_JVBS, needed for condit bel
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% finding node index with belief and belief number as needed by VBS
	for i = 1:length(node)
		if strcmp(node(i).nature,'belief')
			for k = 1:length(list_bels_number_mem)  % for all old numbers
				if node(i).obj_num == list_bels_number_mem(k)
					node(i).obj_num_for_JVBS = list_bels_number(k); % store new number
				end
			end
		else
			node(i).obj_num_for_JVBS = []; % not a belief
		end
	end

	% 	% control
	% 	for i = find(strcmp({node(:).nature},'belief'));
	% 		node(i).nature
	% 		node(i).obj_num
	% 		node(i).obj_num_for_JVBS
	% 	end
	
	list_var_num = belief(list_bels_index(1)).domain_variable;
	% so at least there is something in the request for a marginal	
	all_marginals = 1; % I ask for every marginals on single var
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% preparing data for VBS, = clean all OBJECTS, build new one based on list_bels_number 
	bel_ind_for_BEL = prepare_initial_VBS_request(list_bels_number)
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
case 'from_EVN_again'


	% HOW DO I GET HERE
	
	
	
	% propagation has already been performed on every singletons and you ask for marginalization on a set of var	
	list_bels_number = [node(strcmp({node(:).nature},'belief')).obj_num_for_JVBS];
	
	% no prepare_initial_VBS_request as not new
	list_var_num = parameter(1); % the var on which we margionalize
	all_marginals = 1;
	
case 'from_Batch_Mode'
	list_bels_number = parameter{1};
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% create m if absent + transform conditonal_beliefs into joint_beliefs 
	% if there are conditional. done as in GBT/DRC algo. NCI
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	[list_bels_number list_bels_index] = building_joint_from_condit(list_bels_number)
	
	list_var_num = parameter{2};
	% so at least there is something in the request for a marginal	
	all_marginals = 0; % I donot ask for every marginals on single var
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% preparing data for VBS, = clean all OBJECTS, build new one based on list_bels_number 
	bel_ind_for_BEL = prepare_initial_VBS_request(list_bels_number);
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% runing the VBS, output is belief number
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% list_bels_number : those belief used (numbers)
% list_var_num     : the variable numbers on which results are marginalized

% solve computes the unique bel on a marginal made of a set of variables, 
% based on all bp in list_bels_number


% % useful for debugging purpose
% for i = 1:length(list_bels_number)
% 	BELIEF(list_bels_number(i))
% 	BELIEF(list_bels_number(i)).bp(1)
% 	BELIEF(list_bels_number(i)).bp(1).form(1)
% 	BELIEF(list_bels_number(i)).bp(1).form(1).values
% 	BELIEF(list_bels_number(i)).bp(1).form(1).focal_sets
% end

%
% THE REAL COMPUTATION IS ASKED HERE
% bel_idnum = ssolve(list_bels_number, list_var_num) if all marginals = 0
% else with the list of the idnum generated by each request 

bel_idnum = execute_vbs(list_bels_number,list_var_num,all_marginals);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% help report in command window
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(1,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n',[]);
fprintf(1,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n',[]);
fprintf(1,'\nVBS : which computation is requested?\n',[]);
fprintf(1,'Used belief objects:\nNumber Index Short_name  Full_name\n',[]);
for i = 1:length(list_bels_number)
	indexx = find_field('belief','number',list_bels_number(i));
	fprintf(1,'%5i %5i    %s\n',[list_bels_number(i),indexx,[belief(indexx).short_name,'   ',belief(indexx).full_name]]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ending restoring belief objects
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

print_message = 1;

switch modality

case 'from_Tools_Menu'
	
	ind_bel = finishing_belief_from_VBS(bel_idnum,list_bels_number);
	
	belief(ind_bel).computed = 'VBS';
	belief(ind_bel).origin = [list_bels_number' ones(length(list_bels_number),1)];
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% displaying results TOO SLOW IF MANY focal sets
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
	if length(belief(ind_bel).bp(1).form(1).values) < 1028
		results_display(ind_bel,1,1) % adapt bp form #
	end

case 'from_EVN'
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% building the marginal for every variable involved in the EVN
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	for i = 1:length(bel_idnum)
		
		ind_bel = finishing_belief_from_VBS(bel_idnum(i),list_bels_number);
		belief(ind_bel).computed = 'VBS';
		belief(ind_bel).origin = [list_bels_number' ones(length(list_bels_number),1)];		
		bel_num = belief(ind_bel).number;
		% get BetP on each unidimensional marginal as asked for EVN	
		[bel_ind,ind_bp,ind_form_betp] = build_BetP(bel_num);
		% beware: two belief obejcts can concerned the same variables.
		% assigning these results to the node that contains this variable
		
		% i am working on variable = belief(ind_bel).domain_variable
		% I look for node with variable and obj_num is that number			
		ix1 = find([node(:).obj_num] == belief(ind_bel).domain_variable); 
		% the index of node with correct number
		
		if length(ix1) == 0 
			% programming error
			BOUM
		end
		ix2 = find(strcmp({node(ix1).nature},'variable'));
		if length(ix2) ~= 1 
			% programming error
			BOUM
		end
		node(ix1(ix2)).betp_localization = [bel_ind,ind_bp,ind_form_betp,bel_num ];
	end	

case 'from_EVN_again'
	print_message = 0;
	% TO BE BUILT
	
case 'from_Batch_Mode'	
	ind_bel = finishing_belief_from_VBS(bel_idnum,list_bels_number);
end

if print_message
	% belief(ind_bel).full_name =  ['B_VBS_',num2str(Belief(ind_BEL(i)).number)];
	% I store the number of BELIEF in belief full_name
	% bel_idnum are the numbers of the BELIEF created by VBS
	
	for i = 1:length(bel_idnum)
		for j = 1:length(belief)		
			if strcmp(belief(j).full_name,['B_VBS_',num2str(bel_idnum(i))])				
				fprintf(1,'Results stored in belief number = %5i, name = %s\n',[belief(j).number,belief(j).full_name]); 
				fprintf(1,'Built on variables\nNumber Index  Short_name   Full_name\n',[]);
				for i = 1:length(belief(j).domain_variable)
					indexx = find_field('variable','number',belief(j).domain_variable(i));
					fprintf(1,'%5i   %5i    %s\n', [belief(j).domain_variable(i), indexx,...
					[variable(indexx).short_name,'   ',variable(indexx).full_name]]);
				end
			end
		end
	end
	fprintf(1,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n',[]);
	fprintf(1,'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n',[]);
end

