function [log_expr,weight,cancel_it] = ask_one_value_log_expr(ibel,bp_handle_fig,h_expr,...
last_log_expr,list_typ_var,list_ind_var,button_operator,nw)
% input for the logical expression that builds a focal set
% ibel 				= index of the belief object
% bp_handle_fig 	= handle of figure where data are entered
% h_expr 			= handle of box for logical epxression display

% log_expr			= the logical expression entered
% weight 			= the weight allocated to that focal set
% cancel_it			= 0 if OK, = 1 if operation canceled

% Needed : button list of variables for that belief(ibel) (and their possible values if not logical)
% button list with operators and, or, not, impl, equiv, (, ), end

% at exit, we MUST produce (an expression and a weight)   or   flag_bp_cancel = 1
% or a wuit request (thus with empty mass exprs but flag_bp_cancel = 0)
%
%   Version : Jan 19, 03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global gui_position
global stored_text 
global expr
global flag
global button_size
global keyhit

log_expr = [];
weight = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% loop repeated till we have a OK expr + OK weight
% or we say cancel this weight, or we quit (so repeat if OK not pushed twice (for expr and for weigth))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
cancel_it = 0;
go_go_on = 1;
while go_go_on
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% the logical expression and weight request
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
	empty_expr = 1;
	go_on = 1;
	flag_expr_ok = 0;   % 0 = cancel expr, 1 = OK
	flag_mass_ok = 0;   % 0 = weight not OK, 1 = OK, 

	set(h_expr,'string',last_log_expr);
	if ~isempty(last_log_expr)
		% you must correct your logical entry, it was not ok
		empty_expr = 0;
		expr = last_log_expr;
	end	
	
	while go_on % loop for entering log expr
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% I wait for your selection
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		flag = [0 0];
		keyhit = 0;
		
		uiwait(bp_handle_fig)
		
		% fprintf(1,'\nflags = %3i %3i keyhit = %3i %s',[flag keyhit get(h_expr,'string')]);
		% uiresume from return in expression box
		if flag(2) == 0 & keyhit == 1
			% generated by clb on return in expr box, and not by button
			flag(2) = 3;
			flag(1) = 1; % expression is OK
		end
	
		% uiresume from button panel
		% examining the flag(2) which state which panel was selected 1 = log, 2 = var, 3 = control
		if flag(2) == 3 % the control panel
			switch flag(1) 
			case 1 % Expr. OK
				flag_expr_ok = 1;
				flag_mass_ok = 0; 
				flag(2) = 1;
				flag(1) = 10; % so I end expression
			case 2 % Clear Expr.
				flag_expr_ok = 0; 
				set(h_expr,'String','');
				expr = '';
				flag(2) = 0; % so I skip switch logvar and ask for new expr	
				empty_expr = 1;
			case 3 % Quit
				cancel_it = 1;
				flag(2) = 0; % so I skip switch logvar and ask for new expr	
				go_on = 0; % and I leave while go_on
				empty_expr = 1; % so I skip the weight querry too
				go_go_on = 0;
			end
		end
		% 	flag_expr_ok = 0;   % 0 = cancel expr, 1 = OK
		% 	flag_mass_ok = 0;   % 0 = mass not OK, 1 = OK, 
		
		if cancel_it == 0
			
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%
			% list of values in a listbox
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			% flag comes from callback of button_list 
			butt   = flag(1);	% the selected button
			logvar = flag(2);	% the pushed button is in log operator list (1) or in var list (2)
			
			expr = get(h_expr,'String');
			expr = [expr, ' ']; % to be sure it is not empty 
			
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% logvar 		
			switch logvar 
			
			case 1 % logical operator		
				if butt == 10  % end of expression input 
					if length(deblank(expr)) 
						% thus I typed data other than blank
						empty_expr = 0;
						flag_expr_ok = 1;
						go_on = 0;
					else
						empty_expr = 1;
						flag_expr_ok = 0;
						go_on = 1;
					end
				elseif butt <= 7
					expr = [expr,' ',button_operator{butt}];
					set(h_expr,'String', expr)	
					empty_expr = 0;
				elseif butt == 8
					expr = [expr, ' ','1'];
				elseif butt == 9
					expr = [expr, ' ','0'];
				end
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% logvar 
			case 2 % variable
				empty_expr = 0;
				
				%%%%%%%%%%%%%
				%%%%%%%%%%%%% list_typ_var(butt)
				% 1 for logical, 2 for integers, 3 for other
				%%%%%%%%%%%%% list_typ_var(butt)
				switch list_typ_var(butt)
				case 1 % logical
					% easy, just keep it,	elem = {'F','T'};
					%%%%%%%%%%%%% list_typ_var(butt)
				case 2 % numerical
					num_fr = variable(list_ind_var(butt)).frame;
					index_fr = pin_manipulation('frame','num2ind',num_fr);
					for i = frame(index_fr).min: frame(index_fr).max
						elem{i} = num2str(i);
					end
					%%%%%%%%%%%%% list_typ_var(butt)
				case 3 % symbolic
					num_fr = variable(list_ind_var(butt)).frame;
					index_fr = pin_manipulation('frame','num2ind',num_fr);
					elem = frame(index_fr).elements;		
				end % switch list_typ_var(butt)
				%%%%%%%%%%%%% list_typ_var(butt)
				%%%%%%%%%%%%%
				
				if list_typ_var(butt) > 1 
					% not for logical variable
					% the listbox with the possible values of the frame of the variable
					hd_elem(1) = uicontrol(...
					'Parent',bp_handle_fig, ...
					'Position',gui_position(:,17), ...
					'Style','listbox', ...
					'string',elem,...
					'max',1000,...
					'value',[],...
					'callback','uiresume;');
					
					text = 'Select the possible values by multi selection. Then click Go_On';
					choice = text_with_n_buttons(bp_handle_fig, text, {'Go_On'}, [1]);
					
					sel_elem = get(hd_elem,'value');
					delete(hd_elem);		
					
				end % if list_typ_var(butt) > 1 
			
				%%%%%%%%%%%%%
				%%%%%%%%%%%%%  list_typ_var(butt)
				switch list_typ_var(butt)
				case 1 % logical variable
					expr = [expr,' ',variable(list_ind_var(butt)).short_name];
					set(h_expr,'String', expr) ;
				case 2 % numbers. I need [ 43 56 17] as a string
					text = ['(', variable(list_ind_var(butt)).short_name,' in [', num2str(sel_elem) ,'])'];	
					expr = [expr,text];
					set(h_expr,'String', expr);
				case 3 % symbols. I need {'vv','cc'}
					text = [' (', variable(list_ind_var(butt)).short_name,' in {','''',elem(sel_elem(1),:),''''];
					for i = 2:length(sel_elem)
						text = [text,',''', elem(sel_elem(i),:),''''];
					end
					text = [text,'})'];
					expr = [expr,text];
					set(h_expr,'String', expr);		
				end	% switch list_typ_var(butt)
				%%%%%%%%%%%%% list_typ_var(butt)
				%%%%%%%%%%%%%
				
			end % switch logvar
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% logvar 
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
		end % cancel_it
	end % of while go_on layout of instruction is not Ok with comd I, it get lost with nested switch	

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% entering the weight input area
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	if empty_expr ==  0 & cancel_it == 0 % thus there is a focal set = logical expression and I need its weight
		log_expr = expr;
		
		% for entering the weight
		h_mass(1) = uicontrol(...
		'Parent',bp_handle_fig, ...
		'Position',gui_position(:,21)', ...
		'Style','edit', ...
		'HorizontalAlignment','left', ...
		'String','0',...
		'CallBack','callback_CR_edit;uiresume;');
		
		text_mass = ['Enter ', nw];
		
		h_mass(2) = uicontrol(...
		'Parent',bp_handle_fig, ...
		'Position',gui_position(:,22)', ...
		'Style','text', ...
		'HorizontalAlignment','left', ...
		'String',text_mass);
		
		you_are_stupid = 1; % to avoid hitting out of control or using return
		while you_are_stupid	
			
			flag = [0 0];
			keyhit = 0;
			figure(bp_handle_fig);
			uiwait(bp_handle_fig)  % input weight, entered by either return or pushbuton OK
			
			if keyhit == 1 % I hit return, hence weight entered
				go_go_on = 0;
			end
		
			if go_go_on == 0 % i  hit return in h_mass(1)
				flag_bp_cancel = 0;	
				you_are_stupid = 0;
			else % I select a control button
				% examining the flag(2) which state which panel was selected 1 = log, 2 = var, 3 = control
				if flag(2) == 3 % the control panel
					you_are_stupid = 0;
					switch flag(1) 
					case 1 % weight OK
						flag_bp_cancel = 0;
						go_go_on = 0;
					case 2 % Cancel Expr.
						set(h_expr,'String','');
						expr = '';
					case 3 % Cancel Pot.
						flag_bp_cancel = 1; 
						go_go_on = 0;
					case 4 % Quit
						flag_bp_cancel = 0;	
						go_go_on = 0;
					end
				else % i hit another set of buttons, how stupid
					text = 'Hit a button in CONTROL list. To proceed click Go_On';
					choice = text_with_n_buttons(bp_handle_fig, text, {'Go_On'}, [1]);
					% insults and repeat uiwait			
				end
				% 	flag_expr_ok = 0;   % 0 = cancel expr, 1 = OK
				% 	flag_mass_ok = 0;   % 0 = mass not OK, 1 = OK, 
				
			end % if go_go_on == 0
		end % while you_are_stupid	
		% the saved mass
		weight = str2num(get(h_mass(1),'String'));
		delete (h_mass);
	end % if empty_expr ==  0

end % while go_go_on
