function showing_bp_selected(ibel,index_bp,out_option)
% display the masses with focal set for belief(ibel).bp(index_bp) when focal selected
% belief is not and may not be modified
% a list with one mass and a list of elemetn for focal set, then next mass and ...
% not very user friendly, not yet ready for cont/int variables XXXX

% out_option = 0 : everything on the screen
% out_option = n > 0 : saved on a file TBM_Potential_n in User_Data folder

%
%   Version : Jan 11,03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global nbr_bit_per_word  
global gui_position  belief
global max_pc_cumul_mass  bbm_display_order pathname2TBMLABvers
global Which_Courier   figure_data flags

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% verifying we have a correct bp (mass) and focal sets selected
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ok = 0;
if length(belief(ibel).bp(index_bp)) % bp exists
	n_form = length(belief(ibel).bp(index_bp).form);
	if n_form    
		cardinal = prod(belief(ibel).domain_cardinality);
		m_form = find(strcmp( { belief(ibel).bp(index_bp).form(:).type} ,'m'));		
		
		% verifying m extsis and can be displayed, if not user must build m himself
		% m does not exist because cardinality HUGE!!! see GBT where I only have BetP
		if isempty(m_form)
			hdw = warndlg(['The m form does not exist for this potential object.',...
			'Build it yourself if you really want to display it.',...
			'The cardinality is ',num2str(cardinal),'.']);
			uiwait(hdw)
			return
		end
	
		vect_m = belief(ibel).bp(index_bp).form(m_form).values;
		switch belief(ibel).bp(index_bp).form(m_form).representation		
		case 'selected'
			vect_foc = belief(ibel).bp(index_bp).form(m_form).focal_sets;
			ok = 1;
		case 'ordered' % when ordered, order binary and foc elem rebuilt
			% accept only cardinalmity <= 50 (over 10 is already VERY long)
			vect_foc = [0:pow2(cardinal)-1]';
			ok = 1;
		end
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% localizing empty set and normalizing masses
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ok 
	% nbr full words, nbr remaining in next word
	[nw rm] = words_for_set(cardinal);
	
	% universe is a set where every bit is 1
	universe(1:nw)=pow2(nbr_bit_per_word)-1;
	universe(nw+1)=pow2(rm)-1;
	
	% finding the mass to the empty set
	ind_es = locating_a_set(zeros(1,nw+1) ,vect_foc);
	% beware unsused bits are 0 in the long number
	
	% norm_factor is 1 - mass(empty_set)
	switch length(ind_es)
	case 0
		mass0 = 0;
		norm_factor = 1;
	case 1
		mass0 = vect_m(ind_es);
		norm_factor = 1 - mass0;
		vect_m(ind_es) = 0;
	otherwise
		ok = 0;
		% beware of not twice the same focal sets !!!
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the window if out_option = 0, the file fid if out_option > 0
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if out_option == 0
	text = ['Belief : ',belief(ibel).full_name,' Up tp ',num2str(max_pc_cumul_mass),'% of normalized masses'];		
	hd_fig = figure('name',text);	
	set(hd_fig,'position',gui_position(:,5));
	ind_fig = length(figure_data) + 1;
	figure_data(ind_fig).name = text;
	figure_data(ind_fig).handle = hd_fig;
	figure_data(ind_fig).position = gui_position(:,5);
	% put its menus
	make_fig_menu(ind_fig,1,'File',0,'','on');
	make_fig_menu(ind_fig,2,'Quit',1,'general_clb(1)','on');
	menu_manager('create',ind_fig)	
else
	% not working as yet
	add_saved_file = fullfile(pathname2TBMLABvers,...
	'User_Data',['TBM_Potential_',num2str(out_option)] );	
	fid = fopen(add_saved_file,'wt');	
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% preparing the display of the focal elements
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ok	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% preparing the display of the focal elements
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% sorting the mass so the list start with the largest ones	
	switch bbm_display_order
	case 'by magnitude' % you get largest bbm first
		[useless, ind_sort_M] = sort(-vect_m); 
		% putting - so that the largest are in front
	case 'as input' % you get bbm as they are input
		ind_sort_M = [1:size(vect_m,1)];
	end

	[var_name_list, element_list, element_numbers, useless] = ...
	creating_elements(belief(ibel).domain_variable);
	
	% line is a line separator
	[nr nc] = size(element_list);
	ncl = max(nc,50); % 50 to be sure the line is long enough to be seen easily
	line(1:ncl) = '-';
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% deciding how many focal sets to display (total mass <= max_pc_cumul_mass/100)
	% default = 95
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% normalized masses
	nM = vect_m/norm_factor;
	if length(ind_es)
		vect_m(ind_es) = mass0;
	end
	% from here nM is normalized m, vect_m is unnormalized m
	% .95 is the threshold of the masses displayed
	if out_option == 0
		limit_totM = min(max_pc_cumul_mass/100,.99999);
	else
		limit_totM = .99999;
	end

	totM = 0; ii = 0;
	while totM <= limit_totM
		ii = ii + 1;
		i = ind_sort_M(ii);
		totM = totM + nM(i);
		cumulM(i) = totM;
	end
	nbr_foc = ii;
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% preparing a big block of data to be displayed or saved
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	ibb = 0;		
	for ii = 1:min(200,nbr_foc)
		i = ind_sort_M(ii);
		% list of bits on in focal sets vect_foc(i,1:nw+1)
		% vect_foc is a long number
		bit_on = long_num2onbit([vect_foc(i,:)], cardinal);
		ibb = ibb + 1;
		big_block{ibb} = ['Unnormalized mass            = ',num2str(vect_m(i))];
		ibb = ibb + 1;
		big_block{ibb} = ['Normalized Mass              = ', num2str(nM(i))]; 
		ibb = ibb + 1;
		big_block{ibb} = ['percent of normalized Masses = ',num2str(100*cumulM(i))];	
		ibb = ibb + 1;
		big_block{ibb} = ['m(empty set/contradiction)   = ',num2str(mass0)];	
		ibb = ibb + 1;
		big_block{ibb} = ' ';
		
		if isfield(belief(ibel).bp(index_bp).form(m_form),'log_expr')
			if length(belief(ibel).bp(index_bp).form(m_form).log_expr) >= i
				if length(belief(ibel).bp(index_bp).form(m_form).log_expr{i})	
					ibb = ibb + 1;
					big_block{ibb} = 'Logical expression : ';			
					ibb = ibb + 1;
					big_block{ibb} = ddblank(belief(ibel).bp(index_bp).form(m_form).log_expr{i});
					ibb = ibb + 1;
					big_block{ibb} = ' ';				
				end
			end
		end
	
		if vect_foc(i,:) == universe
			ibb = ibb + 1;
			big_block{ibb} = 'Given to universe.';
		else
			ibb = ibb + 1;
			big_block{ibb} = 'Given to next focal set:';
			
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%
			% what are the elements of this focal set 
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			ibb = ibb + 1;
			big_block{ibb} = var_name_list; 
			for jbb = 1:length(bit_on)
				ibb = ibb + 1;
				big_block{ibb} = element_list{bit_on(jbb)}   ; 
			end
		end
		ibb = ibb + 1;		
		big_block{ibb} = line; 
		% this is the focal set		
	end

	size(big_block);
	for i = 1:min(300,length(big_block))
		the_big_block{i} = big_block{i};
	end

	% PROBLEM WITH THE SIZE OF BIG BLOCK, WHEN 1908 BOUM, WHEN 100 OK, SO 
	% we must find how many lines listbox accepts
	% horror it depen ds not on nbr of lines, but number of characters
	
	if out_option == 0 % data on screen
		hui = uicontrol('Parent',hd_fig, ...
		'position',gui_position(:,14),...
		'style','listbox',...
		'fontname',Which_Courier,...
		'callback','uiresume',...
		'string',the_big_block);
		go_on = 1;
		flags = [];
		flags(1) = 0;
		while go_on 
			uiwait(gcf)
			if flags(1) == 1
				go_on = 0;
			end
		end		
	else % data on the file 
		fprintf(fid,'List of masses with their focal set.\n\n',[]);		
		for i = 1:length(big_block)
			fprintf(fid,'%s\n',big_block{i});
		end
		fclose(fid);
	end % of screen or file : see out_option

else
	% I cannot do it as data not available
	text = {'Display mass not posible.',...
	'Selected basic belief masses do not exist, or several focal sets are the empty set.'}
	hd_warn = warndlg(text);
	uiwait(hd_warn)	
end

if out_option == 0
	if ishandle(hui); delete(hui); end
	delete(hd_fig)
end
