function duplicate_bel(index_which_belief)
% belief(index_which_belief)
% is given to a variable made of a frame and an attribute with indexes
% the bba assigned to belief() given to every variable 
% 	- that shares the SAME frame
%	- that shares the SAME attribute
%	- is given to another value of the same indexes of the attribute
% It coresponds to: 
% For_all possible value of the index values, same belief on variables (on same frame)

% for all i,j, bel^Omega_Frame{i,j}(x(i,j) in A) = c(A).
% Input : for belief(index_which_belief)
% 	- index_which_belief : the index of the reference belief, an integer, not a vector
% it must be done for beliefs one per one

% output : empty
% but lot of beliefs have been created (one per ndex value)

% from index_which_belief, look its variable = ref_ver
% find all variable that share same attribute and same frame as ref_var
% look at index of common attribute 
% take all possible values of the indexes 
% for each index value case, find its variable, 
% built a belief on that variable = to reference belief 

% the end result is a new belief wihic difference with te original one is in the domain_variable
% Beware that the variables have been already built by the option all var index in run_variable

% suppose bel(X(i), Y(i,j), Z(j,k)) on var X Y Z, and the index are i,j,k.
% we take every combination of (i,j,k), define 
% var X(i), var Y(i,j), var Z(j,k)
% and store the num of these 3 var to build a bel on it.

% suppose each index is binary, 
% then we have 2 X var (as only 1 index), 4 Y var (as 2 indexes), 4 Z var (as 2 indexes)
% once this is done we can only have 8 bel (and not 2*4*4)
% so we store the existing triple 
% X     Y       Z			X	Y	Z
% i    ij      jk	ijk		
% 1    11      11	111		1	3	7
% 1    11      12	112		1	3	8
% 1    12      21	121		1	4	9	
% 1    12      22	122		1	4	10
% 2    21      11	211		2	5	7
% 2    21      12	212		2	5	8
% 2    22      21	221		2	6	9
% 2    22      22	222		2	6	10
% the last 3 numbers are the variable numbers that belong to domain_variable 
% of the 8 belief we nust build

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%



global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global max_nbr_bel_create_attXindex


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% for each variable, find its siblings (index, numbers), {} var
% thus the list of var index/numbers that share same frame, attribute, set of index
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ind_var_siblings, num_var_siblings] = find_equiv_variables;

% result on example =  ind_var_siblings{1} = [1 2]
% var| siblings
% 1  | 1 2
% 2  | 1 2
% 3  | 3 4 5 6 
% 4  | 3 4 5 6 
% 5  | 3 4 5 6 
% 6  | 3 4 5 6 
% 7  | 7 8 9 10
% 8  | 7 8 9 10
% 9  | 7 8 9 10
% 10 | 7 8 9 10

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% which variables I am interested in, those of belief.variable (dom+cond)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% list_var_num = the numbers of the variables that enter in the object belief to duplicate
% they are stored in belief(index_which_belief).domain_variable 
list_var_num = [belief(index_which_belief).domain_variable belief(index_which_belief).conditioning_variable];
list_var_ind = pin_manipulation('variable','num2ind',list_var_num);

% result on example : list_var_ind/num = [ VarX=1 VarY=3 VarZ=7] 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% finding the underlying attributes, indices and frames
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

list_index_name{1}   = '';
list_index_frame_num = [];
list_index_frame_ind = [];

ik = 0;
for ivar = 1:length(list_var_ind)
	att_num(ivar) = variable(list_var_ind(ivar)).attribute;
	att_ind(ivar) = pin_manipulation('attribute','num2ind',att_num(ivar));
	nbrind = length(attribute(att_ind(ivar)).index);	
	% beware of attribute without index : length([]) = 0.
	
	% att_num(1) = AttX, att_num(2) = AttY, att_num(3) = AttZ
	% nbrind is successively 1 2 2
	
	% cumulated list of name and frame_num used in the attributes underlying the involved variables
	for ij = 1:nbrind % not done if 0
		ik = ik + 1;
		list_index_name{ik} = attribute(att_ind(ivar)).index(ij).name;
		% list_index_name = {'i',   'i','j',   'j','k'}
	end
	list_index_frame_num = [list_index_frame_num   attribute(att_ind(ivar)).index(:).frame];	
	% when over list_index_frame_num = [i i j j k]
	
	% the siblings and brothers (= my siblings minus me) of the involved variables
	its_siblings_ind{ivar} = ind_var_siblings{list_var_ind(ivar)};
	% its_siblings_ind = [ 1 2]  [3 4 5 6] [7 8 9 10]
	its_brothers_ind{ivar} = setdiff([its_siblings_ind{ivar}],list_var_ind(ivar));
	% its_brothers_ind = = [ 2]  [4 5 6] [8 9 10]	
end

% list without replicate of name and frame_num; ordered
% used in the attributes underlying the involved varaibles
[list_index_name_unique   i_indname   j_indname] = unique(list_index_name);
% list_index_name_unique = i j k

% two index can share same frame without being equal
% list in same order as names in list_index_name_unique of involved frame
list_index_frame_num_unique = list_index_frame_num(i_indname);   
list_index_frame_ind_unique = pin_manipulation('frame','num2ind',list_index_frame_num_unique);
% list_index_frame_num_unique = i j k

ok = 1; % we verify we don't face a continuous frame 
card_frames = [frame(list_index_frame_ind_unique).cardinality];
% card_frames = 2 2 2
nbr_bel = 1;
for i = 1:length(card_frames)
	if strcmp(frame(list_index_frame_ind_unique(i)).type,'Continuous')
		ok = 0;
	else
		nbr_bel = nbr_bel * card_frames(i);
		% 		if strcmp(frame(list_index_frame_ind_unique(i)).type,'Integers')
		% 			elements_frame{i} = strvact(num2str([frame(list_index_frame_ind_unique(i)).min:...
		% 			frame(list_index_frame_ind_unique(i)).max]'));
		% 		else
		elements_frame{i} = frame(list_index_frame_ind_unique(i)).elements;
		%		end
	end
end
% elements_frame{1 2 3} = i1 i2    j1 j2      k1 k2

% nbr of belief we are going to build, max_nbr_bel_create_attXindex default limit = 512, 
% can be adapted with pull-down menu TBM_param

if nbr_bel > max_nbr_bel_create_attXindex 
	ok = 0; % too many bel to create
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% we build all these belief (hoping we don't get memory ioverflow, if many)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ok 
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% the involved index
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% for every variable involved in belief, we get
	% its attributes in att_num(ivar) and att_ind(ivar)  (already stored)
	% for every attribute involved, wet get 
	% a list of its index
	% for every belief to build we take one value for each index, 
	% select those needed for each variable
	% and build a new variable if not yet built, or get its number if already built
	% I need for each variable, the index of the index it uses so I can build index_value
	% if not yet built, I refuse to proceed, go fist in variable to duplicate them.
	
	nc_mat_ind = length(list_index_frame_num_unique) + 1;
	for i = 1:1:length(list_var_ind)
		my_ind = [];
		% we consider one variable and get its index
		for j = 1:length(attribute(att_ind(i)).index)
			my_ind = [my_ind  find(strcmp(list_index_name_unique,...
			attribute(att_ind(i)).index(j).name))];
			% name index(j).name is in position kl in list_index_name_unique
			% hence in column nc+1-kl of mat_ind
		end
		my_index_in_mat_ind{i} = [nc_mat_ind - my_ind];
		% poistion in mat_ind for variable list_var_ind(i), the i-th involved
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% build a matrix with all the possible values
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% poss_val{i,j} : j = 1 to nbr of index, i = 1:nbr_bel = nbr poss combination
	% mat_ind are the possible index value with index = 1,2...cardinal frame
	% the symbol of i is farme().element(i,:)
	mat_ind = [1:card_frames(1)]';
	for i = 2:length(card_frames)
		mat_ind = [kron([1:card_frames(i)]', ones(size(mat_ind,1),1)) ...
		kron(ones(card_frames(i),1),mat_ind)];
	end	
	% mat_ind(i,j) tells us that index-j has value element(mat_ind(i,j),:)
	% and we store in index_value the mat_ind(i,j)
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% consider each future belief
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	ind_new_bel = max(pin_belief(:,2));
	num_new_bel = max(pin_belief(:,3)); 
	
	for i_bel = 1:size(mat_ind,1)
		% for each line of mat_ind, I have to build a belief
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% Finding var num for new belief
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		for i = 1:length(list_var_ind)
			ind_val = [mat_ind(i_bel,my_index_in_mat_ind{i})];
			% % % % % % 			
			% % % % % % 			disp('new variable in list_var_ind')
			% % % % % % 			[i_bel i its_siblings_ind{i}]
			% % % % % % 			ind_val
			% % % % % % 			
			for k = 1:length(its_siblings_ind{i})
				% % % % % % 				
				% % % % % % 				its_siblings_ind{i}(k)
				% % % % % % 				variable(its_siblings_ind{i}(k)).index_value
				% % % % % % 				ind_val
				% % % % % % 				
				% % % % % % 				
				if variable(its_siblings_ind{i}(k)).index_value == ind_val;
					% variabble already exist and it found it now
					mem_var_num(i) = pin_manipulation('variable','ind2num', ...
					its_siblings_ind{i}(k));
				end						
			end
			% what if index not in same order!!!! ouf! they are in same order
		end
	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% we build the new belief 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		ind_new_bel = ind_new_bel + 1;
		num_new_bel = num_new_bel + 1;
		belief(ind_new_bel) = belief(index_which_belief);
		belief(ind_new_bel).number = num_new_bel;
		
		
		fn0 = belief(index_which_belief).full_name;
		sn0 = belief(index_which_belief).short_name;
		lg_name = strcat(fn0,'(');
		sh_name = strcat(sn0,'(');
		for i = 1:length(mem_var_num)
			lg_name = strcat(lg_name,num2str(mem_var_num(i)),',');
			sh_name = strcat(sh_name,num2str(mem_var_num(i)),',');
		end
		lg_name(length(lg_name)) = ')';
		sh_name(length(sh_name)) = ')';		
		% we call them ln(1,3 6) where 1 3 6 are the numbers of the variables (in mem_var_num)
		
		belief(ind_new_bel).full_name = lg_name;
		belief(ind_new_bel).short_name = sh_name;
		
		
		% Here below NOT ok for condit I don't see which are domaion or conditioning
		
		var_dom = mem_var_num; % intersect(mem_var_num, );  % with intersect data are sorted !!!
		% var_cond = intersect(mem_var_num, );
		
		belief(ind_new_bel).domain_variable = pin_manipulation('variable','ind2num',var_dom) ;		
		% belief(ind_new_bel).domain_cardinality = 
		% belief(ind_new_bel).conditioning_variable = pin_manipulation('variable','ind2num',var_cond) ;		
		% belief(ind_new_bel).conditioning_cardinality = 	
		
		
		% XXX XXXXX
		
	end

else
	hdw = warndlg(['I cann''t do the automatic creation of belief on every combination of attribute by index value. ',...
	'One index is continuous or you ask to create too many (>',num2str(max_nbr_bel_create_attXindex),...
	') belief objects (see TBM_Param to increase that limit, but beware for memory overflow). ',...
	'I did nothing. If you want to do it nevertheless, change the GUI parameter. ',...
	'You need ',num2str(nbr_bel),' beliefs']);
	uiwait(hdw);
end	

% we rebuild pin_belief
[useless] = pin_manipulation('belief','all',[]);















