function get_natural_frames 
% building a set of natural frames  (selected or all of them)
% if still other are needed, they will have to be fed by the user 
% with the pop menu TBM_Object/Frame 

%
%   Version : Dec 8,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute  variable belief
global pin_frame pin_structure pin_attribute  pin_variable 
global pin_mapping pin_belief 
global gui_position
global stored_text 
global flags flag_str

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% text on strucure in Root Window 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

help_text_handles = display_helps(0,[6 7 8]);	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% opening the UR figure 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[index_fig, hd_fig] = handling_all_figures('put_in_front',{'General Window'});
hd_fig_natural_fr = hd_fig(1);
window_name_help(1,hd_fig,'Get Natural Frames',[30]);

% input: empty

% output: 
% frame = set of natural frames (objects) 
% position_of_frame = index of frame that contains frame number 
% 	a nx2 matrix [cij] that states that frame number ci1 is in frame{ci2}.
% 	one line per existing frame

% list of natural frames
%   F T
%   0-1
%   No Yes
%   Male Female
%   Friend Foe
%   + -
%   For Against
%   A B
%  ternary unordered: A B C
%  4-ary unordered:   A B C D
%  ternary ordered:   1 2 3 
%  4-ary ordered:     1 2 3 4
% obtained a	utomatically if you choose the all of them option.

% those precooked but asking for furhter information
% k-ary unordered:  A B C... to be input:  k
% k-ary ordered:    1 2 3... to be input:  k
% integers: 				 to be input: from to 
% intervals:				 to be input:  k and [fromi toi]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% finding existing frames
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
i=0;
i=i+1;full_n{i} = 'Logical'; 
i=i+1;full_n{i} = 'Binary';
i=i+1;full_n{i} = 'No_Yes';
i=i+1;full_n{i} = 'Male_Female';
i=i+1;full_n{i} = 'Friend_Foe';
i=i+1;full_n{i} = 'Pos_Neg';
i=i+1;full_n{i} = 'For_Against';
i=i+1;full_n{i} = 'UnOrd_Classes_AB';
i=i+1;full_n{i} = 'UnOrd_Classes_ABC';
i=i+1;full_n{i} = 'UnOrd_Classes_ABCD';
i=i+1;full_n{i} = 'UnOrd_Classes_A,...';
i=i+1;full_n{i} = 'Ord_Classes_12';
i=i+1;full_n{i} = 'Ord_Classes_123';
i=i+1;full_n{i} = 'Ord_Classes_1234';
i=i+1;full_n{i} = 'Ord_Classes_C1,...C';
i=i+1;full_n{i} = 'Integers';
i=i+1;full_n{i} = 'Continuous';

nbr_queries = length(full_n) + 2;
exist_frame = 0;
active(1:nbr_queries) = 1; 
if length(frame)
	exist_frame = 1; % some frames exist already
	% finding those frames that exist
	for i = 1:length(full_n)		
		if ~isempty(find(strcmp({frame(:).full_name},full_n{i} )))
			active(i) = 0;
		end
	end
end
% togglebuttton will exist only for active item (those frames not yet exisitng)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% lisrting existing frames
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist_frame 
	handle_list = listing_existing('frame', gui_position(:,12), hd_fig_natural_fr,[]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% asking which frame you want 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

title = 'Selection of the requested natural frames. If *, extra data asked.';

k = 0;
k=k+1;text{k} = ' Logical False-True';
k=k+1;text{k} = ' Binary 0-1';
k=k+1;text{k} = ' Binary No-Yes';
k=k+1;text{k} = ' Binary Male-Female';
k=k+1;text{k} = ' Binary Friend-Foe';
k=k+1;text{k} = ' Binary + -';
k=k+1;text{k} = ' Binary For-Against';
k=k+1;text{k} = ' 2-ary A-B';
k=k+1;text{k} = ' 3-ary A-B-C';
k=k+1;text{k} = ' 4-ary A-B-C-D';
k=k+1;text{k} = '*k-ary A-B-C-D...';k_ary = k;
k=k+1;text{k} = ' 2-ary ordered 1-2';
k=k+1;text{k} = ' 3-ary ordered 1-2-3';
k=k+1;text{k} = ' 4-ary ordered 1-2-3-4';
k=k+1;text{k} = '*k-ary ordered 1-2-3-4...';k_ary_ord = k;
k=k+1;text{k} = '*Integers from-to';k_integer = k;
k=k+1;text{k} = '*Continuous from-to';k_continuous = k;
k=k+1;text{k} = 'All of them except those preceded with *.'; k_all = k;
k=k+1;text{k} = 'Do variable for each selected frame.'; k_dovar = k;

position = [.002 .002 .700 .996];
flags(1:k) = 0;

[boxes_status] = list_check_boxes(title, text, hd_fig_natural_fr, position, active);
% all checked boxes read

if boxes_status(k_all) % I avoid creating twice the same frame
	unstared_options = [1:10 12:14];
	for i = 1:length(unstared_options)
		if active(unstared_options(i)) 
			boxes_status(unstared_options(i)) = 1;
		else 
			boxes_status(unstared_options(i)) = 0;
		end
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the simple ones (no parameter, the unstared selected)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[new_pin] = get_next_pin('frame');
ibel = new_pin(2) - 1; % index for frame object

if boxes_status(1)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{1};
	frame(ibel).short_name = 'FT';
	frame(ibel).elements = strvcat('F','T');
	frame(ibel).type = 'Logical';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;	
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');
	end
end

if boxes_status(2)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{2};
	frame(ibel).short_name = 'B01';
	frame(ibel).elements = strvcat('0','1');
	frame(ibel).type = 'Binary';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(3)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{3};
	frame(ibel).short_name = 'NY';
	frame(ibel).elements = strvcat('N','Y');
	frame(ibel).type = 'Binary';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;	
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(4)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{4};
	frame(ibel).short_name = 'MF';
	frame(ibel).elements =  strvcat('M','F');
	frame(ibel).type = 'Binary';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');
	end
end


if boxes_status(5)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{5};
	frame(ibel).short_name = 'FrFo';
	frame(ibel).elements = strvcat('Fr','Fo');
	frame(ibel).type = 'Binary';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(6)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{6};
	frame(ibel).short_name = 'PN';
	frame(ibel).elements = strvcat('P','N');
	frame(ibel).type = 'Binary';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(7)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{7};
	frame(ibel).short_name = 'FA';
	frame(ibel).elements = strvcat('F','A');
	frame(ibel).type = 'Binary';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(8)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{8};
	frame(ibel).short_name = 'AB';
	frame(ibel).elements = strvcat('A','B');
	frame(ibel).type = 'Classes_Un_ordered';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(9)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{9};
	frame(ibel).short_name = 'ABC';
	frame(ibel).elements = strvcat('A','B','C');
	frame(ibel).type = 'Classes_Un_ordered';
	frame(ibel).cardinality=3;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(10)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{10};
	frame(ibel).short_name = 'ABCD';
	frame(ibel).elements = strvcat('A','B','C','D');
	frame(ibel).type = 'Classes_Un_ordered';
	frame(ibel).cardinality=4;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(12)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{12};
	frame(ibel).short_name = 'F_12';
	frame(ibel).elements = strvcat('C1','C2');
	frame(ibel).type = 'Classes_Ordered';
	frame(ibel).cardinality=2;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(13)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{13};
	frame(ibel).short_name = 'F_123';
	frame(ibel).elements = strvcat('C1','C2','C3');
	frame(ibel).type = 'Classes_Ordered';
	frame(ibel).cardinality=3;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end


if boxes_status(14)
	ibel = ibel + 1;
	frame(ibel).number = ibel;
	frame(ibel).full_name = full_n{14};
	frame(ibel).short_name = 'F_1234';
	frame(ibel).elements = strvcat('C1','C2','C3','C4');
	frame(ibel).type = 'Classes_Ordered';
	frame(ibel).cardinality=4;
	frame(ibel).min = 0;
	frame(ibel).max = 0;
	if boxes_status(k_dovar)
		frame_do_var(ibel)
		useless = pin_manipulation('structure','all','');
		useless = pin_manipulation('attribute','all','');
		useless = pin_manipulation('variable','all','');		
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the complex ones (with a parameter)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear query def_answ

iq = 0;
qu = 0;
if boxes_status(k_ary)
	qu = 1;
	iq=iq+1;query{iq} = 'Value of k for k_ary A-B-C... #';
	def_answ{iq} = '5';
end
if boxes_status(k_ary_ord)
	qu = 1;
	iq=iq+1;query{iq} = 'Value of k for k_ary ordered 1-2-3... #';
	def_answ{iq} = '5';
end
if boxes_status(k_integer)
	qu = 1;
	iq=iq+1;query{iq} = 'Integers from to #';
	def_answ{iq} = '0   100';
end
if boxes_status(k_continuous)
	qu = 1;
	iq=iq+1;query{iq} = 'Continuous from to #';
	def_answ{iq} = '0 100';
end

if qu
	[answer,choice] = dialog_queries(hd_fig_natural_fr,title,query,def_answ,[.002 .002],[],2,[],[],[],[]);
	iq = 0;
	if choice ~= 1
		answer = [];
	else
		if boxes_status(k_ary)
			iq=iq+1;		 
			k = str2num(answer{iq});
			for j = 1:k
				label{j} = char(64+j); % the ascii of A B,C...
			end		
			ibel = ibel + 1;
			frame(ibel).number = ibel;
			frame(ibel).full_name = ['UnOrd_Classes_A,...',label{k}];
			frame(ibel).short_name = ['A',label{k}];
			frame(ibel).elements = strvcat(label);
			frame(ibel).type = 'Classes_Un_ordered';
			frame(ibel).cardinality=k;
			frame(ibel).min = 0;
			frame(ibel).max = 0;
			if boxes_status(k_dovar)
				frame_do_var(ibel)
				useless = pin_manipulation('structure','all','');
				useless = pin_manipulation('attribute','all','');
				useless = pin_manipulation('variable','all','');				
			end
		end
		if boxes_status(k_ary_ord)		
			iq=iq+1;
			k = str2num(answer{iq});
			for j = 1:k
				label{j} = strcat('C',num2str(j)); 
			end	
			ibel = ibel + 1;
			frame(ibel).number = ibel;
			frame(ibel).full_name = ['Ord_Classes_C1,...C',answer{iq}];
			frame(ibel).short_name = ['C1_C',answer{iq}];
			frame(ibel).elements = strvcat(label);
			frame(ibel).type = 'Classes_Ordered';
			frame(ibel).cardinality=k;
			frame(ibel).min = 0;
			frame(ibel).max = 0;
			if boxes_status(k_dovar)
				frame_do_var(ibel)
				useless = pin_manipulation('structure','all','');
				useless = pin_manipulation('attribute','all','');
				useless = pin_manipulation('variable','all','');				
			end
		end
		if boxes_status(k_integer)	
			iq=iq+1;
			kk = str2num(answer{iq});
			from = kk(1);
			to = kk(2);
			k = to-from+1;
			ibel = ibel + 1;
			frame(ibel).number = ibel;
			frame(ibel).full_name = ['Integers_from_',num2str(from),'_to_',num2str(to)];   
			frame(ibel).short_name = ['Int',num2str(from),'-',num2str(to)];
			frame(ibel).elements = '';
			frame(ibel).type = 'Integers';
			frame(ibel).cardinality=k;
			frame(ibel).min = from;
			frame(ibel).max = to;
			if boxes_status(k_dovar)
				frame_do_var(ibel)
				useless = pin_manipulation('structure','all','');
				useless = pin_manipulation('attribute','all','');
				useless = pin_manipulation('variable','all','');				
			end
		end
		if boxes_status(k_continuous)	
			iq=iq+1;
			kk = str2num(answer{iq});
			from = kk(1);
			to = kk(2);
			ibel = ibel + 1;
			frame(ibel).number = ibel;
			frame(ibel).full_name = ['Continuous_from_',num2str(from),'_to_',num2str(to)];   
			frame(ibel).short_name = ['Cont',num2str(from),'-',num2str(to)];
			frame(ibel).elements = '';
			frame(ibel).type = 'Continuous';
			frame(ibel).cardinality=1E50;
			frame(ibel).min = from;
			frame(ibel).max = to;
			if boxes_status(k_dovar)
				frame_do_var(ibel)
				useless = pin_manipulation('structure','all','');
				useless = pin_manipulation('attribute','all','');
				useless = pin_manipulation('variable','all','');				
			end
		end	
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building the pin_frame
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
window_name_help(2,hd_fig,'Get Nataral Frames',[30]);
if exist('handle_list'); delete(handle_list);end
delete(help_text_handles)
useless = pin_manipulation('frame','all',[]);
