function [out_option] = plot_network(hd_fig,hd_axes,hd_menu)
% plot a network where nodes and hedges can be moved around and zoomed
% belief and variable objects can be created
% computation by belief propagation can be performed
% results can be displayed

% option_EVN_Control 
% 'create_EVN' for the case you have (maybe) some objects and want to start a new EVN
% 'buld_EVN_from_loaded_nodes': you had saved a EVN, and you reload it, I display it and then you proceed as with create_EVN
% 'create_new_objects_in_EVN' : you are in EVN and decide to build new objects
% if 'buld_EVN_from_loaded_nodes' I need build figure, pin, check objects are loaded,
% 			draw as with set of bel fct (but not messy as already nicely plotted)
% if 'create_EVN', I just enter EVN
% if 'create_new_objects_in_EVN', I return where I came from

% out_option is just used to tell run_plot_network what it must do now
% used as in some case I go back to run_plot_network, do something there and coma bacck here
% out_option = Load EVN, Save EVN, Clear EVN, Save Objects, Quit EVN

%
%   Version : Jan 4,03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building the nodes and the links for those variables and beliefs 
% fed one by one and located by user
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute  pin_variable 
global pin_mapping pin_belief 
global root_file_name save_path file_name_begin gui_position
global FRAME STRUCTURE MAPPING ATTRIBUTE VARIABLE BELIEF QUERY

global node pin_node link pin_link  histo pin_histo need_uiresume flags flags3 flagstr scaling_coef
global hd_smallUL_Button_EVN light_green light_red   plot_limits_initial  plot_limits_now 
global EVN_New_Object_flag   option_EVN_Control  EVN_Contro_mem
global pull_down_menu
global help_level

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% get the figure back
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

set(hd_fig,'visible','on')
figure(hd_fig)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% plot parameters
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sel_col = [0.5 1 1];

zoom_coef = .15;
zoom_in_matrix = [1-zoom_coef zoom_coef ; zoom_coef 1-zoom_coef ];
zoom_out_matrix = inv(zoom_in_matrix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building figure, axes and menus, where network will be displayed
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pos_axes = get(hd_axes,'position');
% hd_fig Units = pixel

xlim_nsc = get(hd_axes,'Xlim'); % for getting normal scale 
ylim_nsc = get(hd_axes,'Ylim'); % the one in pixel, as the centers

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Initiating the pin tables for node, link and histo
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pin_node = pin_manipulation('node','all',[]);
pin_link = pin_manipulation('link','all',[]);
pin_histo = pin_manipulation('histo','all',[]);

out_option = ' '; % just to initialize it

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%										     %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%     Preparing/restoring the EVN figure    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%										     %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

brand_new_plot = 0;

switch option_EVN_Control
% option_EVN_Control = 
% 	'brand_new_EVN'		I just build a new EVN from scratch
% 	'loaded EVN'		I have loaded past data
%	'bel_var changed'	I have asked for new bel and var objects
% 	'just_in_out'		I just came out of EVN (for savionf data) and go back

case 'brand_new_EVN'
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% starting new EVN, all I do is waiting for objects to be displayed
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	plot_limits_initial = [xlim_nsc ylim_nsc];	
	plot_limits_now = plot_limits_initial;	
	scaling_coef = 1;
	
	which_menu_EVN('brand_new_EVN');
	
	propagation_done = 0;
	plot_EVN_done = 0;
	plot_betp_done = 0;
	displayed_plot = 'none';
	
	% when enterning for the first tiem in EVN, I list all belief objets 
	% as it seems normal to select some of them
	brand_new_plot = 1;
	
case 'loaded EVN'
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% An EVN has been loaded, and I must redraw it, data were verified already
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	plot_limits_initial = [xlim_nsc ylim_nsc];	
	plot_limits_now = plot_limits_initial;	
	scaling_coef = 1;
	
	if length(histo)
		for i=1:length(histo)
			histo(i).handle = plot_histo(i,hd_fig); 
			% I draw it so it exist for later display when I switch
			hide_plot('betp_plot');
		end
		propagation_done = 1; % because betp exists
		plot_betp_done = 1;		
		which_menu_EVN('loaded EVN with histo');
	else
		% no histo betp
		propagation_done = 0;
		plot_betp_done = 0;
		which_menu_EVN('loaded EVN no histo');
	end	

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%	
	% putting the EVN somehow in the middel of the window, no zoom
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% finding the extrema of my full plot
	% for getting fully zoomed image
	vmin = min([node(:).localization_init_sc]');
	vmax = max([node(:).localization_init_sc]');
	x = (vmin(1)+vmax(2))/2;
	y = (vmin(3)+vmax(4))/2; 
	
	xlim0 = get(hd_axes,'Xlim');  % now limits
	ylim0 = get(hd_axes,'Ylim');  % now limits
	center_pt = [sum(xlim0)/2 sum(ylim0)/2]; % cetner point on now limits	
	dx = x - center_pt(1);
	dy = y - center_pt(2);
	
	set(hd_axes,'Xlim',xlim0 + [dx dx], 'Ylim',ylim0 + [dy dy]); % shifting plot	
	% memorizing the shift for future use of node move
	plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% I draw the network with its initial scale and location, and I stay in EVN 
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%		
	draw_network(hd_fig,hd_axes);
	plot_EVN_done = 1;
	displayed_plot = 'EVN';
	
case 'bel_var changed'	 % within EVN, I asked for new var/bel object from their editor

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% an EVN did exist, and I check you still have all the objects used by the nodes
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	if length(node) == 0;
		% when the first thing you did in EVN was asking for New Object		
		% I go back in EVN window with my new objects.
		exist_it = 1; % means OK
	else
		% some nodes already existed, and I just create new var/bel
		exist_it = 1;
		for i = 1:size(pin_node,1)
			node_ind = pin_node(i,2);		
			switch node(node_ind).nature
			case 'belief'
				if ~ismember(node(node_ind).obj_num,[belief(:).number]); 
					% it does not exists
					exist_it = 0;
				end
			case 'variable'
				if ~ismember(node(node_ind).obj_num,[variable(:).number]); 
					% it does not exists
					exist_it = 0;
				end
			end
		end
	end

	if exist_it == 0
		% you broke something, it is stupid,
		text = ['Data corrupted. Maybe you deleted an object used for the EVN.',...
		'I cannot proceed, so I quit and EVN lost.']
		choice = text_with_n_buttons(hd_fig,text,{'OK'}, [0 0]);	
		node = [];         link = [];         histo = [];
		pin_node = [0 0 0];pin_link = [0 0 0];pin_histo = [0 0 0];			
		propagation_done = 0;
		plot_EVN_done = 0;
		plot_betp_done = 0;
		displayed_plot = 'none';	
		option_EVN_Control = 'all cleared';
		return
		
	else
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% plot parameters
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		xlim_nsc = plot_limits_initial(1:2);
		ylim_nsc = plot_limits_initial(3:4);		
		scaling_coef =  (plot_limits_initial(2)-plot_limits_initial(1))/(plot_limits_now(2)-plot_limits_now(1));
		% so I get the EVN figure parameters back (=those I had when I left)	
		
		% I kill histo as we got new bel fct, hence hsito will have to be recomputed
		histo = [];
		pin_histo = [0 0 0];
		
		propagation_done = 0;
		plot_EVN_done = 0;
		plot_betp_done = 0;
		displayed_plot = 'none';	
		which_menu_EVN('bel_var changed');
	end

case 'just_in_out'
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% we had quit the figure to go to the buttons menus, and are back. Nothing has changed.
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	xlim_nsc = plot_limits_initial(1:2);
	ylim_nsc = plot_limits_initial(3:4);	
	scaling_coef =  (plot_limits_initial(2)-plot_limits_initial(1))/(plot_limits_now(2)-plot_limits_now(1));
	% so I get the EVN figure parameters back (=those I had when I left)	
	
	propagation_done = EVN_Contro_mem.propagation_done;
	plot_EVN_done = EVN_Contro_mem.plot_EVN_done;
	plot_betp_done = EVN_Contro_mem.plot_betp_done;
	displayed_plot = EVN_Contro_mem.displayed_plot;
	% I restore the menus the way they were when I said quit
	for i = 1:length(pull_down_menu)
		if strcmp(pull_down_menu(i).fig_parent,'EVN figure')			
			if pull_down_menu(i).status == 0
				howst = 'off';
			else
				howst = 'on';
			end	
			set(pull_down_menu(i).hd,'enable',howst);
		end
	end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%                                        %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%                 Part 2                 %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%   Excecuting the menus of EVN window   %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%                                        %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% what to do next, click on a control node, drag and drop nodes, 
% select node with right-click/ctrl_click
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
go_on = 1;

pos_fig = get(hd_fig,'position');

while go_on % the only way out is thru the quit that send you back to the General Window
	
	
	% when it is first time,I go direct to 'add_belief'
	if brand_new_plot 
		brand_new_plot = 0;
		flagstr = 'add_belief';
		flags = [];		
	else
		flagstr = [];
		flags = [];
		% flagstr states which menu is selected
		% flags used to communicate with clb_node
		
		need_uiresume = 1; % so I finish clb with uiresune even if I hit a node
		
		uiwait(gcf) % get data from the EVN menus, the figure is the hd_fig till I quit
		
		if flags3 == 2 % message from callback that bbm have changed
			flags3 = 0;
			flagstr = 'Menu_node';
		end
	end

	% flagstr tells us what to do (menus, node_move, node_menu)	build by callback of menus
	switch flagstr
	
	case 'add_belief' % menu
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% add some beliefs to  EVN
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		if strcmp(displayed_plot,'betp')
			hide_plot('betp_plot');
			show_back('EVN_plot');
			displayed_plot ='EVN';
		end	
		% If I go in the figure, I need a red button at UL
		set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_red,'String','Menus Off');		
		[choice, sel_bel_ind] = list_object_for_EVN('belief',hd_fig);
		
		if choice == 2 % I ask for an EVN
			EVN_Contro_mem.propagation_done = propagation_done;
			EVN_Contro_mem.plot_EVN_done = plot_EVN_done;
			EVN_Contro_mem.plot_betp_done = plot_betp_done;
			EVN_Contro_mem.displayed_plot = displayed_plot;
			go_on = 0;
			out_option = 'Load EVN';			
		end
		if choice == 1 & length(sel_bel_ind)  % not cancel and list not empty, otherwise we cancel	
			if length(node) == 0	
				% the first time I use belief2nodes and draw_network
				num_bel = pin_manipulation('belief','ind2num',sel_bel_ind);
				
				% preparing the nodes
				belief2nodes(num_bel,hd_axes);
				% drawing the network
				% it uses picto_with_text_and_menu to really draw the nodes
				draw_network(hd_fig,hd_axes);		
				
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				%
				% I center the EVN
				%
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				
				xlim0 = get(hd_axes,'Xlim');  % now limits
				ylim0 = get(hd_axes,'Ylim');  % now limits
				center_pt = [sum(xlim0)/2 sum(ylim0)/2]; % cetner point on now limits
				
				nd = reshape([node(:).center],2,length(node));
				nd_center = (min(nd,[],2) + max(nd,[],2))/2;
				
				dx = nd_center(1) - center_pt(1);
				dy = nd_center(2) - center_pt(2);
				
				set(hd_axes,'Xlim',xlim0 + [dx dx], 'Ylim',ylim0 + [dy dy]); % shifting plot	
				% memorizing the shift for future use of node move
				plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
				
			else
				% an EVN is alreday displayed, I add new beliefs 
				exit_choice = new_node_for_belief(sel_bel_ind); % I put the beli node and its links to old EVN nodes
				if exit_choice  % I add at least one belief
					locate_node(plot_limits_initial, plot_limits_now, pos_axes, sel_col,hd_fig,hd_axes);
					% I move the node to its nice place by hand or with nice graph
				end
			end				
			propagation_done = 0;
			clear_global_vbs;
			plot_EVN_done = 1;
			plot_betp_done = 0;
			displayed_plot = 'EVN';
			histo = [];
			pin_histo = [0 0 0];
			delete(findobj('userdata','betp_plot'))
			which_menu_EVN('add_object')			
			
			% list_object_for_EVN adds the   Get New Object
			
		end	% if choice == 1 & length(sel_bel_ind)
		% get UL button green
		set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_green,'String','Menus On');
		
	case 'add_variable' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% add variables to EVN
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if strcmp(displayed_plot,'betp')	
			hide_plot('betp_plot');
			show_back('EVN_plot');
			displayed_plot ='EVN';
		end	
		% If I go in the figure, I need a red button at UL
		set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_red,'String','Menus Off');	
		[choice, sel_var_ind] = list_object_for_EVN('variable',hd_fig);
		
		if choice == 1 % if not, I do nothing as you ask for cancel
			if length(sel_var_ind) == 0
				% we just skip as nothing selected
			else 
				exit_choice = new_node_for_variables(sel_var_ind);	
				if exit_choice 			
					locate_node(plot_limits_initial, plot_limits_now, pos_axes, sel_col,hd_fig,hd_axes);
				end	
				propagation_done = 0;
				clear_global_vbs;
				plot_EVN_done = 1;
				plot_betp_done = 0;
				displayed_plot = 'EVN';
				histo = [];
				pin_histo = [0 0 0];
				delete(findobj('userdata','betp_plot'))					
				which_menu_EVN('add_object')
				
			end
		end % if choice == 1
		% get UL button green
		set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_green,'String','Menus On');
		
	case 'new_propositions'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% new variables as propositions (several)
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% I am in front of EVN Window, 
		% I open the window for entering new propositions, answer the queries, propo are built,
		% I come back HERE		
		gcf_mem = gcf;
		getting_propositions
		figure(gcf_mem)
		% to display the new propo, select menu variables and select what you want to be added 
		% (probably those you have just built)
		
	case 'new_variables'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% new variables in general (several)
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% as for propositions, but another window
		gcf_mem = gcf;
		object_editor
		figure(gcf_mem)
		
	case 'new_belief'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% one new belief 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% we get the whole belief editor
		gcf_mem = gcf;
		run_belief('EVN')
		figure(gcf_mem)
		
	case 'Wrong_place' % figure but not a node
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% I hit an empty location in the figure, I do nothing 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
	case 'Nice_Graph' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% repositioning nodes automatically in a better way (P.H. Wuillemin algo)
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		num_nod = [node(:).number];
		spread_nodes(num_nod);		
		plot_EVN_done = 1;
		displayed_plot = 'EVN';
		
	case 'Move_node' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% node: move or menu, can be entered either from menu or by hitting anywhere 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		switch displayed_plot
		case 'EVN'
			% If I go in the figure, I need a red button at UR
			set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_red,'String','Menus Off','Userdata','Menus Off');				
			locate_node(plot_limits_initial, plot_limits_now,pos_axes, sel_col,hd_fig,hd_axes);
			
		case 'betp'
			set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_red,'String','Menus Off','Userdata','Menus Off');		
			set(findobj('Userdata','ntw_menu'),'Enable','off');	% no menu while in tree
			set(hd_fig,'Selected','on')
			go_on_histo = 1;
			
			while go_on_histo
				waitforbuttonpress % so I see where you click
				cur_pt = get(hd_fig,'CurrentPoint');   % button down position in figure units			
				pos_end = dragrect  ([cur_pt(1,1) cur_pt(1,2) 10 10]);
				
				% If I click on Menus Off I stop this histo move
				xy = get(hd_smallUL_Button_EVN(2),'position');
				if cur_pt(1) >= xy(1) & cur_pt(1) <= xy(1)+xy(3) ...
					&  cur_pt(2) >= xy(2) & cur_pt(2) <= xy(2)+xy(4) 
					% I am thus above the little UL button, not an histo node
					go_on_histo = 0;
					set(hd_smallUL_Button_EVN(2),'BackgroundColor',light_green,'String','Menus On','Userdata','Menus On');						
					set(findobj('Userdata','ntw_menu'),'Enable','on');	% get menu back
				else			
					% I had click above an histo, which one, 
					i_histo = 0;
					for i = 1:length(histo)
						xy = histo(i).frame_pos;
						if cur_pt(1) >= xy(1) & cur_pt(1) <= xy(1)+xy(3) ...
							&  cur_pt(2) >= xy(2) & cur_pt(2) <= xy(2)+xy(4) 
							i_histo = i;
						end
					end
					if i_histo 
						% i delete previous one, and draw new one 
						histo(i_histo).center = pos_end(1:2);
						delete(histo(i_histo).handle);
						histo(i_histo).handle = plot_histo(i_histo,hd_fig);
					else 
						% you miss the histos
					end		
				end
			end % while go_on_histo
		
		end % end switch
		% I restore the menus the way they were when I hit the button to get back to the menus
		for i = 1:length(pull_down_menu)
			if strcmp(pull_down_menu(i).fig_parent,'EVN figure')			
				if pull_down_menu(i).status == 0
					howst = 'off';
				else
					howst = 'on';
				end	
				set(pull_down_menu(i).hd,'enable',howst);
			end
		end		
	
	case 'Initial_scale' % on the pixel scale, as the centers of the nodes
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% on the pixel scale, as the centers of the nodes 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if strcmp(displayed_plot,'EVN')			
			set(hd_axes,'Xlim',xlim_nsc, 'Ylim',ylim_nsc);
			plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
			scaling_coef = 1;
		end
	
	case 'Full_Network' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% the fully zoomed network
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if strcmp(displayed_plot,'EVN')	
			% finding the extrema of my full plot
			% for getting fully zoomed image
			vmin = min([node(:).localization_init_sc]');
			vmax = max([node(:).localization_init_sc]');
			xlim_fz = [ min(vmin(1),plot_limits_initial(1))   max(vmax(2),plot_limits_initial(2)) ];
			ylim_fz = [ min(vmin(3),plot_limits_initial(3))   max(vmax(4),plot_limits_initial(4)) ];
			
			dx = xlim_fz(2)-xlim_fz(1); dxr = dx / (plot_limits_initial(2)-plot_limits_initial(1));
			dy = ylim_fz(2)-ylim_fz(1); dyr = dy / (plot_limits_initial(4)-plot_limits_initial(3));
			
			if dxr > dyr
				sht = (dxr*(plot_limits_initial(4)-plot_limits_initial(3)) - dy )/2;
				ylim_fz = ylim_fz + [-sht sht];
			else
				sht = (dyr*(plot_limits_initial(2)-plot_limits_initial(1)) - dx )/2;
				xlim_fz = xlim_fz + [-sht sht];
			end
			% we must keep the same ratio for the axes as initially otherwise they don't fit nicely the whole screen
			set(hd_axes,'Xlim',xlim_fz, 'Ylim',ylim_fz);
			plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
			scaling_coef = (plot_limits_initial(2)-plot_limits_initial(1))/(plot_limits_now(2)-plot_limits_now(1));
		end
	
	case 'Re_Center' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% recenter: produces the cross, drag it, raise finger 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		switch displayed_plot
		case 'EVN'
			xlim0 = get(hd_axes,'Xlim');  % now limits
			ylim0 = get(hd_axes,'Ylim');  % now limits
			center_pt = [sum(xlim0)/2 sum(ylim0)/2]; % center point on now limits
			
			[x y] = ginput(1); % the point of the graph that must go to the center of the figure
			flags = []; flagstr = []; % suppress call back values: I click on figure	
			
			dx = x - center_pt(1);
			dy = y - center_pt(2);
			
			set(hd_axes,'Xlim',xlim0 + [dx dx], 'Ylim',ylim0 + [dy dy]); % shifting plot	
			% memorizing the shift for future use of node move
			
			plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
			% the limts are now those in 
			% get(hd_axes,'Xlim')  % now limits
			% get(hd_axes,'Ylim')  % now limits
		case 'betp'
			posit = get(gcf,'position');
			set(hd_axes,'visible','on');
			xlim0 = get(hd_axes,'Xlim');  % now limits
			ylim0 = get(hd_axes,'Ylim');  % now limits
			center_pt = [sum(xlim0)/2 sum(ylim0)/2]; % cetner point on now limits
			
			[x y] = ginput(1); % the point of the graph that must go to the center of the figure
			flags = []; flagstr = []; % suppress call back values: I click on figure	
			
			dxy = [posit(3)*(x-center_pt(1))/(xlim0(2)-xlim0(1))   posit(4)*(y-center_pt(2))/(xlim0(2)-xlim0(1)) ];
			set(hd_axes,'visible','off');
			
			for i_histo = 1:length(histo)
				histo(i_histo).center = histo(i_histo).center - dxy;
				delete(histo(i_histo).handle);
				histo(i_histo).handle = plot_histo(i_histo,hd_fig);
			end		
		end
	
	case 'EVN_at_center'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% center EVN, not for the BetP histo, these are put back with relocate betp
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		xlim0 = get(hd_axes,'Xlim');  % now limits
		ylim0 = get(hd_axes,'Ylim');  % now limits
		center_pt = [sum(xlim0)/2 sum(ylim0)/2]; % cetner point on now limits
		
		nd = reshape([node(:).center],2,length(node));
		nd_center = (min(nd,[],2) + max(nd,[],2))/2;
		
		dx = nd_center(1) - center_pt(1);
		dy = nd_center(2) - center_pt(2);
		
		set(hd_axes,'Xlim',xlim0 + [dx dx], 'Ylim',ylim0 + [dy dy]); % shifting plot	
		% memorizing the shift for future use of node move
		plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
		
	case 'Zoom_in' % menu: zoom in		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% menu: zoom in 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if strcmp(displayed_plot,'EVN')	
			set(hd_axes,'Xlim',get(hd_axes,'Xlim') * zoom_in_matrix,...
			'Ylim', get(hd_axes,'Ylim') * zoom_in_matrix);
			
			plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
			
			% if scaling is: [0 100] becomes [15 85] ; scaling_coef = 100/70 = 1.43, 
			% then (100/70)^2, then(100/70)^3,.... if zoom_in 3 times
			% it means that 14.3 pixel in new scame = 10 pixel in olde scale
			% thus future shift by n pixel corresponds to a n/scaling_coef shift in original scale
			
			scaling_coef =  (plot_limits_initial(2)-plot_limits_initial(1))/(plot_limits_now(2)-plot_limits_now(1));
		end
	
	case 'Zoom_out' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% menu: zoom out 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if strcmp(displayed_plot,'EVN')	
			set(hd_axes,'Xlim',get(hd_axes,'Xlim') * zoom_out_matrix,...
			'Ylim', get(hd_axes,'Ylim') * zoom_out_matrix);
			plot_limits_now = [get(gca,'Xlim') get(gca,'Ylim')];
			scaling_coef = (plot_limits_initial(2)-plot_limits_initial(1))/(plot_limits_now(2)-plot_limits_now(1));
		end
	
	case 'MultiVar_bel'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% menu: MultiVar bel, ask for list of var and get m,bel,betp,pl on it using VBS
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% propagation MUST have been performed before I do this option. What I ask is maybe already computed
		
		% we get the list of variables from its list
		gcf_mem = gcf;
		[index_fig, hd_fig] = handling_all_figures('put_in_front',{'Root Window','General Window'});
		hd_bel_fig =  hd_fig(2); 
		window_name_help(1,hd_bel_fig,'Multivariate Belief',[41]);
		
		list_var_num = [];
		list_bels_number = [];
		for i = 1:length(node)
			if strcmp(node(i).nature,'variable')
				list_var_num = [list_var_num node(i).obj_num];
			else
				list_bels_number = [list_bels_number node(i).obj_num];
			end
		end
		list_var_ind =  pin_manipulation('variable','num2ind',list_var_num);
		
		for i = 1:length(list_var_ind)
			list{i} = variable(list_var_ind(i)).full_name;
		end
	
		position = gui_position(:,6);
		
		posit = [ position(1) (position(2)+position(4)-.03)  position(3) .03];
		handle_list(1) = uicontrol('Parent', hd_bel_fig,...
		'position',posit,...
		'string','Variables',...
		'tag','listbox',...
		'Style','text');
		
		posit = [ position(1) position(2) position(3) (position(4)-.04)];
		handle_list(2) = uicontrol('Parent', hd_bel_fig,...
		'style','listbox',...
		'position',posit,...
		'string',list,...
		'max',1000,...
		'tag','listbox',...
		'value',[], ...
		'callback', 'uiresume',...
		'Enable','on'); 
		
		text = 'Select the set of variables on which marginal must be computed and displayed.';
		sel_var_pos = [];
		while isempty(sel_var_pos)
			
			choice = text_with_n_buttons(hd_bel_fig, text, {'Done'}, gui_position(:,13)');		
			% % % % 			if choice == 2 
			% % % % 				delete(handle_list)
			% % % % 				window_name_help(2,hd_bel_fig,'Run VBS Window',[36]);
			% % % % 				return
			% % % % 			end
			% % % % 		
			sel_var_pos = get(handle_list(2),'value');
			list_var_num = list_var_num(sel_var_pos);
			list_var_ind = list_var_ind(sel_var_pos);	
		end
		delete(handle_list)		
		figure(gcf_mem)
		all_marginals = 0;		
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% creating those objects as requested by VBS author 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		FRAME = frame;
		STRUCTURE = structure;
		MAPPING = mapping;
		ATTRIBUTE = attribute;
		VARIABLE = variable;
		BELIEF = belief; 
		% beware foset in BELIEF are not those expected by VBS, they must be fliplr before running solve
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% THE REAL COMPUTATION IS ASKED HERE
		% bel_idnum = ssolve(list_bels_number, list_var_num) if all marginals = 0
		% else with the list of the idnum generated by each request 
		bel_idnum = execute_vbs(list_bels_number,list_var_num,all_marginals);
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% storing results
		ind_bel = find([BELIEF(:).number] == bel_idnum);	
		next_pin = get_next_pin('belief');	
		my_ind_bel = next_pin(2);  
		bel_num = next_pin(3);
		belief(my_ind_bel) = BELIEF(ind_bel); % get back VBS results
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% Saving results, make it a query XXXX
		% I have to fill all the fields of belief, all I got from VBS is 
		% a number, domain variable and dom cardinilaity and bp
		% with the foset reverse
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% building the names, step 1.
		
		computed = 'dc(';
		for i = list_bels_number
			computed = [computed,num2str(i),'.1'];
		end
		computed = [computed,')mg(',num2str(my_ind_bel),')'];
		
		belief(my_ind_bel).number = bel_num;
		belief(my_ind_bel).full_name = ['B',num2str(bel_num)];
		belief(my_ind_bel).short_name = ['B',num2str(bel_num)];
		belief(my_ind_bel).computed = computed;
		
		belief(my_ind_bel).disposable = 1;
		belief(my_ind_bel).agent = '';
		belief(my_ind_bel).time = [];
		
		ind_var_marg = pin_manipulation('variable','num2ind',belief(my_ind_bel).domain_variable);
		num_frames = [variable(ind_var_marg).frame];
		cardi = [frame(pin_manipulation('frame','num2ind',num_frames)).cardinality];
		belief(my_ind_bel).domain_cardinality = cardi;
		
		belief(my_ind_bel).conditioning_variable = [];
		belief(my_ind_bel).conditioning_cardinality = 1;	
		belief(my_ind_bel).bp(1).form(1).type = 'm'; 
		
		belief(my_ind_bel).bp(1).form(1).representation = 'selected';
		belief(my_ind_bel).bp(1).form(1).foset_meaning = ''; 
		
		% control that focal sets are indeed a column
		[nl nc] = size(belief(my_ind_bel).bp(1).form(1).focal_sets);
		if prod(cardi) <= nbr_bit_per_word & nc > 1
			belief(my_ind_bel).bp(1).form(1).focal_sets = belief(my_ind_bel).bp(1).form(1).focal_sets';
		end
	
		% updating pin_belief
		useless = pin_manipulation('belief','all',[]);
		% Belief is built
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% we show the results
		results_display(my_ind_bel,1,1);
		figue(gcf_mem)
		
	case 'Menu_node' % the call lb_node for the menu attached to the nodes
		% put propagation off
		propagation_done = 0;
		clear_global_vbs;
		plot_EVN_done = 1;
		plot_betp_done = 0;
		displayed_plot = 'EVN';
		which_menu_EVN('bbm_change');
		
	case 'Propagate' % menu: propagate
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% menu: propagate 		
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if strcmp(displayed_plot,'EVN')	
			% check node is not empty for belief objects
			if length(strcmp({node(:).nature},'belief'))	
				
				run_VBS('from_EVN');
				
				% I get_all_marginals
				propagation_done = 1;
				clear_global_vbs;
				plot_EVN_done = 1;
				plot_betp_done = 0;
				displayed_plot = 'EVN';
				which_menu_EVN('Propagate');
				text = 'Propagation is done. You can proceed.';
				choice = text_with_n_buttons(hd_fig, text, {'OK'}, [1]);				
			else
				text = 'No belief objects in EVN. Nothing to propagate. I skip.';
				choice = text_with_n_buttons(hd_fig, text, {'OK'}, [1]);
			end
		end
	
	case 'Display_BetP'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% display the BetP plots
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		hide_plot('EVN_plot')
		if plot_betp_done 
			show_back('betp_plot');
		else
			draw_histo_betp(hd_fig,hd_axes)
			plot_betp_done = 1;
		end
		displayed_plot = 'betp';
		which_menu_EVN('Display_BetP');
		
	case 'Relocate_BetP'
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% display the BetP plots in the location of the EVN nodes
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		hide_plot('EVN_plot')
		delete([histo(:).handle]) 
		histo = [];
		pin_histo = [0 0 0]; 
		draw_histo_betp(hd_fig,hd_axes)
		plot_betp_done = 1;
		displayed_plot = 'betp';
		which_menu_EVN('Relocate_BetP');
		
	case 'Display_Network' 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% going back to the EVN when I was in BetP plot
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% hiding betp abnd showing EVN  
		hide_plot('betp_plot');
		show_back('EVN_plot');
		displayed_plot ='EVN';	
		which_menu_EVN('Display_Network');
		
	case 'HelpShow'
		gcf_mem = gcf;
		help_text_handles = forced_display_helps(1,47);
		delete(help_text_handles)
		figure(gcf_mem);
		
	case  'Print EVN';
		print_figure('EVN')
		
	case 'Load EVN'
		EVN_Contro_mem.propagation_done = propagation_done;
		EVN_Contro_mem.plot_EVN_done = plot_EVN_done;
		EVN_Contro_mem.plot_betp_done = plot_betp_done;
		EVN_Contro_mem.displayed_plot = displayed_plot;
		go_on = 0;
		out_option = 'Load EVN';
	case 'Save EVN'
		EVN_Contro_mem.propagation_done = propagation_done;
		EVN_Contro_mem.plot_EVN_done = plot_EVN_done;
		EVN_Contro_mem.plot_betp_done = plot_betp_done;
		EVN_Contro_mem.displayed_plot = displayed_plot;
		go_on = 0;
		out_option = 'Save EVN';
	case 'Save Objects'
		EVN_Contro_mem.propagation_done = propagation_done;
		EVN_Contro_mem.plot_EVN_done = plot_EVN_done;
		EVN_Contro_mem.plot_betp_done = plot_betp_done;
		EVN_Contro_mem.displayed_plot = displayed_plot;
		go_on = 0;
		out_option = 'Save Objects';
	case 'Clear EVN'
		EVN_Contro_mem.propagation_done = propagation_done;
		EVN_Contro_mem.plot_EVN_done = plot_EVN_done;
		EVN_Contro_mem.plot_betp_done = plot_betp_done;
		EVN_Contro_mem.displayed_plot = displayed_plot;
		go_on = 0;
		out_option = 'Clear EVN';
	case 'Quit' % menu: quit
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% menu: quit 
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		go_on = 0;
		out_option = 'Quit EVN';
	end
end % while go_on

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%                 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%       END       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%                 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% other programming tricks, not used here but maybe later

cur_pt = get(hd_axes,'currentpoint');
fprintf(1,'%10g %10g %10g %10g %10g %10g \n',cur_pt)

set(hd_axes,'ButtonDownFcn','clb_sel_obj;');

if object_num_where_pointer > 0
	% so I click on one object, its index = object_num_where_pointer 
end

btnup

%   getting zoom     one step  with   zoom(1.5)

% Create toolbar button group.
icons = ['text(.5,.5,''B1'',''Horiz'',''center'')'
'text(.5,.5,''B2'',''Horiz'',''center'')'];
callbacks = ['disp(''B1'')' 'disp(''B2'')'];
btngroup('GroupID', 'TestGroup', ...
'ButtonID', ['B1';'B2'], ...
'Callbacks', callbacks, ...
'Position', [.475 .45 .05 .1], ...
'IconFunctions', icons);

return
btngroup(hd_fig,'GroupID','my_buttons',...
'ButtonID',strvcat('Button_1','The_Other_One'));


% RUBBERBOX si a box with corner where I push mouse and size change with pointer moving, 
% here it end with drwaing a nice rectangle
k = waitforbuttonpress;
point1 = get(gca,'CurrentPoint');    % button down detected
finalRect = rbbox;                   % return Figure units
point2 = get(gca,'CurrentPoint');    % button up detected
point1 = point1(1,1:2);              % extract x and y
point2 = point2(1,1:2);
p1 = min(point1,point2);             % calculate locations
offset = abs(point1-point2);         % and dimensions
x = [p1(1) p1(1)+offset(1) p1(1)+offset(1) p1(1) p1(1)];
y = [p1(2) p1(2) p1(2)+offset(2) p1(2)+offset(2) p1(2)];
hold on
axis manual
plot(x,y)                            % redraw in dataspace units

return

% VERY USEFUL
% for dragging an icone from here to there
% getting the mouse on an icone, dragging a fake rectangle and when button off
% r2 is new position (where icone dropped)
waitforbuttonpress
point1 = get(gcf,'CurrentPoint') % button down detected
rect = [point1(1,1) point1(1,2) 50 100]
[r2] = dragrect(rect)
return

%for i = 1:10

set(gca,'CameraPosition', [500 300 1000])
drawnow
%end
% set(gca,'CameraViewAngle',.05*get(gca,'CameraViewAngle')+5)





