function [hd_one_panel,hd_title,hd_text] = tabulator(title,text_comment,dim_table,...
LLposition,heavy_lines,title_table,comment_table)
% draw a frame in LLpositon with all the boxes in nl x nc matrix,
% a frame around in bold lines
% data are   entered as set(...)
% no scrolling, for tutorials only
% title of the window, a free text in a box under the title, 

% nbr_char_data = max nbr of char for a data (=width cells)
% nbr_char_left = max nbr of char for left column (=width cells at left) 0 = none

% dim_table = for each table = nrow, ncol, option_borders, nbr_char_width, nbr_char_left_col
% option_borders = 0, no border, 1 = upper border, 2 = left border, 3 = left+upper borders

% INPUT				
% title				'blabla'
% text_comment		{'text'}
% dim_table(tab,1:5) = [nr ncol opt ncw nc ncl]
% LLposition = [n1 m1; n2 m2; ...];
% heavy_lines(tab).hori{k} = [3 5 8] hori 3, from 5 to 8,  or [3] (full line)
% heavy_lines(tab).vert{k} = [3 5 8] vert 3, from 5 to 8,  or [3] (full line)
% heavy_lines(tab).hori = []; when no hori heavy lines
% heavy_lines(tab).vert = []; when no vert heavy lines
% title_table = {'t','tt','ttt',...}	
% comment_table = {'t1','t2',...}

% OUTPUT
% hd_one_panel(i,j) i=1:nr,j=1:nc = cells in table
% hd_one_panel(nr+1,j) = upper line
% hd_one_panel(i,nc+1) = left column
% hd_one_panel(nr+1,nc+1) = upper_corner
%
% hd_title : the handle of the title
% hd_text : the handle of ext just under the title

%
%   Version : May 1, 2003.
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global screen_size
global Which_Courier

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% get the figure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hd_one_panel = [];
hd_title = [];
hd_text = []; 

% fontsize for Which_Courier
switch computer
case 'MAC2'
	ftsz = 10;
	ftn = Which_Courier;
otherwise
	ftsz = 9;
	ftn = Which_Courier;
end

scr_size = screen_size;
pos_fig = [10   60   scr_size(3)-30   scr_size(4)-120];

hd_fig = figure(...
'name','Tabulator Window',...
'DefaultuicontrolUnits','pixels',...
'DefaultuicontrolFontName',ftn,...
'Color','white',...
'DefaultuicontrolBackgroundColor','white',...
'DefaultuicontrolFontSize',ftsz);
set(gcf,'position',pos_fig);
pos_free = [0 0 pos_fig(3) pos_fig(4)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The title
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hd_title = -1;
switch computer
case 'Mac2'
	text_h = 20;
case 'PCWIN'
	text_h = 24;
otherwise
	text_h = 24;
end

if ~isempty(title)
	pos_title = [5 pos_fig(4)-1.5*text_h  pos_fig(3)-5   text_h];
	pos_free = [0  0  pos_fig(3) pos_fig(4)-2*text_h];
	hd_title = uicontrol(gcf,'position',pos_title,'style','text','string',title,...
	'horizontalalignment','center','FontSize',18,'ForegroundColor','r');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The text box under the title
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hd_text = -1;
if ~isempty(text_comment)
	hd_text = uicontrol(gcf,'position',pos_free+[10 0 -20 0],'style','frame');
	[outtext, pos_text_comment] = textwrap(hd_text(1) ,text_comment);
	delete(hd_text);
	pos_text = [5 pos_free(4)-pos_text_comment(4) pos_fig(3)-5  pos_text_comment(4)];
	pos_free = [0  0  pos_free(3) pos_free(4)-pos_text(4)];	
	hd_text = uicontrol(gcf,'position',pos_text,'style','text','string',outtext);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% for every panel: 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nbr_panels = size(dim_table,1);

for i = 1:nbr_panels
	hd_one_panel{i} = panel_tabulator(dim_table(i,:),pos_free,LLposition(i,:),...
	heavy_lines(i),title_table{i},comment_table{i});
end
