function gbt_demo

%
%   Version : April 30, 2003
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the input condit bba on X given ti, i=1,...I in  bba(i).values 
% the prior on H in bba(I+1).values 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cardx = 3; 
pcx = pow2(cardx);
cardh = 3;
pch = pow2(cardh);

for i = 1:cardh
	m = fix(100*sort([rand(pcx-1,1);1]))/100;
	bba(i).values  = [m(1); diff(m)];
end

m = fix(100*sort([rand(pcx-1,1);1]))/100;
bba(cardh+1).values  = [m(1); diff(m)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% title 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

title = 'The Generalized Bayesian Theorem (GBT): Demonstrator.';
text_comment = {['Input = mX[hi] for i = 1,2,3, an a priori on H, and a subset x of X.',...
'Output: Bel, BetP, Pl on the singletons of H, and mH({})',...
' To create an example, you provide the prior on H, the conditional bba on X given hi, i=1,2,3, and the observation data.']};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the loop 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cw = 8; % col width
dim_table = [ 8 1 3 cw 9; 8 3 3 cw 9 ; 8 3 3 cw 9 ; 8 4 3 cw 9 ; 8 3 3 cw 9 ];

ht = 300;
switch computer
case 'MAC2'
	LLposition = [30 ht  ; 200 ht  ; 450 ht ; 700 ht ; 200 50];
case 'PCWIN'
	LLposition = [10 ht  ; 150 ht  ; 410 ht ; 670 ht ; 150 50];
otherwise
	LLposition = [30 ht  ; 200 ht  ; 450 ht ; 700 ht ; 200 50];
end

for i = 1:5
	heavy_lines(i).vert{1} = 2;
	heavy_lines(i).hori{1} = 2;
end

title_table = {'prior mH','mX[hi] = cond. bba on X given hi','Simple support functions on H',...
'Posterior potential on H','plX[hi] = cond. pl on X given hi'};
comment_table = {'','','','',''};

leftx = {'{}','x1','x2','x1,x2','x3','x1,x3','x2,x3','x1,x2,x3'};
lefth = {'{}','h1','h2','h1,h2','h3','h1,h3','h2,h3','h1,h2,h3'};

topc{1} = {'mH'};
topc{2} = {'mX[h1]','mX[h2]','mX[h3]'};
topc{3} = {'[h1]','[h2]','[h3]'};
topc{4} = {'m','Bel','Pl','BetP'};
topc{5} = {'plX[h1]','plX[h2]','plX[h3]'};

nc = [1 3 3 4 3];



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% draw tabulator  
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[hd_panel,hd_title,hd_text] = tabulator(title,text_comment,dim_table,LLposition,...
heavy_lines,title_table,comment_table);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% put upper/left data
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:8
	set(hd_panel{1}(i,nc(1)+1),'string',lefth{i});
	set(hd_panel{2}(i,nc(2)+1),'string',leftx{i});
	set(hd_panel{3}(i,nc(3)+1),'string',lefth{i});
	set(hd_panel{4}(i,nc(4)+1),'string',lefth{i});
	set(hd_panel{5}(i,nc(5)+1),'string',leftx{i});	
end

for j = 1:5
	for i = 1:nc(j)
		set(hd_panel{j}(9,i),'string',topc{j}{i});
	end		
	set(hd_panel{j}(9,nc(j)+1),'string','H');
end
set(hd_panel{2}(9,nc(2)+1),'string','X');
set(hd_panel{5}(9,nc(5)+1),'string','X');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the data loop
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xsel = 4; % initial conditioning event
go_on = 1;
while go_on
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% enterin g the priors (random)  in bba(i).values, i = 1:3 for [hi] and 4 for prior on H
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% prior on H
	for j = 1:8
		if bba(4).values(j) == 0
			tx = '    ';
		else
			tx = sprintf(' %4.2f ',bba(4).values(j));
		end
		set(hd_panel{1}(j,1),'string',tx,'horizontalalignment','center');
	end

	% mX[hi]
	for i = 1:3
		for j = 1:8
			if bba(i).values(j) == 0
				tx = '    ';
			else
				tx = sprintf(' %4.2f ',bba(i).values(j));
			end
			set(hd_panel{2}(j,i),'string',tx,'horizontalalignment','center');
		end
	end
	% plX[hi]
	for i = 1:3
		pl(i,1:8) = fmt(bba(i).values,'m','pl')';
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% the computed data
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	for i = 1:3
		ssf(i,1:8) = zeros(1,8);
		ssf(i,8) = 1-pl(i,xsel);
	end
	ssf(1,7) = pl(1,xsel);
	ssf(2,6) = pl(2,xsel);
	ssf(3,4) = pl(3,xsel);
	
	q(1:8) = fmt(ssf(1,:)','m','q').*fmt(ssf(2,:)','m','q').*fmt(ssf(3,:)','m','q').*fmt(bba(4).values,'m','q');
	
	gbt(1,1:8) = fmt(q,'q','m')';
	gbt(2,1:8) = fmt(q,'q','bel','n')';
	gbt(3,1:8) = fmt(q,'q','pl','n')';
	prob = fmt(q,'q','betp','n');
	gbt(4,1:8) = zeros(1,8);
	gbt(4,[2 3 5]) = prob';
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% display computed data
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% SSF on H
	for i = 1:3
		for j = 1:8
			if ssf(i,j) == 0
				tx = '    ';
			else
				tx = sprintf(' %4.2f ',ssf(i,j));
			end
			set(hd_panel{3}(j,i),'string',tx,'horizontalalignment','center');
		end
	end

	% gbt results on H
	for i = 1:4
		for j = 1:8
			if gbt(i,j) == 0
				tx = '    ';
			else
				tx = sprintf(' %4.2f ',gbt(i,j));
			end
			set(hd_panel{4}(j,i),'string',tx,'horizontalalignment','center');
		end
	end

	% plX[hi]
	for i = 1:3
		for j = 1:8
			if pl(i,j) == 0
				tx = '    ';
			else
				tx = sprintf(' %4.2f ',pl(i,j));
			end
			set(hd_panel{5}(j,i),'string',tx,'horizontalalignment','center');
		end
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% what to do next?
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% proceed quit area	
	
	set(gcf,'DefaultuicontrolUnits','normalized')
	
	text = 'What next? proceed and enter new bba''s, or proceed and keep bba''s (only change = conditioning event), or quit';
	temp_button_text = {'Go-New','Go-Keep','Quit'};
	choice = text_with_n_buttons(gcf, text, temp_button_text, [-1]);
	
	switch choice 
	case 1
		% proceed ,new bba
		kp = 0;		
	case 2
		% proceed , keep bba, change conditioning event only
		kp = 1;
	case 3
		go_on = 0;
	end

	if go_on == 1
		if kp == 0
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			% changing some bba area
			% we blank table 3 4 5 so user does not get surprise that data there do not fit the new entered one
			for i = 1:8
				for ik = 1:3
					set(hd_panel{3}(i,ik),'string',' ','horizontalalignment','center');
					set(hd_panel{5}(i,ik),'string',' ','horizontalalignment','center');
				end
				for ik = 1:4
					set(hd_panel{4}(i,ik),'string',' ','horizontalalignment','center');
				end
			end				
		
			ttx = {'H','X given h1','X given h2','X given h3'};	
			for ib = 1:4
				text = ['For the bba on ',ttx{ib},'.'];
				temp_button_text = {'Keep','Vacuous','EquiPr','Random','Keybrd'};
				choice = text_with_n_buttons(gcf, text, temp_button_text, [-1]);
				
				ibk = ib - 1;if ib == 1; ibk = 4; end
				ibb = 1;ik = 1;if ib > 1; ibb = 2;ik = ib-1; end
				switch choice
				case 1
					% nothing
				case 2
					bba(ibk).values = [zeros(7,1) ; 1];
				case 3
					v = zeros(8,1) ; v(2) = 1/3;v(3) = 1/3;v(5) = 1/3;	
					bba(ibk).values = v;
				case 4
					v = fix(100*sort([rand(pcx-1,1);1]))/100;
					bba(ibk).values  = [v(1); diff(v)];
				case 5			
					% blank data
					for i = 1:8
						set(hd_panel{ibb}(i,ik),'string',' ','horizontalalignment','center');
					end				
					text = ['Enter the bba on ',ttx{ib},'.'];
					temp_button_text = {'Done'};
					choice = text_with_n_buttons(gcf, text, temp_button_text, [-1]);
					for i = 1:8
						v(i) = get_num_gui(hd_panel{ibb}(i,ik));
					end
					bba(ibk).values = v;
				end
				% display neede here for the user, otherwise he does not see what he has done
				for j = 1:8
					if bba(ibk).values(j) == 0
						tx = '    ';
					else
						tx = sprintf(' %4.2f ',bba(ibk).values(j));
					end
					set(hd_panel{ibb}(j,ik),'string',tx,'horizontalalignment','center');
				end
			end % for ib = 1:4
		end % if kp == 0
	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% Conditioning event area
		text = 'Conditioning event, select a subset of X.';
		temp_button_text = {'x1','x2','x3','x1,x2','x1,x3','x2,x3'};
		choice = text_with_n_buttons(gcf, text, temp_button_text, [-1]);
		switch choice
		case 1 
			xsel = 2;
		case 2 
			xsel = 3;
		case 3 
			xsel = 5;
		case 4 
			xsel = 4;
		case 5 
			xsel = 6;
		case 6 
			xsel = 7;
		end
	
	end % if go_on == 1
	set(gcf,'DefaultuicontrolUnits','pixels');
end % while go_on

delete(gcf)

