function mobius_matrices
% a display of the matrices that transform forms into forms
% no normalization anywhere

%
%   Version : May 1, 2003.
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global Which_Courier
global gui_position  figure_data  flags  help_level help_label


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building the various matrices XfrY, some do not exist
% forms = m b q bel pl , never normalized
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

card = 3;
nr = pow2(card);
J = zeros(nr,nr);
for i = 1:nr
	J(i,nr+1-i) = 1;
end
one = ones(nr,1);
label = {'M','B','Q','Bel','Pl'} ; 

% the bases of the 3 basic one used in Kron products
MfrB0 = [1  0 ; -1  1]; % inv(B)
MfrQ0 = [1 -1 ;  0  1]; % J*inv(B)*J;
BfrM0 = [1  0 ;  1  1]; % B
BfrQ0 = [1 -1 ;  1  0]; % B*J*inv(B)*J;
QfrM0 = [1  1 ;  0 1]; % J*B*J;
QfrB0 = [0  1 ; -1 1]; % J*B*J*inv(B);

% the 3 basic ones
for in = 1:3
	for out = 1:3
		if in ~= out
			mat0 = eval([label{out},'fr',label{in},'0']);
			mat = 1;
			for i = 1:card
				mat = kron(mat0,mat);
			end	
			eval([label{out},'fr',label{in},' = mat;']);
		end
	end
end

% building the PlfrX
PlfrM = ones(nr,nr) - J*BfrM; % singular
PlfrB = [zeros(nr,nr-1) one] - J ; % singular
PlfrQ = [one zeros(nr,nr-1)] - J*BfrQ; % singular
PlfrBel = [zeros(nr,nr-1) one] - J; % singular

% building the BelfrX
BelfrM = BfrM - [one zeros(nr,nr-1)]; % singular
BelfrB = eye(nr) - [one zeros(nr,nr-1)]; % singular
BelfrPl = [zeros(nr,nr-1) one] - J; BelfrPl(nr,1) = 0; % singular but could be desingularized
% desingularisation could be BelfrPl(nr,1) = 1; BelfrPl(1,1) = 1;
BelfrQ = BelfrPl*PlfrQ; % singular

% Impossible to writer (M B Q)fr(Bel Pl) as a matrix: there is always a missing term (the reverse matrix are singular).
% In fact 
% MfrBel = MfrB and put 1-bel(W) in m({})
% MfrPl = ...   and put 1-pl(W) in m({})
% BfrBel = eye   and add 1-b(W)  to everyone and I cann't get the 1 from bel
% BfrPl          and add 1-pl(W) to everyone and I cann't get the 1 from bel
% QfrBel I miss the q({}) = 1
% QfrPl  I miss the q({}) = 1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% opening the figure in tabulator
% one option = 6 basic table (M B Q)
% the other 8 with Bel,Pl
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

title = 'The matrices that transform the m,b,q  into each other.';
text_comment = {['For the case ''X from Y'', you multiply the ''X from Y'' matrix with the column vector representing the Y function,',...
' the result is the column vector representing the X function. For bel and pl, use bel = b - b({})*1 and pl = 1 - J*b',...
' (where 1 is the column vector with elements 1, and J is the matrix with 1 on the minor (UR to LL) diagonal, 0 otherwise).']};
dim_table = ones(6,1)*[8 8 3 4 5];

switch computer
case 'MAC2'
	h1 = 330; h2 = 100;
case 'PCWIN'
	h1 = 300; h2=20; 
otherwise
	h1 = 300; h2=20; 
end
l1 = 10; ltab = 300;

LLposition = [l1 h1;   2*l1+ltab h1;   3*l1+2*ltab h1;   l1 h2;   2*l1+ltab h2;    3*l1+2*ltab h2];

for i = 1:5
	heavy_lines(i).hori{1} = [6 2 10];
	heavy_lines(i).hori{2} = [4 2 6];
	heavy_lines(i).hori{3} = [2];
	heavy_lines(i).vert{1} = [2];
	heavy_lines(i).vert{2} = [6 2 10];
	heavy_lines(i).vert{3} = [4 2 6];	
end
heavy_lines(6).hori{1} = [6 2 10];
heavy_lines(6).hori{2} = [8 6 10];
heavy_lines(6).hori{3} = [2];
heavy_lines(6).vert{1} = [2];
heavy_lines(6).vert{2} = [6 2 10];
heavy_lines(6).vert{3} = [8 6 10];	

title_table = {'m from b = inv(B)','m from q = J*inv(B)*J','b from m = B',...
'b from q = B*J*inv(B)*J','q from m = J*B*J','q from b = J*B*J*inv(B)'};
comment_table = {'','','','','',''};

% dim_table = for each table = nrow, ncol, option_borders, nbr_char_width, nbr_char_left_col

[hd_one_panel,hd_title,hd_text] = tabulator(title,text_comment,dim_table,...
LLposition,heavy_lines,title_table,comment_table);

line_label = {'{}','a','b','ab','c','ac','bc','abc'};
for i_tab = 1:6
	for j = 1:8
		set(hd_one_panel{i_tab}(j,9),'string',line_label{j},'horizontalalignment','center');
		set(hd_one_panel{i_tab}(9,j),'string',line_label{j},'horizontalalignment','center');
	end	
end

for i_tab = 1:6
	switch i_tab
	case 1		
		mat = MfrB;
	case 2		
		mat = MfrQ;
	case 3		
		mat = BfrM;
	case 4		
		mat = BfrQ;
	case 5		
		mat = QfrM;
	case 6		
		mat = QfrB;
	end	
	for i = 1:8
		for j = 1:8
			if mat(i,j) == 0
				tx = '    ';
			else
				tx = sprintf(' %2i ',mat(i,j));
			end
			set(hd_one_panel{i_tab}(i,j),'string',tx,'horizontalalignment','center');
		end
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% put its menus
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ind_fig = length(figure_data) + 1;
k=0;
k=k+1;make_fig_menu(ind_fig,k,'File',0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Quit',1,'general_clb(2)','on'); 
k=k+1;make_fig_menu(ind_fig,k, help_label,0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Help', 3,'general_clb(1)','on');

menu_manager('create',ind_fig)

go_on = 1;
while go_on
	uiwait(gcf)	
	if flags == 1	
		gcf_mem = gcf;
		help_level_mem = help_level;
		help_level = 1;
		help_text_handles = display_helps(1,48,Which_Courier);
		% text 48 in init_stored_texts 
		delete(help_text_handles)
		help_level = help_level_mem;
		figure(gcf_mem);
	else
		% set(gcf,'DefaultuicontrolUnits','normalized')
		% choice = text_with_n_buttons(gcf,'To quit, select it',{'Quit'}, [-1]);
		delete(gcf)
		go_on = 0; % we quit
		figure_data(ind_fig) = [];
	end	
end

