function potential_tabulator
% showing a tabulator to compute all potentails on space with 3 elements 

%
%   Version : April 30, 2003
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% title 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

title = 'The various potential forms';
text_comment = {['Given any of the functions, you get all the others.',...
' Except for BetP, all values for a function are required. For BetP, only the values for the singletons are required.',... 
' For BetP, you get the least committed isopignistic potential',...
' If you want to enter data yourself, select the Keybrd button, enter the data, then select in the new panel the button indicating the nature of the entered data.',...
' The potential forms are : m = basic belief assignment, Bel = normalized belief function, pl = plausibiulity function,',...
' b = implicaibility funciton, q = commonality function, wc = canonical conjunctive weigths, wd = canonical disjunctive weigths, BetP = pignistic probability function']};

card = 3;
m = fix(100*sort([rand(pow2(card)-1,1);1]))/100;
m = [m(1); diff(m)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the loop 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

go_on = 1;
while go_on
	
	nr = pow2(card);
	nc = 8;
	dim_table = [ nr nc 3 9 11];
	LLposition = [0 0];
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% draw tabulator  
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	heavy_lines(1).vert{1} = 2;
	heavy_lines(1).hori{1} = 2;
	title_table = {'Potential Computer'};
	comment_table = {['Values of the various potential given the entered one.',...
	' Bel and BetP are normalized, the others not. wc only if m(universe)>0 (non dogmatic bba), wd only if m({})>0. BetP only on singletons.'...
	]};
	
	[hd_panel,hd_title,hd_text] = tabulator(title,text_comment,dim_table,LLposition,...
	heavy_lines,title_table,comment_table);
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% put upper/left data
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	leftc = {'{}','{a}','{b}','{a,b}','{c}','{a,c}','{b,c}','{a,b,c}',...
	'{d}','{a,d}','{b,d}','{a,b,d}','{c,d}','{a,c,d}','{b,c,d}','{a,b,c,d}'};
	for i = 1:nr
		set(hd_panel{1}(i,nc+1),'string',leftc{i});
	end
	topc = {'m','Bel','pl','q','b','wc','wd','BetP'};
	for i = 1:nc
		set(hd_panel{1}(nr+1,i),'string',topc{i});
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% enter random masses and all others
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	for j = 1:8
		switch j
		case 1
			v = m; 
		case 2
			v = fmt(m,'m','bel')';
		case 3
			v = fmt(m,'m','pl')';
		case 4
			v = fmt(m,'m','q')';
		case 5
			v = fmt(m,'m','b')';
		case 6
			if m(2^card) == 0
				% dogmatic bel, no wc here
				v = zeros(2^card,1);
			else
				v = fmt(m,'m','w')';
			end
		case 7
			if m(1) == 0
				% if m({}) = 0, no wd here
				v = zeros(2^card,1);
			else
				v = fmt(m,'m','wd')';
			end
		case 8
			v = zeros(nr,1);
			v([pow2(0:card-1)+1]) = fmt(m','m','betp')';
		end
		v = round(1000*v)/1000;	
		for i = 1:nr
			if v(i) == 0
				set(hd_panel{1}(i,j),'string',sprintf('   0   ',[]));
			else			
				set(hd_panel{1}(i,j),'string',sprintf(' %5.3f ',v(i)));
			end
		end
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% put buttons to select cardinal = 2,3,4 + beween random masses or entered masses
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	cardm = card;
	set(gcf,'DefaultuicontrolUnits','normalized')
	choice = text_with_n_buttons(gcf,'What next: select another cardinality, get new data or opt for entering data from keyboard (Keybrd).', {'2','3','4','Random','Keybrd','Quit'}, [-1]);
	skip_k = 0;
	
	switch  choice 
	case {1,2,3} % new cardinal
		card = choice + 1;
		nr = pow2(card);
		in_vect = 'm';
		if card ~= cardm
			v = fix(100*sort([rand(nr-1,1);1]))/100;
			m = [v(1); diff(v)];
			skip_k = 1;
		end
	case 4 % get random
		v = fix(100*sort([rand(nr-1,1);1]))/100;
		m = [v(1); diff(v)];
		in_vect = 'm';
	case 5 % enter with keyboard
		mode = text_with_n_buttons(gcf,'Enter data in ONE column, then select the label of the column.', {'m','Bel','pl','q','b','wc','wd','BetP'}, [-1]);
		set(gcf,'DefaultuicontrolUnits','pixels');
		
		for i = 1:nr
			v(i) = get_num_gui(hd_panel{1}(i,mode)); 
		end
	
		switch mode
		case 1
			m = v;
			in_vect = 'm';
		case 2	
			m = fmt(v,'bel','m');
			in_vect = 'bel';
		case 3	
			m = fmt(v,'pl','m');
			in_vect = 'pl';
		case 4	
			m = fmt(v,'q','m');
			in_vect = 'q';
		case 5	
			m = fmt(v,'b','m');
			in_vect = 'b';
		case 6	
			m = fmt(v,'w','m');
			in_vect = 'w';
		case 7	
			m = fmt(v,'wd','m');
			in_vect = 'wd';
		case 8
			in_vect = 'betp';
			% for betp data only on the singletons, thus length(v) = card
			v = v([pow2(0:card-1)+1]);	
			m = fmt(v,'betp','m');
		end
	case 6 % quit
		go_on = 0;
	end
	gcfm = gcf;
	delete(gcfm)	
end

