function [op,param] = parse_inst(inst)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% instruction parser
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% textbelow = {'op(x): x = r,c means row-column coordinates where object starts. x=r means x=1,c.',...
% 'op(x1;x2;out): xi as above. out is form for results.',...
% 'gbt(x1;...;xn;xr;out): xi condit. pot., xr = cell conditioning event. out = form.',...
% 'drc(x1;...;xn;out). Out = b. disc(x;.7;out): potential in x discounted by .7. label(#,a), bba(#)'};

misspel = 0;
% inst = ddblank(inst); done before
ind1 = findstr(inst,'(');
ind2 = findstr(inst,')');
indv = findstr(inst,',');
indw = findstr(inst,';');

param = [];

try
	if isempty(ind1) | isempty(ind2) 
		misspel = 1;
	else
		if ~isempty(indv)
			if	indv(1) < ind1 | indv(length(indv)) > ind2 
				misspel = 1;
			end
		end
	end

	if misspel == 0
		op = lower(inst(1:ind1-1));
		switch op
		case 'label' % card, label1
			cardi = str2num(inst(ind1+1:indw(1)-1));
			param = build_subsets(cardi,inst(indw(1)+1:ind2-1));
			if length(param) == 0 | ~isnumeric(cardi)
				misspel = 1;
			end
			param{length(param)+1} = cardi;
		case 'key' % card, out
			param{1} = str2num(inst(ind1+1:indw(1)-1));
			param{2} = inst(indw(1)+1:ind2-1);	
			if  ~isnumeric(param{1}) | ~ismember(lower(param{2}),{'m','b','q','bel','pl','wc','wd','betp'})
				misspel = 1;
			end
		case 'bba'
			param{1} = str2num(inst(ind1+1:ind2-1));
			if  ~isnumeric(param{1}) 
				misspel = 1;
			end		
		case {'m','b','q','bel','pl','wc','wd','betp','n','-'} % x
			x = str2num(inst(ind1+1:ind2-1));
			if length(x) == 1
				x = [1 x];
			end
			if length(x) ~= 2
				misspel = 1;
			end	
			param{1} = x;
		case 'disc'  % x;d;out
			x1 = str2num(inst(ind1+1:indw(1)-1));
			if length(x1) == 1
				x1 = [1 x1];
			end
			if length(x1) ~= 2
				misspel = 1;
			end			
			param{1} = x1;
			param{2} = str2num(inst(indw(1)+1:indw(2)-1));	
			param{3} = inst(indw(2)+1:ind2-1);
			if isempty(param{2}) | ~ismember(lower(param{3}),{'m','b','q','bel','pl','wc','wd','betp'})
				misspel = 1;
			end	
		case {'ccr','dcr','gbt','drc'} 
			% ccr dcr x1;x2;...;xn;out   xn = prior on condit domain, if vacuous ;
			% gbt = x1;x2;...;xn;x_prior;cond_event;out   x_prior may be empty, vacuaous bf
			% drc = x1;x2;...;xn;x_prior;out              x_prior may be empty, vacuaous bf
			fr = ind1+1;
			for i = 1:length(indw)
				to = indw(i);
				x = str2num(inst(fr:to-1));
				if length(x) == 1
					x = [1 x];
				end
				param{i} = x;
				fr = to+1;
			end
			param{length(param)+1} = inst(fr:ind2-1);
			
			switch op
			case {'ccr','dcr'}
				for i = 1:length(param)-1
					if length(param{i}) ~= 2
						misspel = 1;
					end	
				end
				if ~ismember(lower(param{length(param)}),{'m','b','q','bel','pl','wc','wd','betp'})
					misspel = 1;
				end				
			case 'gbt'
				for i = [1:length(param)-3   length(param)-1]
					if length(param{i}) ~= 2
						misspel = 1;
					end	
				end
				if ~ismember(lower(param{length(param)}),{'m','b','q','bel','pl','wc','wd','betp'})
					misspel = 1;
				end	
			case 'drc'
				for i = 1:length(param)-2  
					if length(param{i}) ~= 2
						misspel = 1;
					end	
				end
				if ~ismember(lower(param{length(param)}),{'m','b','q','bel','pl','wc','wd','betp'})
					misspel = 1;
				end	
			end
		end		
	end
catch
	misspel = 1;
end

if misspel 
	op = 'wrong';
	param = [];
end					







