function menu_manager(option,ind_fig)
% managing all menus in the gcf figure which has index = ind_fig

% figure_data = an object storing all data about the figure
% figure_data(ind_fig).menu satores the data relative to the menus

% those data entered from calling fcn
% menu(k).label		= label of k'th menu 
% menu(k).father 	= in the tree, the index of its father, = 0 if root 
% menu(k).clb 		= its callback  (a string) 
% menu(k).enable 	= 'on' or 'off'  the value of its enable field 
% those data created when creating menus by this function
% menu(k).handle 	= its handle 
% menu(k).depth 	= depth in the tree, a scalar (root menu = depth 0)
% menu(k).path 		= path in the tree like [4 2 5] if in root 4, child 2, child 5 of 2
% menu(k).seq_nbr 	= index of menu k in the displayed sequence (depth first)

% option 	= 'create' = create it
% = 'delete' = delete it, hardly used as delete(gcf) does the task
% = 'enable' = update the enable status

% beware of the order, order of dispaly is order under which menu are entered

%
%   Version : Aug 14, 03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global figure_data

now_menu = figure_data(ind_fig).menu;

switch option
case 'create'
	% determining everybody path
	for i = 1:length(now_menu)
		ancestors = [];
		ancestors(1) = now_menu(i).father;
		while ancestors(1) 
			ancestors = [now_menu(ancestors(1)).father ancestors];
		end
		ancestors(1) = []; % as it was 0 for the root, root get empty ancestors list		
		now_menu(i).depth = length(ancestors);
		now_menu(i).path = ancestors;
	end	

 	ind_seq = find([now_menu(:).father] == 0); % the list of root indexes	
	for i = 1:max([now_menu(:).depth])		
		ind_now = find([now_menu(:).depth] == i); % those with depth i
		ind_seq_depth = ind_seq;
		
		for j = 1:length(ind_now)		
			pp = find(ind_seq_depth == now_menu(ind_now(j)).father);
			if pp < length(ind_seq_depth)
				nx = ind_seq_depth(1+pp);
				to = find(ind_seq == nx);
			else
				to = length(ind_seq) + 1;
			end
			ind_seq = [ind_seq(1:to-1) ind_now(j) ind_seq(to:end)];
		end
	end
	% ind_seq is the sequence under which the menu are displayed
	% ind_seq = [2 1 3] means I list menu(2), then menu(1), then menu(3)
	
	for i = 1:length(now_menu)
		now_menu(ind_seq(i)).seq_nbr = i;
	end
	% menu(2).seq_nbr = 1, menu(1).seq_nbr = 2, menu(3).seq_nbr = 3, 
	
	for i = 1:length(now_menu)
		j = find([now_menu(:).seq_nbr] == i);
		
		if now_menu(j).father == 0
			% it is a root
			now_menu(j).handle = uimenu(gcf,...
			'label', now_menu(j).label,...
			'callback', now_menu(j).clb);
		else
			% a child
			now_menu(j).handle = uimenu(now_menu(now_menu(j).father).handle,...
			'label', now_menu(j).label,...
			'callback', now_menu(j).clb);
			if strcmp(now_menu(j).separator,'on')
				set(now_menu(j).handle,'Separator','on');
			end
		end
	end

	for i = 1:length(now_menu)
		figure_data(ind_fig).menu(i).depth = now_menu(i).depth;
		figure_data(ind_fig).menu(i).handle = now_menu(i).handle;
		figure_data(ind_fig).menu(i).path = now_menu(i).path;
		figure_data(ind_fig).menu(i).seq_nbr = now_menu(i).seq_nbr;
	end

case 'delete'
	for k = max([now_menu(:).depth]):-1:0
		to_del = find([now_menu(:).depth] == k)
		for i = 1:length(to_del)
			delete(now_menu(to_del(i)).handle)
		end	
	end
	figure_data(ind_fig).menu = [];
	
case 'modify'
	for i = 1:length(now_menu)
		set(now_menu(i).handle,'enable',now_menu(i).enable,...
		'label',now_menu(i).label,...
		'callback',now_menu(i).clb)
	end	
end

