function [pushed_button, hd_buttons] = select_buttons(Wdw_Cmd_handle, positionXY, text, title,...
hidden_buttons, tooltip, index_set_but, pos_help_text)
% drawing a vertical set of pushbuttons with a frame and with a title
% buttons can have tooltip and long help texts attached to each button 
% and that shows up when pointer is located on the button

% input 
% Wdw_Cmd_handle = handle of the figure where button window will show up
% positionXY	= upper left position of window, relative to Wdw_Cmd_handlele
%				= [] means centered in gco
% text  		= the list of texts (cells) that go into the buttons
% title			= the displayed title
% hidden_buttons = the buttons that are not shown and not active
%   			= []    or    [i j k] 
% tooltip		= short help text attached to the buttons; show up with small delay
%				  string 'text', if [] no tooltip to show
% index_set_but	= a nunber that tells which set of long help texts is to be used
%				  to see the displayed texts, see get_help_text_for_set_buttons
%				  0 = no long help texts
% pos_help_text = UR or UL or Other (decided by call)

% texts are in get_help_text_for_set_buttons
% output
% answer = the index of the selected button 
% hd_buttons(i) = the button handles 
% 1 : external frame
% 2 : title
% 3 : button 1
% 4 : button 2 
% ...

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global flags   gui_position   width_long_text_help_window
% a set of global used only for callback, clear when button clicked
global  button_num_where_pointer  long_text_help   position_figure_for_text  
global  pos_button_in_pixels    hd_lg_tx_fig  top_position_for_text   help_level

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% preparing for long helps
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

% deciding where to display help text
switch pos_help_text
case 'UL' 
	position_figure_for_text = [2*gui_position(1,1) 1.1*gui_position(2,1) .9*gui_position(3:4,1)'];
case 'UR'
	position_figure_for_text = [1.1*gui_position(1:2,5)' ...
	width_long_text_help_window*gui_position(3,5)   .9*gui_position(4,5)];
case 'Other'
	% I don't know where is the image, and I put help above comments figure
	position_figure_for_text = gui_position(:,8)';		
otherwise
	% nothing 
end
button_num_where_pointer = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% preparing buttons
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

nbr_buttons = length(text);

posFig = get(Wdw_Cmd_handle,'Position') ; % not normalized as it is a figure
button_h = 20/posFig(4); 	% normalized
button_w = 130/posFig(3);	% normalized

skip_w =.01; skip_h = .01;

% getting wrapped title height
[Title_Wrapped,height_title] = height_of_wrapped_text(title,button_w,Wdw_Cmd_handle);

total_width = button_w + 2*skip_w ;
total_height = nbr_buttons*(button_h + skip_h) + 2*skip_h + height_title ;
% I put skip_h between buttons	

% getting buttons positions 

if isempty(positionXY)
	positionXY = [ .5-total_width/2   .5-total_height/2];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% displaying the external frame hd_buttons
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

pos_frame = [ positionXY(1)  ...
1 - positionXY(2) -  total_height...
total_width 	total_height ];

ind_hd_buttons = 0;
ind_hd_buttons = ind_hd_buttons + 1;
hd_buttons(ind_hd_buttons) = uicontrol('Parent',Wdw_Cmd_handle, ...
'Position',pos_frame, ...
'BackgroundColor',[.9 .9 .9],...
'tag',num2str(index_set_but),...
'Style','frame');				

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% displaying the title
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

pos = [ positionXY(1) + skip_w ...
1 - positionXY(2) - height_title - skip_h ...
total_width - 2*skip_h...
height_title];

ind_hd_buttons = ind_hd_buttons + 1;
hd_buttons(ind_hd_buttons) = uicontrol('Parent',Wdw_Cmd_handle, ...
'Position',pos,...
'String',Title_Wrapped,...
'BackgroundColor',[.9 .9 .9],...
'ForegroundColor',[1 0 0],...
'Style','text');	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% displaying the buttons 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

pos = [pos(1)    pos(2)   button_w      button_h];

for jb = 1:length(text)
	
	% char array built, looks like     pushbut '42' ;uiresume;
	% and the call back put in flags(1) the number 42
	
	clb = ['general_clb(',num2str(jb),') ;'];
	
	pos(2) = pos(2) - skip_h - button_h;	
	ind_hd_buttons = ind_hd_buttons + 1;
	
	hd_buttons(ind_hd_buttons) = uicontrol('Parent',Wdw_Cmd_handle, ...
	'style','pushbutton',...
	'Position',pos, ...
	'String',text{jb},...
	'Callback',clb);
	
	% adding the tooltip if they exist
	if ~isempty(tooltip)
		set(hd_buttons(ind_hd_buttons) ,'TooltipString',tooltip{jb});
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% display of long help texts for ecah button, only if help level = 2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
but_help_exist = 0;
if  help_level  == 2
	if index_set_but % we have long help texts
		% getting the buttons positions in pixel and their help texts
		but_help_exist = 1;
		
		[hd_lg_tx_fig,hd_title_lg_tx,long_text_help, pos_button_in_pixels] = ...
		build_button_helps(Wdw_Cmd_handle,hd_buttons,index_set_but);
		% ordering that pointer position must be followed.
		% callabck is get_button_under_pointer		
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% some buttons are disable and invisible
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
for i = 1:length(hidden_buttons)
	set(hd_buttons(hidden_buttons(i)+2),'enable','off');	
	% killing the buttons so button long help text of hidden buttons are not displayed
	if but_help_exist 
		pos_button_in_pixels(1,hidden_buttons(i)) = 2* pos_button_in_pixels(2,hidden_buttons(i));
		% so the pointer location criteria in get_text_for_button_pointer a callback of the buttons
		% is never satisfied for that buttons, it is just a dirty trick that works.
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% a button is selected
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

figure(Wdw_Cmd_handle); % so I am sure gcf is OK in uiresume
if  help_level  == 2
	set(Wdw_Cmd_handle,'WindowButtonMotionFcn','get_text_for_button_pointer;')	
end	

%%%%%%%%%%
%
uiwait(Wdw_Cmd_handle); 
% I will proceed once a button is selected
%
%%%%%%%%%%
if but_help_exist
	delete(hd_title_lg_tx)
	delete(hd_lg_tx_fig)
end
% clearing the global data without using clear global that might misbehave?
button_num_where_pointer = []; 
long_text_help = [];   
position_figure_for_text = [];
pos_button_in_pixels = [];  
hd_lg_tx_fig = [];
top_position_for_text = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% buttons has been selected
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% index of THE unique pushed button, value = 1,2,3...
% 0 menas I did not select a button, but a menu. 
if length(flags) == 0
	flags(1) = 0;
end
pushed_button = flags(1);
