function choice = text_with_n_buttons(Wdw_Cmd_handle, text, temp_button_text, poss)
% Display a text in ONE line
% and a list of n red buttons in vertical column

% Box drawn in window = Wdw_Cmd_handle
% if Wdw_Cmd_handle = [], box put in the Command Window (UR)
%
% It draws an overal frame, 
% and displays   text    as a wrapped text in a smaller frame 
%
% button_text is the list of texts taht will be put in the buttons.
% Example : button_text = {'Yes','No','Cancel'}, never forget the outside {}
% If temp_button_text = [], the buttons are OK, Cancel
%
% number of buttons = n = length(button_text)
% It puts n buttons verticaly, red, with texts in button_text 

% when one button is selected, the two frames and the buttons are deleted

% The whole box is located at position poss (normalized) of the Window
% poss contains the external frame positions.
% if poss = [], located lower left corner of Window (based on isempty(poss))
% if poss = [-1], located lower right corner of Window (based on poss(1)=-1
% if poss = [-2], located lower right corner of Window with room for scroll bar (based on poss(1)=-2
% if poss = [1], located at upper left corner of Window (based on length(poss) == 1)
% If poss = [ 0 0 0 0 ], center panels (based on length(poss) ~= 1)
% if poss = [ x y z t ], locate at (x,y) with dimensions (z,t).

% choice = the number of the selected button
% 1 for the first button (top left), 2 for the next on same line,... 

% we can also hit a key instead of selecting a button with the mouse.
% if RETURN, the first button is selected (the default answer) (the pure red button)
% if ESC,    the last  button is selected (should always be 'Cancel').

% Note: we need a figure to handle the buttons properly, and use uiwait
% as waitfor works for ONE button only

%
%   Version : Dec 30,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global button_size     keyhit     flags

if isempty(temp_button_text)
	temp_button_text = {'OK','Cancel'};
end
% verification that button_text is {'text'}
button_text = temp_button_text;
if ~iscell(temp_button_text)
	button_text = ['{',temp_button_text,'}'];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% determning space for the butons
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

v_skip_buttons = .005;
left_margin = .007;
right_margin = .008;
top_margin = .008;
bottom_margin = .009;

posFig = get(Wdw_Cmd_handle,'Position') ;

height_but= min(button_size(2)/posFig(4),.99)  ; % .05; 
width_but = min(button_size(1)/posFig(3),.99)  ; %(position(3)-4*left_margin)/3;
total_w = 200/posFig(3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% determining n (nbr buttons), Window, possition of Window
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = length(button_text);  

% % % % % % % if isempty(Wdw_Cmd_handle)	
% % % % % % % 	[index_fig, hd_fig] = handling_all_figures('put_in_front',{'General Window'}); 
% % % % % % % 	Wdw_Cmd_handle = hd_fig(1); 
% % % % % % % end

if isempty(poss) % LL
	position = [.005 .005 total_w .99];
	which_pos = 'LL';
elseif poss(1) == - 1 % LR
	position = [.995-total_w .005 total_w .99];
	which_pos = 'LR';
elseif poss(1) == - 2 % LR with scroll bar at right
	position = [.965-total_w .005 total_w .99];
	which_pos = 'LR';
elseif length(poss) == 1 % UL
	position = [.005 .995 total_w .99];
	which_pos = 'UL';
elseif length(poss) > 1 & sum(poss) == 0
	% UL position
	position = [.5-total_w/2 0 total_w .99];
	which_pos = 'Center';
else
	position = reshape(poss,1,4); % to be sure it is a line vector	
	which_pos = 'Imposed';
end	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% determning space for the buttons
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nbr_but_full_lines = fix(n/3);
nbr_but_last_line = n - 3*nbr_but_full_lines;

% the lines with 3 buttons
pos_but3(1) = position(1) + left_margin ;
pos_but3(2) = position(1) + position(3)/2 - width_but/2 ;
pos_but3(3) = position(1) + position(3) - width_but - right_margin;

nbr_but_lines = nbr_but_full_lines;
switch nbr_but_last_line
case 1 % k*3 buttons + 1 button
	pos_but(1) = position(1) + position(3)/2 - width_but/2;
	nbr_but_lines = nbr_but_full_lines + 1;
case 2 % k*3 buttons + 2 button
	pos_but(1) = position(1) + position(3)/3 - width_but/2;
	pos_but(2) = position(1) + 2*position(3)/3 - width_but/2;
	nbr_but_lines = nbr_but_full_lines + 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% height of displayed text 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[wrapped_text, v_spacen] = fit_text_in_box(Wdw_Cmd_handle, text,...
position(3) - left_margin - right_margin -.02);
v_space = sum(v_spacen) + .02;

position(4) = v_space + nbr_but_lines*(v_skip_buttons + height_but) + bottom_margin + top_margin;
switch which_pos
case 'Center'
	position(2) = .5 - position(4)/2;
case 'UL'
	position(2) = position(2) - position(4);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% opening overall frame, text frame, putting text
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% external frame 
hand(1) = uicontrol('Parent',Wdw_Cmd_handle, ...
'Position',position, ...
'Backgroundcolor',[.7 .7 .7],...
'Style','frame');	

posit = position;

posit(1) = position(1) + left_margin;
posit(2) = position(2) + position(4) - top_margin - v_space;
posit(3) = position(3) - left_margin - right_margin;
posit(4) = v_space ;

posmem = posit;

% frame for text
hand(2) = uicontrol('Parent',Wdw_Cmd_handle, ...
'Position',posit, ...
'Style','frame');	

% long text
posit = 	posit + [.005 .005 -.010 -.01];
hand(3) = uicontrol('Parent',Wdw_Cmd_handle, ...
'Position',posit, ...
'Style','text', ...
'HorizontalAlignment','left',...
'String',wrapped_text{1});	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% drawing the buttons handle = hand(4), hand(5) ....
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% the call back when hitting a key

keyhit = 0;
set(Wdw_Cmd_handle,'KeyPressFcn','callback_key_hit;uiresume;')

% the buttons
switch computer
case 'MAC2'
	butcolor = [1 0.5 0.5]; % color of half tone buttons
case 'PCWIN'
	butcolor = [1 0.6 0.6];
otherwise
	butcolor = [1 0.8 0.8];
end

posit = posmem;

posit(3) = width_but;
posit(4) = height_but;

% 0SX callback   'CallBack','set(gco,''userdata'',''ok'');uiresume;'

% for the full lines
jbut = 0;
for i_line = 1:nbr_but_full_lines 	
	posit(2) = posit(2)- height_but - v_skip_buttons;	
	for i_but = 1:3
		jbut = jbut + 1;	
		posit(1) = pos_but3(i_but);
		clb = ['general_clb(',num2str(jbut),');'];
		
		hand(jbut+3) = uicontrol(...
		'Parent', Wdw_Cmd_handle,...
		'Position',posit,...
		'Style','PushButton',...
		'String',button_text{jbut},...
		'BackgroundColor',butcolor,...
		'CallBack',clb);
	end	
end

% for the last line if incomplete
posit(2) = posit(2)- height_but - v_skip_buttons;
for i_but = 1:nbr_but_last_line
	jbut = jbut + 1;
	posit(1) = pos_but(i_but);	
	clb = ['general_clb(',num2str(jbut),');'];
	
	hand(jbut+3) = uicontrol(...
	'Parent', Wdw_Cmd_handle,...
	'Position',posit,...
	'Style','PushButton',...
	'String',button_text{jbut},...
 	'BackgroundColor',butcolor,...
	'CallBack',clb);	
end

% setting button 1 enhanced, to indicate = default
% the default button is hot red, other lighter red
set(hand(4),'BackgroundColor',[1 0 0])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% waiting for button selection or key press (RETURN or ESC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(Wdw_Cmd_handle);
choice = 0; 
while choice == 0
	flags = 0;
	uiwait(Wdw_Cmd_handle)
	
	switch keyhit
	case 1
		choice = 1;
	case 2
		choice = n;  % the nbr of buttons
	otherwise
		if flags > 0
			choice = flags;
		end
	end
end

delete(hand)
set(Wdw_Cmd_handle,'KeyPressFcn','')
