function data_out = entering_data(nr,nc,name_row, name_col,default_data,limits)
% tabel for entering data in a matrix with nr rows and nc columns
% name_rows are labels at left of rows
% name_col are lables at top of columns
% default_data are the data in eah cell when creating the table  [] = blank everywhere
% limits are boundaries for domain of each value (for verification), can be nmissing

% one can ask more data gthen given in default, sp table can be 20 x 6 with default_data = 12 X 3
% in that case default data refers to lines 1 to 12, rows 1 to 3 (upper left part)

%
%   Version : May 1,04.
%   Copyright (c) 2004 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global gui_position flag_str figure_data screen_size

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% get the figure that contains the several panels with buttons
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

namefig = ['Data Entry Window'];
hd_fig = figure('name',namefig,'position',gui_position(:,19),'Units','pixels');
ind_fig = length(figure_data) + 1;
figure_data(ind_fig).name = namefig;
figure_data(ind_fig).handle = hd_fig;
figure_data(ind_fig).position = gui_position(:,19);

scr_size = gui_position(:,19)';

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %
% % put its menus
% %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% k=0;
% k=k+1;make_fig_menu(ind_fig,k,'File',0,'','on');
% k=k+1;make_fig_menu(ind_fig,k,'Quit',1,'general_clb(''quit'')','on');
% k=k+1;make_fig_menu(ind_fig,k,'Cancel',1,'general_clb(''cancel'')','on');
% menu_manager('create',ind_fig)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the dimensions of the boxes for the names
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hd_temp = uicontrol(gcf,...
'style','text','units','pixels','position',scr_size'); 
[outtext, pos_box] = textwrap(hd_temp,name_row);
name_row_wd = ceil(pos_box(3))+5;
delete(hd_temp) 

hd_temp = uicontrol(gcf,...
'style','text','units','pixels','position',scr_size); 
[outtext, pos_box] = textwrap(hd_temp,name_col);
name_col_wd = ceil(pos_box(3))+5;
text_ht = ceil(1.3*pos_box(4)/length(name_col)); % divider as height for length(name_col
delete(hd_temp) 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building the table
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

margin = 5;

title_text = 'Enter new data in cells. To end select Done or Cancel red button.';

hd_title(1) = uicontrol(gcf,...
'style','text',...
'units','pixels',...
'string',title_text ,...
'fontsize',18,...
'position',[margin  scr_size(4)-margin-2*text_ht  scr_size(3)-margin   2*text_ht]);

hd_title(2) = uicontrol(gcf,...
'style','pushbutton',...
'string','Done',...
'units','pixels',...
'callback','general_clb(''quit'')',...
'backgroundcolor','red',...
'position',[margin+10  scr_size(4)-margin-2*text_ht+10  50   25]);

hd_title(3) = uicontrol(gcf,...
'style','pushbutton',...
'string','Cancel',...
'units','pixels',...
'callback','general_clb(''cancel'')',...
'backgroundcolor','red',...
'position',[scr_size(3)-margin-60  scr_size(4)-margin-2*text_ht+10  50   25]);

% row label
j_box = 1;
i_box = 1;
row_bottom = scr_size(4)-margin-3*text_ht-2;  % -2 for better putting text in middle of row
pos = [margin row_bottom  name_row_wd text_ht];
for i = 1:nr
	i_box = i_box + 1;
	pos(2) = pos(2) - text_ht;
	hd_box(i_box,j_box) = uicontrol(gcf,...
	'style','text',...
	'units','pixels',...
	'string',name_row{i},...
	'position',pos);
end

% column labels
j_box = 1;
i_box = 1;
row_bottom = scr_size(4)-margin-3*text_ht;
pos = [margin+name_row_wd  row_bottom  name_col_wd text_ht];
for j = 1:nc
	j_box = j_box + 1;
	hd_box(i_box,j_box) = uicontrol(gcf,...
	'style','text',...
	'units','pixels',...
	'string',name_col{j},...
	'position',pos);
	pos(1) = pos(1) + name_col_wd;
end

% default values 
i_box = 1;
row_bottom = scr_size(4)-margin-3*text_ht;
for i = 1:nr
	i_box = i_box + 1;
	j_box = 1;
	row_bottom = row_bottom-text_ht;
	pos = [margin+name_row_wd  row_bottom  name_col_wd   text_ht];
	for j = 1:nc
		j_box = j_box + 1;
		if size(default_data,1) < i | size(default_data,2) < j 
			def_dt = '';
		else
			def_dt = num2str(default_data(i,j));
		end
		hd_box(i_box,j_box) = uicontrol(gcf,...
		'style','edit',...
		'string',def_dt,...
		'units','pixels',...
		'position',pos);
		pos(1) = pos(1) + name_col_wd;
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the new data, stop with quit
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
error_data = 1;
while error_data 
	
	flag_str = '';
	go_on = 1;
	while go_on	
		
		uiwait(gcf)
		% uiresulme as I select quit
		
		switch flag_str
		case 'quit'
			go_on = 0;

			i_box = 1;
			for i = 1:nr
				i_box = i_box + 1;
				j_box = 1;
				for j = 1:nc
					j_box = j_box + 1;
					dt = get(hd_box(i_box,j_box),'string');
					if ~isempty(dt)
						data_out(i,j) = str2num(dt);
					end
				end
			end
		case 'cancel'
			data_out = default_data;
			go_on = 0;
		end
	end

	% verifying data if requested
	error_data = 0;
	if nargin == 6		
		for i = 1:nr
			for j = 1:nc
				if data_out(i,j) < limits(i,j,1) | data_out(i,j) > limits(i,j,2)
					hdw = warndlg(['Error in data in cell i = ',num2str(i),', j = ',num2str(j),...
					'. Correct it.'])
					uiwait(hdw)
					error_data = 1;
				end
			end
		end
	end
end

delete(gcf)
figure_data(ind_fig) = [];

