function print_figure(option)
% printing an image, not for uicontrol. OK only for axes

% user is within a figure, in its menu select print, it calls this fct
% user may change parameters in THIS function.

%
%   Version : July 5,03.
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

% Comments
% printdlg(gcf,pos_rbb,pos_fig)    fails
% '-crossplatform': do not use as it uses radio button and make OS9 sick
% read the help of the print function for other choices
% latex is happy ? with -deps  (with c for color)
% when  I omit -noui, matlab blows up if an uicontrol has been used in the figure
% -append forbidden if eps

global pathname2TBMLABvers  screen_size

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the directory where the image goes
% pathname2TBMLABvers is the path up to the folder TBMLAB_x.y
% I created a folder Printed_data in TBMLAB_x.y/User_Data where data are saved
% data are appended
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% I open the directory to save printed figures
cd([pathname2TBMLABvers,filesep,'User_Data',filesep,'Printed_data'])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% setting the parameters for printing gcf
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch computer
case 'PCWIN'
	% do nothing, it is OK. It works fine for Latex, no superfluous axes, background color is OK (white)
	% Latex manages itself to frame the picture, used \epsfig(file = filename, width = .9\linewidth)
	% tune the .9 to get a correct size.
	
case 'MAC2'
	% subaxes selection wrong
	% 	set(gcf,'PaperOrientation','portrait',...
	% 	'PaperUnits','normalized',...
	% 	'PaperPositionMode','manual',...
	% 	'PaperPosition',[0 0 1 1])
 	% ,...
	% 'InvertHardcopy','off')
	% don't use   'PaperType','A4',...   this breaks it, I MUST use default paper type US
	% needed : 'InvertHardcopy','off', else image disappears.
otherwise
	% do nothing, it is OK
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% file name where data saved
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch option
case 'EVN'
	title = {'Plot saved in User_Data/Prined_data. Edit function print_figure line 96 for other options.'};
	quer = {'File name. Max 32 char.','Landscape or portrait','Color or not?','paper type: A4/US',...
	'frame on printed figure (only for full screen): on/off','select part of figure with rubberbox: yes/no'};
	def_ans = {'','portrait','no_color','A4','off','no'};
	rbb = 1;
otherwise
	title = {'Plot saved in User_Data/Prined_data. Edit function print_figure line 96 for other options.'};
	quer = {'File name. Max 32 char.','Landscape or portrait','Color or not?','paper type: A4/US',...
	'frame on printed figure (only for full screen): on/off'};
	def_ans = {'','portrait','no_color','A4','off'};
	rbb = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the answers
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[answer,choice] = dialog_queries(gcf,title,quer,def_ans,[],[],1,[],[],[],[]);

if choice == 2
	return
end

if isempty(answer{1})
	file_name = 'your_file';
else
	file_name = answer{1};
end
nchar = min(32,length(file_name)); % file name not too long
file_name = file_name(1:nchar);
% produce a string

if strcmp(answer{2},'portrait')
	set(gcf,'paperorientation','portrait');
else
	set(gcf,'paperorientation','landscape');
end

if strcmp(answer{3},'no_color')
	dep = 'deps2';
	color = 0;
else
	dep = 'depsc2';
	color = 1;
end

if strcmp(answer{4},'A4')
	set(gcf,'papertype','A4'); % I force usletter
else
	set(gcf,'papertype','A'); % I force usletter
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% fixing units
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% memorizing past ones
units_mem 	= get(gcf,'units');
posit_mem 	= get(gcf,'position');
paper_units = get(gcf,'PaperUnits');
paper_pos 	= get(gcf,'PaperPosition');
gca_mem = gca;

% get new ones
unit_pr = 'points'; % points
set(gcf,'Units',unit_pr,'PaperUnits',unit_pr);
pos_fig = get(gcf,'position');  % in now units

% clearing all colors
if color == 0
	color_mem.hd(1) = gcf;
	color_mem.data{1} = get(gcf,'color');
	color_mem = figure_colors('none',color_mem);
end

if rbb 
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% The ENV case, rubber box allowed
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	if strcmp(answer{6},'yes') % I ask for rrbox
		% the trick is that data are not censured at lower left with PaperPosition
		% hence I shift it with axes
		
		[out_choice,pos_rbb] = select_rectangle; % units alreday fixed
		
		if out_choice == 3
			% restore gcf as it was when entering this function
			set(gcf,'Units',units_mem,...
			'paperorientation','portrait',...
			'papertype','A',...
			'PaperUnits',paper_units);
			return
		end
	
		pos_axes = get(gca,'position');
		Xlim_mem = get(gca,'Xlim');
		Ylim_mem = get(gca,'Ylim');
		
		dx = (pos_rbb(1)/pos_fig(3))*(Xlim_mem(2)-Xlim_mem(1));
		dy = (pos_rbb(2)/pos_fig(4))*(Ylim_mem(2)-Ylim_mem(1));
		
		set(gca,'Xlim',Xlim_mem + [dx dx], 'Ylim',Ylim_mem + [dy dy]); 
		% shifting lower left corner
		
		pos_sel_fig = [0 0 pos_rbb(3) pos_rbb(4)];
		set(gcf,'PaperPosition',pos_sel_fig);
		
		if strcmp(answer{5},'on')
			set(gca,'box','on') % when off, no frame on printed picture
		else
			set(gca,'visible','off')
		end
	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% PRINT
		eval(['print -',dep,'  -noui ',file_name])
		% beware of space between noui and the file name
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% rebuild the figure by shifting origin for lower left and using PaperPosition to cut upper right
		set(gca,'Xlim',Xlim_mem, 'Ylim',Ylim_mem); 			
		
	else % no rubberbox asked for
		set(gcf,'PaperPosition',pos_fig);
		
		if strcmp(answer{5},'on')
			set(gca,'box','on') % when off, no frame on printed picture
		else
			set(gca,'visible','off')
		end
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% PRINT
		eval(['print -',dep,'  -noui ',file_name])
		% beware of space between noui and the file name
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%		
	end

else
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% other case, no rubber box allowed
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	set(gcf,'PaperPosition',pos_fig);
	
	if strcmp(answer{5},'on')
		set(gca,'box','on') % put a box around figure
	else
		set(gca,'visible','off')  % when off, no frame on printed picture
	end
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% PRINT
	eval(['print -',dep,'  -noui ',file_name])
	% beware of space between noui and the file name
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ending print operations
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% getting back all colors
if color == 0
	color_mem = figure_colors('back',color_mem);
end
% restore gcf as it was when entering this function
set(gcf,'Units',units_mem,...
'position',posit_mem,...
'PaperUnits',paper_units,...
'paperorientation','portrait',...
'papertype','A',...
'PaperPosition',paper_pos);
% when frame on, one must put the real image in front with ???

if strcmp(answer{5},'on')
	set(gca_mem,'box','off')
else
	set(gca_mem,'visible','on')
end

% report of action
fprintf(1,'\n\nImage saved in User_Data/Printed_data/%s \n\n',file_name)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  color_mem = figure_colors(option,color_mem)

switch option
case 'none'
	hd = findall(gcf);
	k = 0;
	for i = 1:length(hd)
		switch get(hd(i),'type')
		case 'figure'
			k=k+1;
			color_mem.hd(k) = hd(i);
			color_mem.data{k} = get(hd(i),'color');
			set(hd(i),'color','w')
		case 'axes'
			k=k+1;
			color_mem.hd(k) = hd(i);
			color_mem.data{k} = get(hd(i),'color');
			set(hd(i),'color','none')
		case 'patch'
			k=k+1;
			color_mem.hd(k) = hd(i);
			color_mem.data{k} = get(hd(i),'facecolor');
			set(hd(i),'FaceColor',[1 1 1])
		end
	end

case 'back'
	for i = 1:length(color_mem.hd)
		switch get(color_mem.hd(i),'type')
		case {'figure','axes'}
			set(color_mem.hd(i),'color',color_mem.data{i})
		case 'patch'
			set(color_mem.hd(i),'facecolor',color_mem.data{i})
		end
	end
end


